/* $Id$ */
/*============================================================================*
 |                 Copyright (c) 2018 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK

CREATE OR REPLACE PACKAGE BODY xxas_cal_rslt_cache_pkg AS 

--
-- Private Global Types and Constants
--
g_PACKAGE_NAME CONSTANT varchar2(30) := 'xxas_cal_rslt_cache_pkg';

-- ------------------------------------------------------------------------ *
-- Name    : is_otl_absence_element
-- ------------------------------------------------------------------------ *
FUNCTION is_otl_absence_element
  (p_element_type_id IN number) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN

  $ELSE
    RESULT_CACHE RELIES_ON (hxc_absence_type_elements,hxc_alias_values)
  $END

IS

  CURSOR csr_is_absence_ele_exists IS
  SELECT 'Y'
  FROM   hxc_absence_type_elements hate
        ,hxc_alias_values hav
  WHERE  hate.element_type_id          = p_element_type_id
  AND    to_char(hate.element_type_id) = hav.attribute1;

  l_absence_element_flag varchar2(1);

BEGIN

  IF p_element_type_id is not null THEN
    OPEN  csr_is_absence_ele_exists;
    FETCH csr_is_absence_ele_exists INTO l_absence_element_flag;
    CLOSE csr_is_absence_ele_exists;
  ELSE
    l_absence_element_flag := 'N';
  END IF;

  RETURN nvl(l_absence_element_flag,'N');

END is_otl_absence_element;

-- ------------------------------------------------------------------------ *
-- Name    : tcm_periods_source_block
-- ------------------------------------------------------------------------ *
FUNCTION tcm_periods_source_block RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN

  $ELSE
    RESULT_CACHE RELIES_ON (fnd_form_functions,fnd_form_functions_tl,fnd_menus,fnd_menus_tl,fnd_menu_entries,fnd_menu_entries_tl,fnd_objects,xxas_object_values)
  $END

IS

  CURSOR csr_tcm_periods_source_block IS
  SELECT replace(xbv.function_name,'_','$') source_block
  FROM   xxas_blocks_v xbv
        ,xxas_pages_v  xpv
  WHERE  xpv.slug = 'time-and-absence'
  AND    xpv.menu_id    = xbv.menu_id
  AND    xbv.block_type = 'list'
  AND    xxas_com_sec_pkg.get_parameter_value(xbv.function_parameters,'pLayoutType') = 'calendar'
  AND    xpv.published_flag = 'Y';

  c_method CONSTANT varchar2(30) := 'tcm_periods_source_block';
  l_source_block    varchar2(2000);

BEGIN

  OPEN  csr_tcm_periods_source_block;
  FETCH csr_tcm_periods_source_block INTO l_source_block;
  CLOSE csr_tcm_periods_source_block;

  RETURN l_source_block;

END tcm_periods_source_block;

-- ------------------------------------------------------------------------ *
-- Name    : team_absence_function_id
-- ------------------------------------------------------------------------ *
FUNCTION team_absence_function_id RETURN number
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN

  $ELSE
    RESULT_CACHE RELIES_ON (fnd_form_functions,fnd_form_functions_tl,fnd_menus,fnd_menus_tl,fnd_menu_entries,fnd_menu_entries_tl,fnd_objects,xxas_object_values)
  $END

IS

  CURSOR csr_block_function IS 
  SELECT l.function_id 
  FROM   xxas_pages_v p
        ,xxas_block_list_v l 
  WHERE  p.slug like 'team-absence'
  AND    l.menu_id = p.menu_id
  AND    lower(l.prompt) like '%calendar%'
  AND    p.published_flag = 'Y';

  c_method CONSTANT varchar2(30) := 'team_absence_function_id';
  l_function_id     number;

BEGIN

  OPEN  csr_block_function;
  FETCH csr_block_function INTO l_function_id;
  CLOSE csr_block_function;

  RETURN l_function_id;

END team_absence_function_id;

-- ------------------------------------------------------------------------ *
-- Name    : calendar_function_code
-- ------------------------------------------------------------------------ *
FUNCTION calendar_function_code
  (p_slug IN varchar2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN

  $ELSE
    RESULT_CACHE RELIES_ON (fnd_form_functions,fnd_form_functions_tl,fnd_menus,fnd_menus_tl,fnd_menu_entries,fnd_menu_entries_tl,fnd_objects,xxas_object_values)
  $END

IS

  CURSOR csr_block_function IS 
  SELECT l.function_name
  FROM   xxas_pages_v p
        ,xxas_block_list_v l 
  WHERE  p.slug = p_slug
  AND    l.menu_id = p.menu_id
  AND    lower(l.prompt) like '%calendar%'
  AND    p.published_flag = 'Y';

  c_method CONSTANT varchar2(30) := 'calendar_function_code';
  l_function_code   fnd_form_functions.function_name%TYPE;

BEGIN

  IF p_slug is not null THEN
    OPEN  csr_block_function;
    FETCH csr_block_function INTO l_function_code;
    CLOSE csr_block_function;
  END IF;

  RETURN l_function_code;

END calendar_function_code;

END xxas_cal_rslt_cache_pkg;
/
COMMIT;
EXIT;
