/* $Id$ */
/*============================================================================*
 |                 Copyright (c) 2014 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK

CREATE OR REPLACE PACKAGE BODY xxas_com_sr_methods_pkg AS 

--
-- ------------------------------------------------------------------------ *
-- Name    : get_rest_methods
-- ------------------------------------------------------------------------ *
FUNCTION get_rest_methods RETURN g_methods_t
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (xxas_com_service_repository)
  $END
IS

  l_rest_methods g_methods_t;
  i              number := 1;

  --
  -- Gets REST methods
  --
  CURSOR csr_rest_methods IS
  SELECT    r.service_name
           ,m.method_name
           ,m.method
           ,xxas_com_rest_util_pkg.remove_fwd_trailing_slash
             (extractvalue
               (r.seeded_descriptor, '/ServiceDescriptor/@path'))
             ||
             decode
               (m.path, null, null
               ,'/' || xxas_com_rest_util_pkg.remove_fwd_trailing_slash(m.path)) path_template
           ,nvl(m.consumes, extractvalue(r.seeded_descriptor, '/ServiceDescriptor/@consumes')) consumes
           ,nvl(m.produces, extractvalue(r.seeded_descriptor, '/ServiceDescriptor/@produces')) produces
  FROM      xxas_com_service_repository r
           ,XMLTable
             ('//Method' PASSING r.seeded_descriptor
                         COLUMNS method_name varchar2(400) PATH '@name'
                                ,method      varchar2(400) PATH '@methods'
                                ,path        varchar2(400) PATH '@path'
                                ,consumes    varchar2(4000) PATH '@consumes'
                                ,produces    varchar2(4000) PATH '@produces') m
  ORDER BY r.service_name, m.method_name;

BEGIN

  --
  -- Loop through the services and add to the PL/SQL table
  --
  FOR csr_rec IN csr_rest_methods LOOP
    l_rest_methods(i).service_name := csr_rec.service_name;
    l_rest_methods(i).method_name := csr_rec.method_name;
    l_rest_methods(i).method := csr_rec.method;
    l_rest_methods(i).path_template := csr_rec.path_template;
    l_rest_methods(i).path_regexp
      := xxas_com_rest_util_pkg.uri_path_template_to_regexp
           (csr_rec.path_template);
    l_rest_methods(i).consumes := csr_rec.consumes;
    l_rest_methods(i).produces := csr_rec.produces;
    i := i + 1;
  END LOOP;
  
  RETURN l_rest_methods;

END get_rest_methods;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_method_details
-- ------------------------------------------------------------------------ *
FUNCTION get_method_details
  (p_service_name IN varchar2
  ,p_method_name  IN varchar2) RETURN g_method_details_r
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (xxas_com_service_repository)
  $END
IS

  l_details g_method_details_r;
  l_xml     XMLType;

  --
  -- Gets the method details
  --
  CURSOR    csr_details
  IS
  SELECT    r.service_name
           ,m.method_name
           ,m.description
           ,m.methods
           ,xxas_com_rest_util_pkg.remove_fwd_trailing_slash
             (extractvalue
               (r.descriptor, '/ServiceDescriptor/@path'))
             ||
             decode
               (m.path, null, null
               ,'/' || xxas_com_rest_util_pkg.remove_fwd_trailing_slash(m.path)) path_template
           ,nvl(m.consumes, extractvalue(r.descriptor, '/ServiceDescriptor/@consumes')) consumes
           ,nvl(m.produces, extractvalue(r.descriptor, '/ServiceDescriptor/@produces')) produces
           ,m.provider_type
           ,nvl(m.remove_root_ele, 'false')
           ,m.bipub_dd_app
           ,m.bipub_dd_code
           ,m.method_xml
  FROM      (SELECT service_name
                   ,nvl(custom_descriptor, seeded_descriptor) descriptor
             FROM   xxas_com_service_repository) r
           ,XMLTable
             ('//Method' PASSING r.descriptor
                         COLUMNS method_xml         XMLType        PATH '.'
                                ,method_name        varchar2(400)  PATH '@name'
                                ,description        varchar2(4000) PATH '@description'
                                ,methods            varchar2(400)  PATH '@methods'
                                ,path               varchar2(400)  PATH '@path'
                                ,consumes           varchar2(4000) PATH '@consumes'
                                ,produces           varchar2(4000) PATH '@produces'
                                ,provider_type      varchar2(400)  PATH 'Provider/@type'
                                ,remove_root_ele    varchar2(400)  PATH 'Provider/@removeRootElement'
                                ,bipub_dd_app       varchar2(400)  PATH 'BIPublisher/@app'
                                ,bipub_dd_code      varchar2(400)  PATH 'BIPublisher/@code') m
  WHERE    r.service_name = p_service_name
  AND      m.method_name  = p_method_name;

BEGIN

  OPEN  csr_details;
  FETCH csr_details INTO l_details.service_name
                        ,l_details.method_name
                        ,l_details.description
                        ,l_details.methods
                        ,l_details.path_template
                        ,l_details.consumes
                        ,l_details.produces
                        ,l_details.provider_type
                        ,l_details.remove_root_ele
                        ,l_details.bipub_dd_app
                        ,l_details.bipub_dd_code
                        ,l_xml;
  CLOSE csr_details;
  
  IF l_xml IS NOT NULL THEN
    l_details.provider :=
      dbms_xmlgen.convert
        (l_xml.extract('//Provider/text()').getStringVal()
        ,dbms_xmlgen.entity_decode);
        
    --
    -- Set the provider implementation
    --
    IF l_details.provider_type = 'PLSQL' THEN
      l_details.provider_impl := 'xxas_com_rest_mgr_pkg.run_plsql';
    ELSIF instr(lower(l_details.provider), 'xxas_http_response.') > 0 THEN
      l_details.provider_impl := 'xxas_com_rest_mgr_pkg.run_get_response_sql';      
    ELSE
      l_details.provider_impl := 'xxas_com_rest_mgr_pkg.run_dbms_xmlgen_sql';
    END IF;
  END IF;
  
  IF l_details.provider_impl IS NULL THEN
    l_details.provider_impl := 'none';
  END IF;
  
  RETURN l_details;

END get_method_details;

END xxas_com_sr_methods_pkg;
/
COMMIT;
EXIT;
