/* $Id$ */
/*============================================================================*
 |                 Copyright (c) 2011 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
SET ESCAPE '\'
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK

DECLARE

  --
  -- Local constants and variables
  --
  e_no_bg_id          EXCEPTION;
  e_bg_not_exists     EXCEPTION;
  e_ff_type_not_found EXCEPTION;
  l_BG_ID   CONSTANT  NUMBER        := to_number('&1');
  l_USER_ID CONSTANT  NUMBER        := to_number('&2');
  l_START   CONSTANT  DATE          := to_date('1900-01-01','YYYY-MM-DD');
  l_END     CONSTANT  DATE          := xxas_util_pkg.eot;
  l_FF_NAME CONSTANT  VARCHAR2(80)  := 'XXAS_EMP_ASG_EIT_EDIT';
  l_FF_DESC CONSTANT  VARCHAR2(240) := 'ASG EIT Edit';
  l_FF_TYPE CONSTANT  varchar2(30)  := 'Applaud Assignment';
  l_formula_text      LONG;
  l_formula_type_id   NUMBER;
  l_ff_id             NUMBER;
  l_bg_exists         VARCHAR2(1);

  --
  -- Tests if the Business Group already exists
  --
  CURSOR csr_bg_exists IS
  SELECT 'Y'
  FROM   per_business_groups_perf bg
  WHERE  bg.business_group_id = l_BG_ID;

  --
  -- Gets the FF Id
  --
  CURSOR csr_ff_id IS
  SELECT ff.formula_id
  FROM   ff_formulas_f ff
  WHERE  ff.business_group_id = l_BG_ID
  AND    ff.formula_name = l_FF_NAME;

  --
  -- Gets the Formula Type Id
  --
  CURSOR csr_ff_type IS
  SELECT fft.formula_type_id
  FROM   ff_formula_types fft
  WHERE  fft.formula_type_name = l_FF_TYPE;

BEGIN

  --
  -- Check that the Business Group Id was passed in
  --
  IF l_BG_ID IS NULL THEN
    RAISE e_no_bg_id;
  END IF;

  --
  -- Check the Business Group exists
  --
  OPEN  csr_bg_exists;
  FETCH csr_bg_exists INTO l_bg_exists;
  CLOSE csr_bg_exists;

  IF l_bg_exists IS NULL THEN
    RAISE e_bg_not_exists;
  END IF;

  --
  -- Fetch the Fast Formula Id
  --
  OPEN  csr_ff_id;
  FETCH csr_ff_id INTO l_ff_id;
  CLOSE csr_ff_id;

  --
  -- Get the Formula Type; error if it cannot be found
  --
  OPEN  csr_ff_type;
  FETCH csr_ff_type INTO l_formula_type_id;
  CLOSE csr_ff_type;

  IF l_formula_type_id IS NULL THEN
    RAISE e_ff_type_not_found;
  END IF;

  --
  -- Set the Formula Text
  --
  l_formula_text := '

  /*************************************************************************/
  /*  LOGIC BLOCK                                                          */
  /*                                                                       */
  /*  Change this section to create your field logic within the Edit panel */
  /*                                                                       */
  /*  Set the following Properties:                                        */
  /*  Aei_Information1_Label = ''Label to display''                        */
  /*  Aei_Information1_Enabled = ''true'' or ''false''                     */
  /*  Aei_Information1_Required = ''true'' or ''false''                    */
  /*  Aei_Information1_Visible = ''true'' or ''false''                     */
  /*  Aei_Information1_Value = ''The value of the field''. You can         */
  /*             set this or use the existing value to drive other logic,  */
  /*             e.g. make another field required based on the value in    */
  /*             current field.                                            */
  /*  Aei_Information1_Description = ''Hint or Tip''. This display         */
  /*                underneath the the field to give help to user.         */
  /*                Supports HTML so you can make this a link or format it */
  /*  Aei_Information1_Refresh_On_Change = ''true'' or ''false''. Set this to ''true''   */
  /*                       if there are other fields that have behaviour   */
  /*                           that is dependent on this field.            */
  /*                                                                       */
  /*                                                                       */
  /* Controls for Text fields only:                                        */
  /*                                                                       */
  /*  Aei_Information1_Maxchars = <number> Use for text fields only to limit Chars */
  /*  Aei_Information1_Empty_Value = ''Hint or Tip''. Use this for text fields and   */
  /*                         the text displays in an empty field. It will  */
  /*                         disappear when the user starts typing         */
  /*                                                                       */
  /* Controls for Poplist / Combolist fields only:                         */
  /*                                                                       */
  /* Aei_Information1_List_Provider = ''VALUE_SET_ID:112828'' or           */
  /*                          ''LOOKUP_TYPE:PHONE_TYPE''. This overrides   */
  /*                          any flexfield definition and populates the   */
  /*                          List using your specified Valueset or Lookup */
  /* Note: Any validation against the flexfield will always be run when    */
  /*       the flexfield is saved, so ensure the LIST_PROVIDER gives a     */
  /*       ''subset'' of possible values defined against the master flexfield*/
  /*                                                                       */
  /*************************************************************************/

  date_from_required = ''false''
  date_from_visible  = ''false''
  date_to_visible    = ''false''
  Comments_to_approver_visible = ''false''
  aei_Information30_visible = ''true''



  /*************************************************************************/
  /*  End of BLOCK #1                                                      */
  /*************************************************************************/



  /*************************************************************************/
  /*  RETURN BLOCK                                                         */
  /*  Ensure you pass as outputs any attributes you may have set in the    */
  /*  the LOGIC BLOCK. Order is not important.                             */
  /*************************************************************************/

RETURN

     date_from_required,
     date_from_visible,
     date_to_visible,
     Comments_to_approver_visible,
     Aei_Information30_visible

 ';


  BEGIN

    IF l_ff_id IS NULL THEN
      --
      -- Create the Fast Formula
      --
      INSERT INTO ff_formulas_f
        (formula_id
        ,effective_start_date
        ,effective_end_date
        ,business_group_id
        ,formula_type_id
        ,formula_name
        ,description
        ,formula_text)
      VALUES
        (ff_formulas_s.nextval
        ,l_START
        ,l_END
        ,l_BG_ID
        ,l_formula_type_id
        ,l_FF_NAME
        ,l_FF_DESC
        ,l_formula_text);

    ELSE
      --
      -- Update the existing Fast Formula text
      -- (if it has multiple date-track versions the text of
      -- all versions is updated).
      --
      UPDATE ff_formulas_f
      SET    formula_text = l_formula_text
            ,description  = l_FF_DESC
      WHERE  formula_id = l_ff_id;

    END IF;

    COMMIT;

    EXCEPTION
      WHEN others THEN
        ROLLBACK;
        RAISE;
    END;

    --
    -- Compile the Fast Formula
    --
    xxas_util_pkg.compile_ff
      (p_user_id => l_USER_ID
      ,p_ff_type => l_FF_TYPE
      ,p_ff_name => l_FF_NAME);

EXCEPTION

  WHEN e_no_bg_id THEN
    RAISE no_data_found;

  WHEN e_bg_not_exists THEN
    RAISE no_data_found;

  WHEN e_ff_type_not_found THEN
    RAISE no_data_found;

END;
/
COMMIT;
EXIT;
