/* $Id$ */
/*============================================================================*
 |                 Copyright (c) 2015 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
SET ESCAPE '\'
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK

DECLARE

  --
  -- Local constants and variables
  --
  e_no_bg_id          EXCEPTION;
  e_bg_not_exists     EXCEPTION;
  e_ff_type_not_found EXCEPTION;
  l_BG_ID   CONSTANT  NUMBER        := to_number('&1');
  l_USER_ID CONSTANT  NUMBER        := to_number('&2');
  l_START   CONSTANT  DATE          := to_date('0001-01-01','YYYY-MM-DD');
  l_END     CONSTANT  DATE          := xxas_util_pkg.eot;
  l_FF_NAME CONSTANT  VARCHAR2(80)  := 'XXAS_EMP_VAC_EDIT';
  l_FF_DESC CONSTANT  VARCHAR2(240) := 'Applaud Vacancy Edit';
  l_FF_TYPE CONSTANT  varchar2(30)  := 'Applaud Person';
  l_formula_text      LONG;
  l_formula_type_id   NUMBER;
  l_ff_id             NUMBER;
  l_bg_exists         VARCHAR2(1);

  --
  -- Tests if the Business Group already exists
  --
  CURSOR csr_bg_exists IS
  SELECT 'Y'
  FROM   per_business_groups_perf bg
  WHERE  bg.business_group_id = l_BG_ID;

  --
  -- Gets the FF Id
  --
  CURSOR csr_ff_id IS
  SELECT ff.formula_id
  FROM   ff_formulas_f ff
  WHERE  ff.business_group_id = l_BG_ID
  AND    ff.formula_name = l_FF_NAME;

  --
  -- Gets the Formula Type Id
  --
  CURSOR csr_ff_type IS
  SELECT fft.formula_type_id
  FROM   ff_formula_types fft
  WHERE  fft.formula_type_name = l_FF_TYPE;

BEGIN

  --
  -- Check that the Business Group Id was passed in
  --
  IF l_BG_ID IS NULL THEN
    RAISE e_no_bg_id;
  END IF;

  --
  -- Check the Business Group exists
  --
  OPEN  csr_bg_exists;
  FETCH csr_bg_exists INTO l_bg_exists;
  CLOSE csr_bg_exists;

  IF l_bg_exists IS NULL THEN
    RAISE e_bg_not_exists;
  END IF;

  --
  -- Fetch the Fast Formula Id
  --
  OPEN  csr_ff_id;
  FETCH csr_ff_id INTO l_ff_id;
  CLOSE csr_ff_id;

  --
  -- Get the Formula Type; error if it cannot be found
  --
  OPEN  csr_ff_type;
  FETCH csr_ff_type INTO l_formula_type_id;
  CLOSE csr_ff_type;

  IF l_formula_type_id IS NULL THEN
    RAISE e_ff_type_not_found;
  END IF;

  --
  -- Set the Formula Text
  --
  l_formula_text := q'[
  
/*==============================================================================

Formula Name: XXAS_EMP_VAC_EDIT
Formula Type: Applaud Person
Description:  Applaud Vacancy Edit

==============================================================================*/

  Default for Refresh_Field_Name                  is 'Unknown'
  Default for Refresh_Event                       is 'Unknown'
  Default for Vacancy_Name_Value_in               is 'Unknown'
  Default for Budget_Value_Value_in               is 0
  Default for Date_To_Value_in                    is '0001/01/01 00:00:00' (date)
  Default for Position_Id_Value_in                is 0
  Default for Location_Id_Value_in                is 0
  Default for Grade_Id_Value_in                   is 0
  Default for Organization_Id_Value_in            is 0
  Default for Job_Id_Value_in                     is 0  
  Default for Min_Salary_Value_in                 is 0
  Default for Employee_Flag_Value_in              is 'Unknown'
  Default for Contractor_Flag_Value_in            is 'Unknown'

  Inputs are Refresh_Field_Name
            ,Refresh_Event
            ,Vacancy_Name_Value_in
            ,Budget_Value_Value_in
            ,Date_To_Value_in
            ,Position_Id_Value_in
            ,Location_Id_Value_in
            ,Grade_Id_Value_in
			,Organization_Id_Value_in
			,Job_Id_Value_in
            ,Min_Salary_Value_in
            ,Employee_Flag_Value_in
            ,Contractor_Flag_Value_in

  COLOR_RED            = '#CC0000'
  COLOR_DEEP_RED       = '#FF0000'

  DATE_FORMAT_MASK     = 'FMDD Mon YYYY'
  CURRENCY_FORMAT_MASK = 'FM999G999G999G990D00'

If XXAS_IS_VAC_AUTO() = 0 then
Vacancy_Name_enabled = 'false'

If Position_Id_Value_in was not defaulted and Job_Id_Value_in was not defaulted 
	and Refresh_Event = 'ROC' and Refresh_Field_Name = 'GRADE_ID' 
	and XXAS_IS_GRADE_VALID(Grade_Id_Value_in,Position_Id_Value_in) = 'N' then
(  Grade_Id_Description = '<font color="#CC0000">The Grade is not valid for the Position.</font>'
)


    /* Check for the Refresh Event if it is RefreshOnChange, then populate Approvers through PLSQL callback */
      If Refresh_Event = 'ROC' and Refresh_Field_Name = 'POSITION_ID' Then
  (
/* Callback */
      Callback = 'xxas_vacancy_pkg.vacancy_change_cb'
      Callback_Event = 'vacancy_change_cb'
	  
    /* Check for the Refresh Event if it is Review, then populate Approvers through PLSQL callback */
  
/* Defaults */
Organization_Id_Value = Organization_Id_Value_in
Grade_Id_Value = Grade_Id_Value_in
Job_Id_Value = Job_Id_Value_in
Location_Id_Value = Location_Id_Value_in

l_result = XXAS_REC_DEFAULT_POS (Position_Id_Value_in, Organization_Id_Value, Location_Id_Value, Job_Id_Value, Grade_Id_Value)
	  
  )

  If Refresh_Event = 'REVIEW' Then
  (
      Callback = 'xxas_vacancy_pkg.populate_approvers_cb'
      Callback_Event = 'POPULATE_VAC_OFF_APPROVERS'
  )	


Return  Vacancy_Name_enabled,
Grade_Id_Description,
Organization_Id_Value
      ,Grade_Id_Value
      ,Job_Id_Value
      ,Location_Id_Value
/*

      ,Employment_Category_Code_Value
      ,Employee_Flag_Value
      ,Contractor_Flag_Value
      ,Salary_Period_Code_Value
      ,Salary_Currency_Value
      ,Posting_Name_Value
      ,Posting_job_Title_Value
      ,Posting_Org_Description_Enabled
      ,Posting_Detailed_Description_Enabled
      ,Posting_Additional_Details_Enabled
      ,Image_URL_Visible
      ,Display_Recruiter_Info_Visible
      ,Comments_To_Approver_Visible
	  */
      ,Callback
      ,Callback_Event
]';


  BEGIN

    IF l_ff_id IS NULL THEN
      --
      -- Create the Fast Formula
      --
      INSERT INTO ff_formulas_f
        (formula_id
        ,effective_start_date
        ,effective_end_date
        ,business_group_id
        ,formula_type_id
        ,formula_name
        ,description
        ,formula_text)
      VALUES
        (ff_formulas_s.nextval
        ,l_START
        ,l_END
        ,l_BG_ID
        ,l_formula_type_id
        ,l_FF_NAME
        ,l_FF_DESC
        ,l_formula_text);

    ELSE
      --
      -- Update the existing Fast Formula text
      -- (if it has multiple date-track versions the text of
      -- all versions is updated).
      --
      UPDATE ff_formulas_f
      SET    formula_text = l_formula_text
            ,description  = l_FF_DESC
      WHERE  formula_id = l_ff_id;

    END IF;

    COMMIT;

    EXCEPTION
      WHEN others THEN
        ROLLBACK;
        RAISE;
    END;

    --
    -- Compile the Fast Formula
    --
    xxas_util_pkg.compile_ff
      (p_user_id => l_USER_ID
      ,p_ff_type => l_FF_TYPE
      ,p_ff_name => l_FF_NAME);

EXCEPTION

  WHEN e_no_bg_id THEN
    RAISE no_data_found;

  WHEN e_bg_not_exists THEN
    RAISE no_data_found;

  WHEN e_ff_type_not_found THEN
    RAISE no_data_found;

END;
/
COMMIT;
EXIT;
