/* $Id$ */
/*============================================================================*
 |                 Copyright (c) 2016 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK

CREATE OR REPLACE PACKAGE BODY xxas_menu_rslt_cache_pkg AS 

--
-- Private Global Types and Constants
--
g_PACKAGE_NAME CONSTANT varchar2(30) := 'xxas_menu_rslt_cache_pkg';

-- ------------------------------------------------------------------------ *
-- Name    : get_xxas_menu
-- ------------------------------------------------------------------------ *
FUNCTION get_xxas_menu
  (p_menu_id IN number) RETURN xxas_menus_v%ROWTYPE
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_menus, fnd_menus_tl, xxas_object_values)
  $END
IS

  l_return xxas_menus_v%ROWTYPE;

  CURSOR csr_menu IS
  SELECT m.*
  FROM   xxas_menus_v m
  WHERE  m.menu_id = p_menu_id;

BEGIN

  --
  -- Check for null parameters
  --
  IF p_menu_id IS NOT NULL THEN
    OPEN  csr_menu;
    FETCH csr_menu INTO l_return;
    CLOSE csr_menu;

  END IF;

  RETURN l_return;

END get_xxas_menu;

--
-- ------------------------------------------------------------------------ *
-- Name    : menu_grants_count
-- ------------------------------------------------------------------------ *
FUNCTION menu_grants_count
  (p_menu_name IN varchar2) RETURN number
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_menus, fnd_grants)
  $END
IS

  CURSOR csr_grants_count IS
  SELECT count(fg.grant_guid)
  FROM   fnd_grants fg
        ,fnd_menus fm
  WHERE  fm.menu_name = p_menu_name
  AND    fm.menu_id   = fg.menu_id
  AND    trunc(sysdate) BETWEEN fg.start_date 
                        AND     nvl(fg.end_date,trunc(sysdate));

  l_return number;

BEGIN

  --
  -- Check for null parameters
  --
  IF p_menu_name IS NOT NULL THEN

    OPEN  csr_grants_count;
    FETCH csr_grants_count INTO l_return;
    CLOSE csr_grants_count;

  END IF;

  RETURN l_return;

END menu_grants_count;

END xxas_menu_rslt_cache_pkg;
/
COMMIT;
EXIT;
