/* $Id: xxas_missed_punch_type.pkb 7116 2019-04-18 11:07:10Z ybudamala $ */
/*============================================================================*
 |                 Copyright (c) 2017 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK

CREATE OR REPLACE TYPE BODY xxas_missed_punch IS

  CONSTRUCTOR FUNCTION xxas_missed_punch
    (p_person_id        IN number
    ,p_assignment_id    IN number
    ,p_pending_clock_id IN number) RETURN SELF AS RESULT
  IS

  BEGIN

    self.person_id         :=  p_person_id;
    self.assignment_id     :=  p_assignment_id;
    self.pending_clock_id  :=  p_pending_clock_id;

    RETURN;

  END;

  MEMBER FUNCTION get_xml RETURN XMLType
  IS

    CURSOR csr_tcm_pending_clock_data(cp_pending_clock_id IN number
                                     ,cp_person_id        IN number) IS
    SELECT xtpc.missed_in_flag
          ,xtpc.missed_out_flag
          ,xtpc.pending_details
          ,xtpc.element_type_id
          ,xtpc.reason_for_miss
          ,xtpc.others_reason_detail
          ,xtpc.auto_saved_start_time
          ,xtpc.auto_saved_end_time
    FROM   xxas_tcm_pending_clocks xtpc
    WHERE  xtpc.pending_clock_id = cp_pending_clock_id
    AND    xtpc.person_id        = cp_person_id;

    CURSOR csr_pending_time_clock_data(cp_pending_details IN XMLType)
    IS
    SELECT fnd_date.canonical_to_date(replace(pend.in_date_time,'"')) in_date_time
          ,fnd_date.canonical_to_date(replace(pend.out_date_time,'"')) out_date_time
          ,fnd_date.canonical_to_date(replace(pend.lunchin_date_time,'"')) lunchin_date_time
          ,fnd_date.canonical_to_date(replace(pend.lunchout_date_time,'"')) lunchout_date_time
          ,replace(pend.pend_missed_in_flag,'"') pend_missed_in_flag
          ,replace(pend.pend_missed_out_flag,'"') pend_missed_out_flag
          ,replace(pend.pend_lunch_missed_in_flag,'"') pend_lunch_missed_in_flag
          ,replace(pend.pend_lunch_missed_out_flag,'"') pend_lunch_missed_out_flag
          ,replace(pend.pend_missed_in_reason,'"') pend_missed_in_reason
          ,replace(pend.pend_missed_out_reason,'"') pend_missed_out_reason
          ,replace(pend.pend_lunch_missed_in_reason,'"') pend_lunch_missed_in_reason
          ,replace(pend.pend_lunch_missed_out_reason,'"') pend_lunch_missed_out_reason
          ,replace(pend.pend_missed_in_detail,'"') pend_missed_in_detail
          ,replace(pend.pend_missed_out_detail,'"') pend_missed_out_detail
          ,replace(pend.pend_lunch_missed_in_detail,'"') pend_lunch_missed_in_detail
          ,replace(pend.pend_lunch_missed_out_detail,'"') pend_lunch_missed_out_detail
    FROM   XMLTable
            ('//PendingDetails/TimeClock'
            PASSING  cp_pending_details
            COLUMNS  xml                          XMLType          path '.'
                    ,in_date_time                 varchar2(30)     path '//In/ServerDateTime'
                    ,out_date_time                varchar2(30)     path '//Out/ServerDateTime'
                    ,lunchin_date_time            varchar2(30)     path '//LunchIn/ServerDateTime'
                    ,lunchout_date_time           varchar2(30)     path '//LunchOut/ServerDateTime'
                    ,pend_missed_in_flag          varchar2(10)     path '//In/MissedInFlag'
                    ,pend_missed_out_flag         varchar2(10)     path '//Out/MissedOutFlag'
                    ,pend_lunch_missed_in_flag    varchar2(10)     path '//LunchIn/MissedInFlag'
                    ,pend_lunch_missed_out_flag   varchar2(10)     path '//LunchOut/MissedOutFlag'
                    ,pend_missed_in_reason        varchar2(100)    path '//In/ReasonForMiss'
                    ,pend_missed_out_reason       varchar2(100)    path '//Out/ReasonForMiss'
                    ,pend_lunch_missed_in_reason  varchar2(100)    path '//LunchIn/ReasonForMiss'
                    ,pend_lunch_missed_out_reason varchar2(100)    path '//LunchOut/ReasonForMiss'
                    ,pend_missed_in_detail        varchar2(4000)   path '//In/OtherMoreDetail'
                    ,pend_missed_out_detail       varchar2(4000)   path '//Out/OtherMoreDetail'
                    ,pend_lunch_missed_in_detail  varchar2(4000)   path '//LunchIn/OtherMoreDetail'
                    ,pend_lunch_missed_out_detail varchar2(4000)   path '//LunchOut/OtherMoreDetail'
            ) pend;

    CURSOR csr_person_data IS
    SELECT XMLElement
            ("EMPLOYEE"
            ,XMLElement
               ("value"
               ,XMLAttributes
                 ('true' as "json:force-array"
                 ,'http://json.org/' as "xmlns:json")
               ,XMLAgg
                 (XMLElement
                   ("value"
                   ,XMLForest
                     (p.person_id "id"
                     ,p.short_name "label"
                     ,xxas_util_pkg.flag_to_boolean('N') "deletable")
                   ,XMLElement
                     ("linked"
                     ,XMLElement
                       ("image"
                       ,XMLForest
                         (xxas_com_rest_util_pkg.uri_from_path
                            ('people/' || p.img_person_id || '/photo') "self"
                         ,p.initials "initials"
                         ,xxas_per_image_pkg.url
                            (p.img_person_id, p.img_last_update_date, 'SMALL') "small"
                         )
                       )
                     )
                   )
                 )
               ).getClobVal()
            )
    FROM  (SELECT papf.person_id
                 ,xxas_util_pkg.get_formatted_name
                    (papf.full_name
                    ,papf.global_name
                    ,papf.local_name
                    ,papf.first_name
                    ,papf.last_name
                    ,papf.known_as) short_name
                 ,xxas_util_pkg.get_initials(papf.first_name, papf.last_name, papf.known_as) initials
                 ,img.person_id              img_person_id
                 ,img.last_update_date       img_last_update_date
           FROM   per_all_people_f papf
                 ,xxas_per_thumbnails img
           WHERE  papf.person_id = self.person_id
           AND    trunc(sysdate) BETWEEN papf.effective_start_date 
                                 AND     papf.effective_end_date
           AND    papf.person_id = img.person_id(+)) p;

    CURSOR csr_missed_punch_xml(cp_start_date            IN date
                               ,cp_payment_type_id       IN number
                               ,cp_time_range            IN varchar2
                               ,cp_last_punch_status     IN varchar2
                               ,cp_time_prod             IN varchar2
                               ,cp_reason_for_miss       IN varchar2
                               ,cp_others_reason_detail  IN varchar2
                               ,cp_current_approver_role IN varchar2
                               ,cp_person_data           IN XMLType) IS
    SELECT XMLElement
            ("ROW"
            ,XMLForest
              (self.person_id           as person_id
              ,self.assignment_id       as assignment_id
              ,self.pending_clock_id    as pending_clock_id
              ,fnd_global.employee_id   as logged_person_id
              ,self.person_id           as employee
              ,cp_start_date            as start_date
              ,cp_payment_type_id       as payment_type_id
              ,cp_time_range            as time_range
              ,self.pending_clock_id    as approvers
              ,cp_last_punch_status     as current_state_code
              ,cp_time_prod             as time_prod
              ,cp_reason_for_miss       as reason_for_miss
              ,cp_others_reason_detail  as other_more_detail
              ,cp_current_approver_role as current_approver_role
              )
            ,cp_person_data
            )
    FROM   dual;

  CURSOR csr_pending_rec_det(cp_pending_clock_id IN number)
  IS
  SELECT xtpcav.current_state_code
        ,xtpcav.current_approver_role
        ,xtpcav.reason_for_miss
        ,xtpcav.others_reason_detail
        ,xtpcav.element_type_id
        ,xtpcav.start_date_time
  FROM   xxas_tcm_pend_clk_approvers_v xtpcav
  WHERE  xtpcav.pending_clock_id       = cp_pending_clock_id;

  l_start_date                   date;
  l_time_range                   varchar2(100);
  l_missed_in_flag               xxas_tcm_pending_clocks.missed_in_flag%TYPE;
  l_missed_out_flag              xxas_tcm_pending_clocks.missed_out_flag%TYPE;
  l_pending_details              xxas_tcm_pending_clocks.pending_details%TYPE;
  l_payment_type_id              xxas_tcm_pending_clocks.element_type_id%TYPE;
  l_reason_for_miss              xxas_tcm_pending_clocks.reason_for_miss%TYPE;
  l_others_reason_detail         xxas_tcm_pending_clocks.others_reason_detail%TYPE;
  l_auto_saved_start_time        xxas_tcm_pending_clocks.auto_saved_start_time%TYPE;
  l_auto_saved_end_time          xxas_tcm_pending_clocks.auto_saved_end_time%TYPE;
  l_in_date_time                 date;
  l_out_date_time                date;
  l_lunchin_date_time            date;
  l_lunchout_date_time           date;
  l_last_punch_type              varchar2(10);
  l_last_punch_date              date;
  l_last_punch_datetime_in       date;
  l_last_punch_datetime_out      date;
  l_xml                          XMLType;
  l_time_prod                    varchar2(100);
  l_last_punch_status            varchar2(100);
  l_last_punch_missed_out_flag   varchar2(1) := 'Y';
  l_last_punch_log_entry_id      number;
  l_current_approver_role        fnd_user.user_name%TYPE;
  l_pend_missed_in_flag          xxas_tcm_pending_clocks.missed_in_flag%TYPE;
  l_pend_missed_out_flag         xxas_tcm_pending_clocks.missed_out_flag%TYPE;
  l_pend_lunch_missed_in_flag    xxas_tcm_pending_clocks.missed_in_flag%TYPE;
  l_pend_lunch_missed_out_flag   xxas_tcm_pending_clocks.missed_out_flag%TYPE;
  l_pend_missed_in_reason        varchar2(100);
  l_pend_missed_out_reason       varchar2(100);
  l_pend_lunch_missed_in_reason  varchar2(100);
  l_pend_lunch_missed_out_reason varchar2(100);
  l_pend_missed_in_detail        varchar2(4000);
  l_pend_missed_out_detail       varchar2(4000);
  l_pend_lunch_missed_in_detail  varchar2(4000);
  l_pend_lunch_missed_out_detail varchar2(4000);
  l_reason_for_miss_apr          varchar2(4000);
  l_others_reason_detail_apr     varchar2(4000);
  l_lunch_type_flag              boolean;
  l_person_data                  XMLType;

  BEGIN

    l_time_prod := fnd_profile.value('XXAS_TIMESTORE');

    IF self.person_id        is not null AND 
       self.assignment_id    is not null AND 
       self.pending_clock_id is not null THEN

      /*l_date_time_range  := to_char(self.start_date_time,'YYYY-MM-DD HH24:MI') || '_' || to_char(self.end_date_time,'YYYY-MM-DD HH24:MI');
      l_date_range       := to_char(self.start_date_time,'YYYY-MM-DD') || '_' || to_char(self.end_date_time,'YYYY-MM-DD');*/

      OPEN  csr_tcm_pending_clock_data(cp_pending_clock_id => self.pending_clock_id
                                      ,cp_person_id        => self.person_id);
      FETCH csr_tcm_pending_clock_data INTO l_missed_in_flag
                                           ,l_missed_out_flag
                                           ,l_pending_details
                                           ,l_payment_type_id
                                           ,l_reason_for_miss
                                           ,l_others_reason_detail
                                           ,l_auto_saved_start_time
                                           ,l_auto_saved_end_time;
      CLOSE csr_tcm_pending_clock_data;

      OPEN  csr_pending_time_clock_data(cp_pending_details => l_pending_details);
      FETCH csr_pending_time_clock_data INTO l_in_date_time
                                            ,l_out_date_time
                                            ,l_lunchin_date_time
                                            ,l_lunchout_date_time
                                            ,l_pend_missed_in_flag
                                            ,l_pend_missed_out_flag
                                            ,l_pend_lunch_missed_in_flag
                                            ,l_pend_lunch_missed_out_flag
                                            ,l_pend_missed_in_reason
                                            ,l_pend_missed_out_reason
                                            ,l_pend_lunch_missed_in_reason
                                            ,l_pend_lunch_missed_out_reason
                                            ,l_pend_missed_in_detail
                                            ,l_pend_missed_out_detail
                                            ,l_pend_lunch_missed_in_detail
                                            ,l_pend_lunch_missed_out_detail;
      CLOSE csr_pending_time_clock_data;

      IF l_in_date_time is not null OR l_out_date_time is not null THEN

        l_lunch_type_flag := false;

        IF l_out_date_time is not null AND l_in_date_time is not null THEN

          l_last_punch_type         := 'OUT';
          l_last_punch_date         := trunc(l_out_date_time);
          l_last_punch_datetime_in  := l_in_date_time;
          l_last_punch_datetime_out := l_out_date_time;

        ELSIF l_in_date_time is not null AND l_out_date_time is null THEN

          l_last_punch_type         := 'IN';
          l_last_punch_date         := trunc(l_in_date_time);
          l_last_punch_datetime_in  := l_in_date_time;
          l_last_punch_datetime_out := null;

        ELSIF l_in_date_time is null AND l_out_date_time is not null THEN

          l_last_punch_type         := 'OUT';
          l_last_punch_date         := trunc(l_out_date_time);
          l_last_punch_datetime_in  := null;
          l_last_punch_datetime_out := l_out_date_time;

        END IF;

      ELSIF l_lunchin_date_time is not null OR l_lunchout_date_time is not null THEN

        l_lunch_type_flag := true;

        IF l_lunchout_date_time is not null AND l_lunchin_date_time is not null THEN

          l_last_punch_type         := 'OUT';
          l_last_punch_date         := trunc(l_lunchout_date_time);
          l_last_punch_datetime_in  := l_lunchin_date_time;
          l_last_punch_datetime_out := l_lunchout_date_time;

        ELSIF l_lunchin_date_time is not null AND l_lunchout_date_time is null THEN

          l_last_punch_type         := 'IN';
          l_last_punch_date         := trunc(l_lunchin_date_time);
          l_last_punch_datetime_in  := l_lunchin_date_time;
          l_last_punch_datetime_out := null;

        ELSIF l_lunchin_date_time is null AND l_lunchout_date_time is not null THEN

          l_last_punch_type         := 'OUT';
          l_last_punch_date         := trunc(l_lunchout_date_time);
          l_last_punch_datetime_in  := null;
          l_last_punch_datetime_out := l_lunchout_date_time;

        END IF;

      END IF;


      l_start_date := l_last_punch_date;

      IF nvl(l_missed_in_flag,'N') = 'Y' AND 
         l_last_punch_type         = 'IN' THEN

        l_time_range := null;

      ELSIF nvl(l_missed_in_flag,'N')  = 'Y' AND
            nvl(l_missed_out_flag,'N') = 'N' AND
            l_last_punch_type          = 'OUT' THEN

        l_time_range := l_auto_saved_start_time || '_' || nvl(l_auto_saved_end_time,to_char(l_last_punch_datetime_out,'HH24:MI'));

      ELSIF nvl(l_missed_in_flag,'N')  = 'N' AND 
            nvl(l_missed_out_flag,'N') = 'Y' AND 
            l_last_punch_type          = 'OUT' THEN

        l_time_range := nvl(l_auto_saved_start_time,to_char(l_last_punch_datetime_in,'HH24:MI')) || '_' || l_auto_saved_end_time;

      ELSIF nvl(l_missed_in_flag,'N')  = 'Y' AND 
            nvl(l_missed_out_flag,'N') = 'Y' AND 
            l_last_punch_type          = 'OUT' THEN

        l_time_range := l_auto_saved_start_time || '_' || l_auto_saved_end_time;

      ELSIF nvl(l_missed_in_flag,'N')  = 'N' AND 
            nvl(l_missed_out_flag,'N') = 'N' AND 
            l_last_punch_type          = 'OUT' THEN

        l_time_range := nvl(l_auto_saved_start_time,to_char(l_last_punch_datetime_in,'HH24:MI')) || '_' || 
                        nvl(l_auto_saved_end_time,to_char(l_last_punch_datetime_out,'HH24:MI'));

        IF (nvl(l_pend_missed_in_flag,'N') = 'Y' AND
            nvl(l_pend_missed_out_flag,'N') = 'Y') OR 
           (nvl(l_pend_lunch_missed_in_flag,'N') = 'Y' AND
            nvl(l_pend_lunch_missed_out_flag,'N') = 'Y') THEN

         IF l_lunch_type_flag THEN

           l_reason_for_miss      := l_pend_lunch_missed_in_reason;
           l_others_reason_detail := l_pend_lunch_missed_in_detail;

         ELSE

           l_reason_for_miss      := l_pend_missed_in_reason;
           l_others_reason_detail := l_pend_missed_in_detail;

         END IF;

        ELSIF (nvl(l_pend_missed_in_flag,'N') = 'Y' AND
               nvl(l_pend_missed_out_flag,'N') = 'N') OR 
              (nvl(l_pend_lunch_missed_in_flag,'N') = 'Y' AND
               nvl(l_pend_lunch_missed_out_flag,'N') = 'N') THEN

          IF l_lunch_type_flag THEN

            l_reason_for_miss      := l_pend_lunch_missed_in_reason;
            l_others_reason_detail := l_pend_lunch_missed_in_detail;

          ELSE

            l_reason_for_miss      := l_pend_missed_in_reason;
            l_others_reason_detail := l_pend_missed_in_detail;

          END IF;

        ELSIF (nvl(l_pend_missed_in_flag,'N') = 'N' AND
               nvl(l_pend_missed_out_flag,'N') = 'Y') OR 
              (nvl(l_pend_lunch_missed_in_flag,'N') = 'N' AND
               nvl(l_pend_lunch_missed_out_flag,'N') = 'Y') THEN

          IF l_lunch_type_flag THEN

            l_reason_for_miss      := l_pend_lunch_missed_out_reason;
            l_others_reason_detail := l_pend_lunch_missed_out_detail;

          ELSE

            l_reason_for_miss      := l_pend_missed_out_reason;
            l_others_reason_detail := l_pend_missed_out_detail;

          END IF;

        ELSIF (nvl(l_pend_missed_in_flag,'N') = 'N' AND
               nvl(l_pend_missed_out_flag,'N') = 'N') OR
              (nvl(l_pend_lunch_missed_in_flag,'N') = 'N' AND
               nvl(l_pend_lunch_missed_out_flag,'N') = 'N') THEN

          l_reason_for_miss      := null;
          l_others_reason_detail := null;

        END IF;
          
      ELSIF nvl(l_missed_in_flag,'N')  = 'N' AND 
            nvl(l_missed_out_flag,'N') = 'N' AND 
            l_last_punch_type          = 'IN' THEN

        l_time_range := nvl(l_auto_saved_start_time,to_char(l_last_punch_datetime_in,'HH24:MI')) || '_' || l_auto_saved_end_time;
      
      
      END IF;
      l_time_prod := 'XXAS_TCM';
      OPEN csr_pending_rec_det(cp_pending_clock_id      => self.pending_clock_id);
      FETCH csr_pending_rec_det INTO  l_last_punch_status
                                     ,l_current_approver_role
                                     ,l_reason_for_miss_apr
                                     ,l_others_reason_detail_apr
                                     ,l_payment_type_id
                                     ,l_start_date;
      CLOSE csr_pending_rec_det;
      OPEN  csr_person_data;
      FETCH csr_person_data INTO l_person_data;
      CLOSE csr_person_data;

      OPEN  csr_missed_punch_xml(cp_start_date            => l_start_date
                                ,cp_payment_type_id       => l_payment_type_id
                                ,cp_time_range            => l_time_range
                                ,cp_last_punch_status     => l_last_punch_status
                                ,cp_time_prod             => l_time_prod
                                ,cp_reason_for_miss       => nvl(l_reason_for_miss,l_reason_for_miss_apr)
                                ,cp_others_reason_detail  => nvl(l_others_reason_detail,l_others_reason_detail_apr)
                                ,cp_current_approver_role => l_current_approver_role
                                ,cp_person_data           => l_person_data);
      FETCH csr_missed_punch_xml INTO l_xml;
      CLOSE csr_missed_punch_xml;

    END IF;

    RETURN l_xml;

  END;

END;
/

COMMIT;
EXIT;