/* $Id$ */
/*============================================================================*
 |                 Copyright (c) 2014 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK

CREATE OR REPLACE PACKAGE BODY xxas_page_rslt_cache_pkg AS 

--
-- Private Global Types and Constants
--
g_PACKAGE_NAME CONSTANT varchar2(30) := 'xxas_page_rslt_cache_pkg';

-- ------------------------------------------------------------------------ *
-- Name    : menu_to_slug
-- ------------------------------------------------------------------------ *
FUNCTION menu_to_slug
  (p_menu_id   IN number
  ,p_menu_name IN varchar2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_lookup_values)
  $END
IS

  l_PROC CONSTANT varchar2(30) := 'menu_name_to_slug';

BEGIN

  IF p_menu_id IS NULL THEN
    RETURN null;
  END IF;
  
  RETURN nvl(lower(xxas_util_pkg.encode_lookup
                    ('XXAS_PAGE_SLUGS', p_menu_name))
            ,to_char(p_menu_id));
  
  --RETURN lower(regexp_replace(p_title, '[^a-z0-9]+', '-', 1, 0, 'i'));

END menu_to_slug;

-- ------------------------------------------------------------------------ *
-- Name    : slug_to_menu_name
-- ------------------------------------------------------------------------ *
FUNCTION slug_to_menu_name
  (p_page_id IN varchar2) RETURN varchar2
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_menus, fnd_lookup_values)
  $END
IS

  l_PROC CONSTANT varchar2(30) := 'slug_to_menu_name';
  l_menu_id       number;
  
BEGIN
 
  IF p_page_id IS NULL THEN
    RETURN null;
  END IF;
  
  --
  -- If the page is a number, return that immediately
  --
  BEGIN
    l_menu_id := to_number(p_page_id);
  EXCEPTION
    WHEN others THEN
      null;    
  END;

  IF l_menu_id IS NOT NULL THEN
    RETURN xxas_rslt_cache_pkg.menu_id_to_name(l_menu_id);
  ELSE
    RETURN xxas_util_pkg.decode_lookup
             ('XXAS_PAGE_SLUGS', upper(p_page_id));
  END IF;
 
END slug_to_menu_name;

-- ------------------------------------------------------------------------ *
-- Name    : slug_to_menu_id
-- ------------------------------------------------------------------------ *
FUNCTION slug_to_menu_id
  (p_page_id IN varchar2) RETURN number
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_menus, fnd_lookup_values)
  $END
IS

  l_PROC CONSTANT varchar2(30) := 'slug_to_menu_id';
  l_menu_id       number;
  
BEGIN
 
  IF p_page_id IS NULL THEN
    RETURN null;
  END IF;
  
  --
  -- If the page is a number, return that immediately
  --
  BEGIN
    l_menu_id := to_number(p_page_id);
  EXCEPTION
    WHEN others THEN
      null;    
  END;

  IF l_menu_id IS NOT NULL THEN
    RETURN l_menu_id;
  ELSE
    RETURN xxas_rslt_cache_pkg.menu_name_to_id
             (xxas_util_pkg.decode_lookup
               ('XXAS_PAGE_SLUGS', upper(p_page_id)));
  END IF;
 
END slug_to_menu_id;

-- ------------------------------------------------------------------------ *
-- Name    : get_page
-- ------------------------------------------------------------------------ *
FUNCTION get_page
  (p_menu_id IN number
  ,p_language_code IN varchar2) RETURN xxas_pages_v%ROWTYPE
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_menus, fnd_menus_tl, xxas_object_values)
  $END
IS

  l_return xxas_pages_v%ROWTYPE;

  --
  -- Gets the lookup code using the meaning
  --
  CURSOR csr_page IS
  SELECT p.*
  FROM   xxas_pages_v p
  WHERE  p.menu_id = p_menu_id;

BEGIN

  --
  -- Check for null parameters
  --
  IF p_menu_id IS NOT NULL THEN
    --
    -- Encode the lookup
    --
    OPEN  csr_page;
    FETCH csr_page INTO l_return;
    CLOSE csr_page;

  END IF;

  RETURN l_return;

END get_page;

-- ------------------------------------------------------------------------ *
-- Name    : block_actions
-- ------------------------------------------------------------------------ *
FUNCTION block_actions
  (p_actions IN varchar2) RETURN g_action_t
  $IF DBMS_DB_VERSION.VER_LE_10 $THEN
    
  $ELSE
    RESULT_CACHE RELIES_ON (fnd_form_functions, fnd_new_messages)
  $END
IS

  l_PROC CONSTANT varchar2(30) := 'block_actions';
  l_actions_tbl   g_action_t;
  l_count         number       := 1;
  
  CURSOR csr_ba
  IS
  SELECT f.function_id
        ,f.function_name
        ,f.web_icon icon
        ,f.object_id obj_id
        ,o.obj_name
        ,xxas_com_msg_pkg.get_string(xxas_com_sec_pkg.get_parameter_value(f.parameters, 'pTitle')) title
        ,xxas_com_sec_pkg.get_parameter_value(f.parameters, 'pTitle') title_message
        ,xxas_com_sec_pkg.get_parameter_value(f.parameters, 'pWeight') weight
        ,xxas_com_sec_pkg.get_parameter_value(f.parameters, 'pColor') color
        ,xxas_com_sec_pkg.get_parameter_value(f.parameters, 'pOperation') operation
        ,xxas_com_sec_pkg.get_parameter_value(f.parameters, 'pHref') href
        ,nvl(xxas_com_sec_pkg.get_parameter_value(f.parameters, 'pPageId')
            ,xxas_com_sec_pkg.get_parameter_value(f.parameters, 'pSlug')) page_id
        ,xxas_com_sec_pkg.get_parameter_value(f.parameters, 'pObjectId') object_id
        ,xxas_com_sec_pkg.get_parameter_value(f.parameters, 'pTarget') target
        ,xxas_com_sec_pkg.get_parameter_value(f.parameters, 'pConfirmation') confirmation_flag
        ,xxas_com_sec_pkg.get_parameter_value(f.parameters, 'pConfirmationText') confirmation_text
        ,xxas_com_sec_pkg.get_parameter_value(f.parameters, 'pConfirmationMessage') confirmation_message
  FROM   TABLE(xxas_util_pkg.string_to_varchar2tbl(p_actions)) ba
        ,fnd_form_functions f
        ,fnd_objects o
  WHERE  ba.column_value = f.function_name
  AND    f.object_id = o.object_id(+);
  
BEGIN

  IF p_actions IS NOT NULL THEN
  
    FOR csr_rec IN csr_ba LOOP
    
      l_actions_tbl(l_count).function_id          := csr_rec.function_id;
      l_actions_tbl(l_count).function_name        := csr_rec.function_name;
      l_actions_tbl(l_count).icon                 := csr_rec.icon;
      l_actions_tbl(l_count).obj_id               := csr_rec.obj_id;
      l_actions_tbl(l_count).obj_name             := csr_rec.obj_name;
      l_actions_tbl(l_count).action_title         := csr_rec.title;
      l_actions_tbl(l_count).action_title_message := csr_rec.title_message;
      l_actions_tbl(l_count).operation            := csr_rec.operation;
      l_actions_tbl(l_count).href                 := csr_rec.href;
      l_actions_tbl(l_count).page_id              := csr_rec.page_id;
      l_actions_tbl(l_count).object_id            := csr_rec.object_id;
      l_actions_tbl(l_count).target               := csr_rec.target;
      l_actions_tbl(l_count).color                := csr_rec.color;
      l_actions_tbl(l_count).weight               := csr_rec.weight;
      l_actions_tbl(l_count).confirmation_flag    := csr_rec.confirmation_flag;
      l_actions_tbl(l_count).confirmation_text    := csr_rec.confirmation_text;
      l_actions_tbl(l_count).confirmation_message := csr_rec.confirmation_message;
      
      l_count := l_count + 1;
    END LOOP;
  END IF;
  
  RETURN l_actions_tbl;
  
END block_actions;

END xxas_page_rslt_cache_pkg;
/
COMMIT;
EXIT;
