/* $Id: xxas_tcm_missed_punch_ff_bg.sql 7275 2019-05-16 11:26:53Z ybudamala $ */
/*============================================================================*
 |                 Copyright (c) 2019 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK

DECLARE

  --
  -- Local constants and variables
  --
  e_no_bg_id          EXCEPTION;
  e_bg_not_exists     EXCEPTION;
  e_ff_type_not_found EXCEPTION;
  l_BG_ID   CONSTANT  NUMBER        := to_number('&1');
  l_USER_ID CONSTANT  NUMBER        := to_number('&2');
  l_START   CONSTANT  DATE          := to_date('1900-01-01','YYYY-MM-DD');
  l_END     CONSTANT  DATE          := xxas_util_pkg.eot;
  l_FF_NAME CONSTANT  VARCHAR2(80)  := 'XXAS_TCM_MISSED_PUNCH';
  l_FF_DESC CONSTANT  VARCHAR2(240) := 'Applaud Timecard Missed Punch - seeded formula';
  l_FF_TYPE CONSTANT  varchar2(30)  := 'Applaud Assignment';
  l_formula_text      LONG;
  l_formula_type_id   NUMBER;
  l_ff_id             NUMBER;
  l_bg_exists         VARCHAR2(1);

  --
  -- Gets the FF Id
  --
  CURSOR csr_ff_id IS
  SELECT ff.formula_id
  FROM   ff_formulas_f ff
  WHERE  ff.business_group_id = l_BG_ID
  AND    ff.formula_name = l_FF_NAME;

  --
  -- Tests if the Business Group already exists
  --
  CURSOR csr_bg_exists IS
  SELECT 'Y'
  FROM   per_business_groups_perf bg
  WHERE  bg.business_group_id = l_BG_ID;

  --
  -- Gets the Formula Type Id
  --
  CURSOR csr_ff_type IS
  SELECT fft.formula_type_id
  FROM   ff_formula_types fft
  WHERE  fft.formula_type_name = l_FF_TYPE;

BEGIN

  --
  -- Check that the Business Group Id was passed in
  --
  IF l_BG_ID IS NULL THEN
    RAISE e_no_bg_id;
  END IF;

  --
  -- Check the Business Group exists
  --
  OPEN  csr_bg_exists;
  FETCH csr_bg_exists INTO l_bg_exists;
  CLOSE csr_bg_exists;

  IF l_bg_exists IS NULL THEN
    RAISE e_bg_not_exists;
  END IF;

  --
  -- Fetch the Fast Formula Id
  --
  OPEN  csr_ff_id;
  FETCH csr_ff_id INTO l_ff_id;
  CLOSE csr_ff_id;

  --
  -- Get the Formula Type; error if it cannot be found
  --
  OPEN  csr_ff_type;
  FETCH csr_ff_type INTO l_formula_type_id;
  CLOSE csr_ff_type;

  IF l_formula_type_id IS NULL THEN
    RAISE e_ff_type_not_found;
  END IF;

  --
  -- Set the Formula Text
  --
  l_formula_text := q'[

/*==============================================================================

Formula Name: XXAS_TCM_MISSED_PUNCH
Formula Type: Applaud Assignment
Description:  Applaud Timecard Missed Punch - seeded formula

==============================================================================*/

/* +-------------------< SECTION 1: DEFAULTS >-------------------------------+
   Defaults must be specified for every item listed in the Inputs section. When
   a value is not set it will be defaulted to the default specified here.   */

  Alias sysdate As Now

  Default for Sysdate                   is '0001/01/01 00:00:00' (date)
  Default for Pending_Clock_Id          is -1
  Default for Person_Id                 is -1
  Default for Time_Prod                 is 'Unknown'
  Default for Current_State_Code        is 'Unknown'
  Default for Current_Approver_Role     is 'Unknown'

/* +-------------------< SECTION 2: INPUTS >---------------------------------+
   This section lists all the available items that you can use in this
   formula.                                                                 */

  Inputs are Person_Id
            ,Pending_Clock_Id
            ,Time_Prod
            ,Current_State_Code
            ,Current_Approver_Role

  If Time_Prod = 'TCM' Then
  (
      If (Current_State_Code was not Defaulted And 
          Current_State_Code = 'P' Or 
          Current_State_Code = 'R' Or
          Current_State_Code = 'E') Then
      (
          Approvers_Visible = 'true'

          If Current_Approver_Role was not defaulted And 
             XXAS_ROLE_NAME_IS_ME(Current_Approver_Role) = 'Y' Then
          (
              Submit_Label      = xxas_get_message('XXAS','XXAS_TCM_MIS_PUNCH_SAVE_APPR')
          )
          Else
          (
              Time_Range_Enabled = 'false'
              Reason_For_Miss_Enabled = 'false'
              Other_More_Detail_Enabled = 'false'
              Show_Submit      = 'false'
          )
      )
      Else
      (
          Approvers_Visible = 'false'
          Submit_Label      = xxas_get_message('XXAS','XXAS_RB_COM_SAVE')
      )
  )
  Else If Time_Prod = 'OTL' Then
  (
      If (Current_State_Code was not defaulted And 
          Current_State_Code = 'SUBMITTED' Or 
          Current_State_Code = 'REJECTED' Or
          Current_State_Code = 'ERRORED') Then
      (
          Approvers_Visible = 'true'

          If Current_Approver_Role was not defaulted And 
             XXAS_ROLE_NAME_IS_ME(Current_Approver_Role) = 'Y' Then
          (
              Submit_Label      = xxas_get_message('XXAS','XXAS_TCM_MIS_PUNCH_SAVE_APPR')
          )
          Else
          (
              Time_Range_Enabled = 'false'
              Reason_For_Miss_Enabled = 'false'
              Other_More_Detail_Enabled = 'false'
              Show_Submit      = 'false'
          )
      )
      Else
      (
          Approvers_Visible = 'false'
          Submit_Label      = xxas_get_message('XXAS','XXAS_RB_COM_SAVE')
      )

  )

  RETURN Approvers_Visible
        ,Submit_Label
        ,Show_Submit
        ,Time_Range_Enabled
        ,Reason_For_Miss_Enabled
        ,Other_More_Detail_Enabled 

]';


  BEGIN

    IF l_ff_id IS NULL THEN
      --
      -- Create the Fast Formula
      --
      INSERT INTO ff_formulas_f
        (formula_id
        ,effective_start_date
        ,effective_end_date
        ,business_group_id
        ,formula_type_id
        ,formula_name
        ,description
        ,formula_text)
      VALUES
        (ff_formulas_s.nextval
        ,l_START
        ,l_END
        ,l_BG_ID
        ,l_formula_type_id
        ,l_FF_NAME
        ,l_FF_DESC
        ,l_formula_text);

    ELSE
      --
      -- Update the existing Fast Formula text
      -- (if it has multiple date-track versions the text of
      -- all versions is updated).
      --
      UPDATE ff_formulas_f
      SET    formula_text = l_formula_text
            ,description  = l_FF_DESC
      WHERE  formula_id = l_ff_id;

    END IF;

    COMMIT;

    EXCEPTION
      WHEN others THEN
        ROLLBACK;
        RAISE;
    END;

    --
    -- Compile the Fast Formula
    --
    xxas_util_pkg.compile_ff
      (p_user_id => l_USER_ID
      ,p_ff_type => l_FF_TYPE
      ,p_ff_name => l_FF_NAME);

EXCEPTION

  WHEN e_no_bg_id THEN
    RAISE no_data_found;

  WHEN e_bg_not_exists THEN
    RAISE no_data_found;

  WHEN e_ff_type_not_found THEN
    RAISE no_data_found;

END;
/
COMMIT;
EXIT;
