/* $Id$ */
/*============================================================================*
 |                 Copyright (c) 2018 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
SET SCAN OFF
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK

CREATE OR REPLACE PACKAGE BODY xxas_us_reports_pkg AS 

gc_log xxas_com_log_type := xxas_com_log_type('xxas_us_reports_pkg');

--
-- Private Global Types and Constants
--
g_PACKAGE_NAME         CONSTANT varchar2(30) := 'xxas_us_reports_pkg';
g_FORM_LAYOUT          CONSTANT varchar2(4)  := 'FORM';
g_TABULAR_LAYOUT       CONSTANT varchar2(7)  := 'TABULAR';
g_W2TEMP_2012          CONSTANT varchar2(15)  := 'payusw2tmpl';
g_W2CTEMP              CONSTANT varchar2(15)  := 'payusw2ctmpl';
g_USW2INSTR            CONSTANT varchar2(15)  := 'PAYUSW2INSTR';


--
-- ------------------------------------------------------------------------ *
-- Name    : log_start
-- ------------------------------------------------------------------------ *
PROCEDURE log_start
  (p_method      IN  varchar2
  ,p_parameter1  IN  varchar2 DEFAULT null
  ,p_parameter2  IN  varchar2 DEFAULT null
  ,p_parameter3  IN  varchar2 DEFAULT null
  ,p_parameter4  IN  varchar2 DEFAULT null
  ,p_parameter5  IN  varchar2 DEFAULT null
  ,p_parameter6  IN  varchar2 DEFAULT null
  ,p_parameter7  IN  varchar2 DEFAULT null
  ,p_parameter8  IN  varchar2 DEFAULT null
  ,p_parameter9  IN  varchar2 DEFAULT null
  ,p_parameter10 IN  varchar2 DEFAULT null)
IS
BEGIN
   IF gc_log.proc=1 
   THEN 
     gc_log.procedure_start_params
       (p_proc_name   =>  p_method
       ,p_parameter1  =>  p_parameter1
       ,p_parameter2  =>  p_parameter2
       ,p_parameter3  =>  p_parameter3
       ,p_parameter4  =>  p_parameter4
       ,p_parameter5  =>  p_parameter5
       ,p_parameter6  =>  p_parameter6
       ,p_parameter7  =>  p_parameter7
       ,p_parameter8  =>  p_parameter8
       ,p_parameter9  =>  p_parameter9
       ,p_parameter10 =>  p_parameter10); 
   END IF;  
END log_start; 

--
-- ------------------------------------------------------------------------ *
-- Name    : log_end
-- ------------------------------------------------------------------------ *
PROCEDURE log_end
  (p_method  IN  varchar2
  ,p_result  IN  varchar2 DEFAULT null)
IS
BEGIN
  IF gc_log.proc=1 
  THEN 
    gc_log.procedure_end_result
      (p_proc_name =>  p_method
      ,p_result    =>  p_result); 
  END IF;
END log_end;  

--
-- ------------------------------------------------------------------------ *
-- Name    : log_statement
-- ------------------------------------------------------------------------ *
PROCEDURE log_statement
  (p_method     IN  varchar2
  ,p_log_string IN  varchar2)
IS
BEGIN
  IF gc_log.stmt=1 
  THEN 
    gc_log.statement
      (p_proc_name  =>  p_method
      ,p_log_string =>  p_log_string); 
  END IF;
END log_statement;

--
-- ------------------------------------------------------------------------ *
-- Name    : check_template_exists
-- ------------------------------------------------------------------------ *
FUNCTION check_template_exists(p_app_short_name   IN VARCHAR2
                              ,p_data_source_code IN VARCHAR2
							  ,p_template_code    IN VARCHAR2) RETURN BOOLEAN
IS
  CURSOR csr_template
  IS
  SELECT 'Y'
  FROM   XDO_TEMPLATES_VL XT
        ,XDO_DS_DEFINITIONS_B XDS
  WHERE  XT.DS_APP_SHORT_NAME = XDS.APPLICATION_SHORT_NAME
  AND    XT.DATA_SOURCE_CODE = XDS.DATA_SOURCE_CODE
  AND    XT.DS_APP_SHORT_NAME = p_app_short_name
  AND    XT.DATA_SOURCE_CODE = p_data_source_code
  AND    XT.TEMPLATE_CODE = p_template_code;
  
  l_exists VARCHAR2(10);
  
BEGIN
  
  OPEN  csr_template;
  FETCH csr_template INTO l_exists;
  CLOSE csr_template;
  
  IF l_exists = 'Y' THEN 
    RETURN TRUE;
  ELSE 
    RETURN FALSE;
  END IF;
  
END check_template_exists;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_main_template
-- ------------------------------------------------------------------------ *
FUNCTION get_main_template(p_business_group_id IN NUMBER
                          ,p_organization_id   IN NUMBER
						  ,p_location_id       IN NUMBER
						  ,p_date              IN DATE
						  ,p_year              IN NUMBER
						  ,p_report_type       IN VARCHAR2
                          ) RETURN VARCHAR2
IS
  l_PROC  CONSTANT VARCHAR2(100) := 'get_main_template';
  
  CURSOR csr_template_preferences
  IS
  SELECT
       nvl(hoi.template_code, 'ABSENT')  template_code,
       xt.application_short_name ApplicationShortName,
       xt.default_language DefaultLanguage,
       xt.default_territory DefaultTerritory
       ,xt.template_type_code TemplateTypeCode 
  FROM (SELECT(
SELECT template_code
FROM
    (
    SELECT template_code
      FROM
            (SELECT lei_information4 template_code,
                   lei_information5 xml_code,
                   1 sort_col
              FROM hr_location_extra_info hlei
             WHERE hlei.location_id = p_location_id
               AND hlei.information_type = 'HR_SELF_SERVICE_LOC_PREFERENCE'
			   AND hlei.lei_information1 = CASE p_report_type
			                               WHEN 'YREND' THEN 'W-2'
										   ELSE 'W-2C'
										   END
            UNION
            SELECT org_information4 template_code,
                   org_information5 xml_code,
                   2 sort_col
              FROM hr_organization_information hoi
             WHERE hoi.organization_id = p_organization_id
               AND hoi.org_information_context = 'HR_SELF_SERVICE_ORG_PREFERENCE'
			   AND hoi.org_information1 = CASE p_report_type
			                               WHEN 'YREND' THEN 'W-2'
										   ELSE 'W-2C'
										   END
            UNION
            SELECT org_information4 template_code,
                   org_information5 xml_code,
                   3 sort_col
              FROM hr_organization_information hoi
             WHERE hoi.organization_id = p_business_group_id
               AND hoi.org_information_context = 'HR_SELF_SERVICE_BG_PREFERENCE'
			   AND hoi.org_information1 = CASE p_report_type
			                               WHEN 'YREND' THEN 'W-2'
										   ELSE 'W-2C'
										   END)
    WHERE template_code IS NOT NULL
    ORDER BY sort_col
    )
WHERE rownum = 1) template_code
FROM dual) hoi,
       xdo_templates_b xt
 WHERE nvl(hoi.template_code, xt.template_code) = xt.template_code
 AND   sysdate BETWEEN xt.start_date AND nvl(xt.end_date, hr_general.end_of_time);
 
 l_rec_csr_template_pref csr_template_preferences%ROWTYPE;
 
BEGIN
  
  log_start(l_PROC);
  
  OPEN csr_template_preferences;
  FETCH csr_template_preferences INTO l_rec_csr_template_pref;
  CLOSE csr_template_preferences;
  
  log_statement(l_PROC,'p_report_type: ' || p_report_type);
  log_statement(l_PROC,'From Preferences W2 code: ' || l_rec_csr_template_pref.template_code);
  
  --
  -- If no preferences are set then pick it based on below logic
  --
  IF l_rec_csr_template_pref.template_code = 'ABSENT' THEN
    IF p_report_type = 'YREND' THEN
	  l_rec_csr_template_pref.template_code := g_USW2_DATA_SOURCE;
	  IF p_year >= 2012 THEN
		l_rec_csr_template_pref.template_code := g_W2TEMP_2012 || p_year;
		IF NOT check_template_exists('PAY',g_USW2_DATA_SOURCE,l_rec_csr_template_pref.template_code) THEN
		  l_rec_csr_template_pref.template_code := g_W2TEMP_2012 || (p_year -1);
		  IF NOT check_template_exists('PAY',g_USW2_DATA_SOURCE,l_rec_csr_template_pref.template_code) THEN
		    l_rec_csr_template_pref.template_code := g_USW2_DATA_SOURCE;
		  END IF;
		END IF;
	  END IF;
	ELSE
	  l_rec_csr_template_pref.template_code := g_W2CTEMP || p_year;
	  IF NOT check_template_exists('PAY',g_USW2C_DATA_SOURCE,l_rec_csr_template_pref.template_code) THEN
	    l_rec_csr_template_pref.template_code := g_W2CTEMP || (p_year-1);
		IF NOT check_template_exists('PAY',g_USW2C_DATA_SOURCE,l_rec_csr_template_pref.template_code) THEN
		  l_rec_csr_template_pref.template_code := g_W2CTEMP;  
		END IF;
	  END IF;
	END IF;
  END IF;
  
  log_statement(l_PROC,'l_rec_csr_template_pref.template_code:' || l_rec_csr_template_pref.template_code);
  log_end(l_PROC);
  
  RETURN l_rec_csr_template_pref.template_code;
  
END get_main_template;

-- ------------------------------------------------------------------------ *
-- Name    : get_instr_template
-- ------------------------------------------------------------------------ *
FUNCTION get_instr_template(p_year              IN NUMBER
						   ,p_report_type       IN VARCHAR2
                           ) RETURN VARCHAR2
IS
 
BEGIN
  
  IF NOT check_template_exists('PAY',g_USW2_DATA_SOURCE,g_USW2INSTR||p_year) THEN
    RETURN g_USW2INSTR;
  ELSE
    RETURN g_USW2INSTR ||p_year;
  END IF;
  
END get_instr_template;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_w2reports_tbl
-- ------------------------------------------------------------------------ *
FUNCTION get_w2reports_tbl
  (p_person_id            IN number
  ,p_assignment_action_id IN number DEFAULT NULL) RETURN xxas_us_report_table PIPELINED
IS

  l_effective_date DATE;
  
  CURSOR csr_w2_and_w2c_actions
           (cp_person_id      IN NUMBER
		   ,cp_effective_date IN DATE
		   )
  IS
  SELECT hou.name
          ||'-'
		  ||to_char(ppa.effective_date,'YYYY') 
		  ||'-'
		  || 'W2 PDF' w2Choice
        ,to_char(ppa.effective_date,'YYYY') year
		,paa.assignment_action_id assignment_action_id
        ,paa.assignment_action_id prev_assignment_action_id
        ,ppa.effective_date
		,ppa.payroll_action_id
		,paaf.business_group_id
		,paaf.organization_id
		,paaf.location_id
		,paa.tax_unit_id
		,ppa.report_type
  FROM   pay_payroll_actions ppa
        ,pay_assignment_actions paa
        ,hr_all_organization_units hou
		,per_all_assignments_f paaf
  WHERE paa.payroll_action_id = ppa.payroll_action_id
  AND   paa.serial_number = cp_person_id
  AND   paa.assignment_id = paaf.assignment_id
  AND   paaf.effective_start_date = (SELECT max(paaf1.effective_start_date)
                                     FROM   per_all_assignments_f paaf1
									 WHERE  paaf1.assignment_id = paaf.assignment_id
									 )
  AND   ppa.report_type = 'YREND'
  AND   ppa.action_type = 'X'
  AND   ppa.action_status = 'C'
  AND   hou.organization_id = paa.tax_unit_id
  AND   ppa.effective_date >= to_date('2001/12/31','yyyy/mm/dd')
  AND   ppa.effective_date < cp_effective_date --to_date(cp_effective_date,'YYYY/MM/DD')
  AND   hou.organization_id NOT IN
          (SELECT organization_id 
		   FROM hr_organization_information
           WHERE org_information_context = '1099R Magnetic Report Rules'
		  )
  AND   hou.organization_id NOT IN
         (SELECT organization_id 
          FROM   hr_organization_information
          WHERE  org_information_context = 'W2 Reporting Rules'
          AND org_information16='P' 
         )
  UNION
  SELECT hou.name
           ||'-'
		   ||to_char(ppa.effective_date,'YYYY')
		   ||'-'
		   ||'W-2c PDF' w2Choice
        ,to_char(ppa.effective_date,'YYYY') Year
		,paa.assignment_action_id assignment_action_id
        ,to_number(substr(paa1.serial_number, 16,15)) prev_assignment_action_id
        ,ppa.effective_date
		,ppa.payroll_action_id
		,paaf.business_group_id
		,paaf.organization_id
		,paaf.location_id
		,paa.tax_unit_id
		,ppa.report_type
  FROM   pay_payroll_actions ppa
        ,pay_assignment_actions paa
        ,hr_all_organization_units hou
	    ,pay_action_interlocks pai
        ,pay_assignment_actions paa1
		,per_all_assignments_f paaf
  WHERE paa.payroll_action_id = ppa.payroll_action_id
  AND   paa.serial_number = cp_person_id
  AND   paa.assignment_id = paaf.assignment_id
  AND   paaf.effective_start_date = (SELECT max(paaf1.effective_start_date)
                                     FROM   per_all_assignments_f paaf1
									 WHERE  paaf1.assignment_id = paaf.assignment_id
									 )
  AND report_type = 'W2C_PRE_PROCESS'
  AND paa.assignment_action_id = pai.locked_action_id
  AND pai.locking_action_id = paa1.ASSIGNMENT_ACTION_ID
  AND action_type = 'X'
  AND ppa.action_status = 'C'
  AND hou.organization_id = paa.tax_unit_id
  AND ppa.effective_date >= to_date('2001/12/31','yyyy/mm/dd')
  AND ppa.effective_date < cp_effective_date
  AND hou.organization_id NOT IN
        (SELECT organization_id 
		 FROM   hr_organization_information
         WHERE  org_information_context = '1099R Magnetic Report Rules')
  AND hou.organization_id NOT IN
        (SELECT organization_id 
		 FROM   hr_organization_information
         WHERE  org_information_context = 'W2 Reporting Rules'
         AND org_information16='P')
  ORDER BY year desc,assignment_action_id DESC;

  c_method CONSTANT       varchar2(30) := 'get_w2reports_tbl';
  e_null_params           exception;
  l_row                   xxas_us_report_type;
  l_template_code         xdo_templates_vl.template_code%TYPE;

  CURSOR csr_asg_info
  IS
  SELECT paaf.business_group_id
        ,paaf.organization_id
		,paaf.location_id
  FROM   per_all_assignments_f paaf
  WHERE  paaf.person_id = p_person_id
  AND    paaf.primary_flag = 'Y'
  AND    paaf.effective_start_date = (select max(effective_start_date)
                                      from per_all_assignments_f paf1
                                      where paf1.assignment_id = paaf.assignment_id
                                      and paf1.effective_start_date
                                                 <= SYSDATE)
  ORDER BY paaf.assignment_id desc;
  
  CURSOR csr_template_type(cp_template_name VARCHAR2)
  IS
  SELECT xtv.template_type_code
  FROM   xdo_templates_vl xtv
  WHERE  xtv.template_code = cp_template_name;
  
  l_asg_info csr_asg_info%ROWTYPE;
  l_template_type         VARCHAR2(100);
  l_main_template         VARCHAR2(100);
  
BEGIN

  log_start(c_method,p_person_id,p_assignment_action_id);

  IF p_person_id IS NULL THEN
    RAISE e_null_params;
  END IF;

  l_effective_date := to_date(fnd_profile.value_wnps('HR_VIEW_ONLINE_W2') || to_char(sysdate,'YYYY'),'MM/DD/YYYY');
  
  FOR w2_report_rec IN csr_w2_and_w2c_actions(p_person_id,l_effective_date)
  LOOP

    log_statement(c_method,'Period Name: ' || w2_report_rec.w2Choice);
    log_statement(c_method,'Payroll Action Id: ' || w2_report_rec.payroll_action_id);
    log_statement(c_method,'Assignment Action Id: ' || w2_report_rec.assignment_action_id);

    l_row.assignment_action_id := w2_report_rec.assignment_action_id;
	l_row.title			       := w2_report_rec.w2Choice;
	
	--
    -- Retrieve W2 template code from preferences and if no preferences are set
    -- then get the appropriate template for the year.
    -- 
    l_main_template := get_main_template(p_business_group_id => w2_report_rec.business_group_id
                                        ,p_organization_id   => w2_report_rec.organization_id
                                        ,p_location_id       => w2_report_rec.location_id
                                        ,p_date              => w2_report_rec.effective_date
  			    						,p_year              => w2_report_rec.year
  									    ,p_report_type       => w2_report_rec.report_type
										);

    -- get XDO file type
    OPEN csr_template_type(l_main_template);
    FETCH csr_template_type INTO l_template_type;
    CLOSE csr_template_type;
	
	IF l_template_type = 'RTF' THEN
   	  l_row.template_code := l_main_template;
	  l_row.tax_unit_id := w2_report_rec.tax_unit_id;
	  l_row.year  := w2_report_rec.year;
	END IF;
	
	l_row.report_type := w2_report_rec.report_type;
	l_row.prev_action_id := w2_report_rec.prev_assignment_action_id;
	
    PIPE ROW (l_row);

  END LOOP;

  log_end(c_method);

  RETURN;

EXCEPTION

  WHEN e_null_params THEN

    log_statement(c_method,'e_null_params exception');
    log_end(c_method);
    RETURN;

END get_w2reports_tbl;

-- ------------------------------------------------------------------------ *
-- Name    : get_w2_report_details_xml
-- ------------------------------------------------------------------------ *
$IF xxas_us_dynamic_pkg.w2_pkg_exists $THEN

FUNCTION get_w2_report_details_xml
  (p_assignment_action_id IN NUMBER
  ,p_tax_unit_id          IN NUMBER
  ,p_year                 IN NUMBER
  ,p_prev_action_id       IN NUMBER
  ,p_report_type          IN VARCHAR2) RETURN XMLType
IS
 l_error VARCHAR2(4000);
 l_blob  BLOB;
BEGIN
 
  IF p_report_type = 'YREND' THEN
    l_blob := PAY_US_W2_INFO_PKG.get_final_xml
        (p_assignment_action_id,p_tax_unit_id,p_year,null,null,null,l_error);
  ELSIF p_report_type = 'W2C_PRE_PROCESS' THEN
    l_blob := PAY_US_W2C_RPT.get_final_xml
        (p_assignment_action_id,p_prev_action_id,p_tax_unit_id,p_year,null,null,null,l_error);
  END IF;
  
  IF l_blob IS NOT NULL THEN
    RETURN XMLType(UTL_RAW.CAST_TO_VARCHAR2(DBMS_LOB.SUBSTR(l_blob, 4000, 1)));
  END IF;
  
  RETURN NULL;
  
END get_w2_report_details_xml;

$END

-- ------------------------------------------------------------------------ *
-- Name    : get_w2_report_details
-- ------------------------------------------------------------------------ *
PROCEDURE get_w2_report_details
  (p_assignment_action_id IN  number
  ,p_tax_unit_id          OUT number
  ,p_year                 OUT number)
IS

BEGIN
 
  SELECT tax_unit_id
        ,to_char(ppa.effective_date,'YYYY')
  INTO   p_tax_unit_id
        ,p_year
  FROM   pay_assignment_actions paa
        ,pay_payroll_actions ppa
  WHERE  paa.assignment_action_id = p_assignment_action_id
  AND    ppa.payroll_action_id = paa.payroll_action_id;
 
END get_w2_report_details;

--
-- ------------------------------------------------------------------------ *
-- Name    : getW2URL
-- ------------------------------------------------------------------------ *
FUNCTION getW2URL
  (p_assignment_action_id IN number
  ,p_inline               IN varchar2 DEFAULT 'N'
  ,p_data_source_code     IN VARCHAR2
  ,p_template_code        IN VARCHAR2
  ,p_tax_unit_id          IN NUMBER
  ,p_year                 IN NUMBER
  ,p_prev_action_id       IN NUMBER
  ,p_report_type          IN VARCHAR2

  ) RETURN varchar2
IS

  c_method CONSTANT varchar2(30) := 'getW2URL';
  
BEGIN

 /*
  IF p_rest_api_path is not null THEN
    log_end(c_method);
    RETURN utl_url.unescape(xxas_com_rest_util_pkg.uri_from_path(p_rest_api_path), xxas_util_pkg.get_db_charset);
  END IF;
*/
  IF p_assignment_action_id   is null
  THEN
    log_end(c_method);
    RETURN null;
  ELSE
    log_end(c_method);
    RETURN xxas_com_rest_util_pkg.uri_from_path
              ('/w2/'
            || to_char(p_assignment_action_id)
			|| '/' 
			|| p_data_source_code
            || '?_accept=application/pdf'
            || CASE WHEN p_inline = 'Y' THEN
                 '&inline=true'
               END
			|| '&reportType=' || p_report_type
			|| CASE WHEN p_template_code IS NOT NULL THEN
			     '&biPublisherTemplate=' || p_template_code
				 || '&taxUnitId=' || p_tax_unit_id
				 || '&year=' || p_year
			   ELSE
                 '&biPublisherDataSourceCode=' || p_data_source_code
			    || '&actionContextId=' || p_assignment_action_id
			   END 
			|| CASE WHEN p_report_type = 'W2C_PRE_PROCESS' THEN
			     '&prevActionId=' || p_prev_action_id
			   END 
			  );
  END IF;

END getW2URL;

--
-- ------------------------------------------------------------------------ *
-- Name    : get_templates
-- ------------------------------------------------------------------------ *
PROCEDURE get_templates(p_action_context_id IN  NUMBER
                       ,p_main_template     OUT VARCHAR2
					   ,p_instr_template    OUT VARCHAR2)

IS
  CURSOR csr_asg_info
  IS
  SELECT paaf.business_group_id
        ,paaf.organization_id
		,paaf.location_id
		,to_char(ppa.effective_date,'YYYY') year
		,ppa.effective_date
		,ppa.report_type
  FROM   per_all_assignments_f paaf
        ,pay_assignment_actions paa
		,pay_payroll_actions ppa
  WHERE  paa.assignment_action_id = p_action_context_id
  AND    paaf.assignment_id = paa.assignment_id
  AND    paa.payroll_action_id = ppa.payroll_action_id
  AND    paaf.primary_flag = 'Y'
  AND    paaf.effective_start_date = (select max(effective_start_date)
                                      from per_all_assignments_f paf1
                                      where paf1.assignment_id = paaf.assignment_id
                                      and paf1.effective_start_date
                                                 <= SYSDATE)
  AND   xxas_com_sec_pkg.test_instance('XXAS_W2_PERM'
		                              ,paaf.person_id) = 'Y' 												 
  ORDER BY paaf.assignment_id desc;
  
  l_asg_info csr_asg_info%ROWTYPE;
  
BEGIN

  OPEN csr_asg_info;
  FETCH csr_asg_info INTO l_asg_info;
  CLOSE csr_asg_info;
  
  --
  -- Retrieve W2 template code from preferences and if no preferences are set
  -- then get the appropriate template for the year.
  -- 
  p_main_template := get_main_template(p_business_group_id => l_asg_info.business_group_id
                                      ,p_organization_id   => l_asg_info.organization_id
                                      ,p_location_id       => l_asg_info.location_id
                                      ,p_date              => l_asg_info.effective_date
  									  ,p_year              => l_asg_info.year
  									  ,p_report_type       => l_asg_info.report_type);
	
  -- Get instruction template	
  p_instr_template := get_instr_template(p_year              => l_asg_info.year
  									    ,p_report_type       => l_asg_info.report_type);
END get_templates;

END xxas_us_reports_pkg;
/
COMMIT;
EXIT;