/* $Id$ */
/*============================================================================*
 |                 Copyright (c) 2017 Applaud Solutions UK Ltd                |
 |                             All rights reserved                            |
 *===========================================================================*/

SET VERIFY OFF
SET FEEDBACK OFF
WHENEVER OSERROR  EXIT FAILURE ROLLBACK
WHENEVER SQLERROR EXIT FAILURE ROLLBACK

CREATE OR REPLACE TYPE BODY xxas_cal_event IS

  CONSTRUCTOR FUNCTION xxas_cal_event
    (p_person_id       IN number
    ,p_resource_id     IN number
    ,p_start_time      IN date
    ,p_end_time        IN date
    ,p_action_id       IN varchar2
    ,p_source_block    IN varchar2
    ,p_saved_search_id IN number
    ,p_manager_id      IN number default null
    ,p_def_abs_type_flag   IN varchar2 default 'Y') RETURN SELF AS RESULT
  IS

  BEGIN

    self.person_id         :=  p_person_id;
    self.resource_id       :=  p_resource_id;
    self.start_date        :=  trunc(p_start_time);
    self.start_time        :=  to_char(p_start_time,'HH24:MI');
    self.end_date          :=  trunc(p_end_time);
    self.end_time          :=  to_char(p_end_time,'HH24:MI');
    self.action_id         :=  p_action_id;
    self.start_date_time   :=  p_start_time;
    self.end_date_time     :=  p_end_time;
    self.source_block      :=  p_source_block;
    self.saved_search_id   :=  p_saved_search_id;
    self.manager_id        :=  p_manager_id;
    self.def_abs_type_flag :=  nvl(p_def_abs_type_flag,'Y');

    RETURN;

  END;

  MEMBER FUNCTION get_xml RETURN XMLType
  IS

    l_xml                 XMLType;
    l_abs_def_att_type_id number;
    l_asg_data            XMLType;
    l_person_data         XMLType;
    l_approver_person_id  number;
    l_ol_absences_char    varchar2(4000);
    l_ol_timecards_char   varchar2(4000);
    l_repeat_entry        varchar2(1);
    l_repeats             varchar2(30);
    l_repeat_daily        number;
    l_repeat_weekly       number;
    l_repeat_monthly      number;
    l_ends                varchar2(30);
    l_occurences          number;
    l_till_end_date       date;
    l_date_time_range     varchar2(100);
    l_date_range          varchar2(100);
    l_time_range          varchar2(30);
    l_periodtab           hxc_timecard_utilities.periods;
    l_rec_period_id       number;
    l_period_start_date   date;
    l_period_end_date     date;
    l_rowindex            number;
	
    CURSOR csr_abs_def_att_type_id IS
    SELECT abt.absence_attendance_type_id
    FROM   per_abs_attendance_types_v abt
    WHERE  abt.business_group_id = (SELECT papf.business_group_id
                                    FROM   per_all_people_f papf
                                    WHERE  papf.person_id = self.person_id
                                    AND    trunc(sysdate) BETWEEN papf.effective_start_date 
                                                          AND     papf.effective_end_date
                                    AND    (nvl(papf.current_employee_flag, 'N') = 'Y'
                                            OR nvl(papf.current_npw_flag, 'N') = 'Y'))
    AND    xxas_com_sec_pkg.test_instance('XXAS_DATA_ABS_TYPE', abt.absence_attendance_type_id) = 'Y'
    AND    trunc(sysdate) BETWEEN abt.date_effective 
                          AND     nvl(abt.date_end, trunc(sysdate))
    AND    upper(abt.name) = 'VACATION'
    ORDER BY nvl(abt.absence_category, 'A') desc;

    CURSOR csr_asg_data IS
    SELECT XMLElement
            ("EMPLOYEES"
            ,XMLElement
               ("value"
               ,XMLAttributes
                 ('true' as "json:force-array"
                 ,'http://json.org/' as "xmlns:json")
               ,XMLAgg
                 (XMLElement
                   ("value"
                   ,XMLForest
                     (p.assignment_id "id"
                     ,p.short_name "label"
                     ,xxas_util_pkg.flag_to_boolean('Y') "deletable")
                   ,XMLElement
                     ("linked"
                     ,XMLElement
                       ("image"
                       ,XMLForest
                         (xxas_com_rest_util_pkg.uri_from_path
                            ('people/' || p.img_person_id || '/photo') "self"
                         ,p.initials "initials"
                         ,xxas_per_image_pkg.url
                            (p.img_person_id, p.img_last_update_date, 'SMALL') "small"
                         )
                       )
                     )
                   )
                 )
               ).getClobVal()
            )
    FROM  (SELECT paaf.assignment_id
                 ,xxas_util_pkg.get_formatted_name
                    (papf.full_name
                    ,papf.global_name
                    ,papf.local_name
                    ,papf.first_name
                    ,papf.last_name
                    ,papf.known_as) short_name
                 ,xxas_util_pkg.get_initials(papf.first_name, papf.last_name, papf.known_as) initials
                 ,img.person_id              img_person_id
                 ,img.last_update_date       img_last_update_date
           FROM   per_all_people_f papf
                 ,per_all_assignments_f paaf
                 ,xxas_per_thumbnails img
           WHERE  paaf.assignment_id = self.resource_id
           AND    paaf.person_id     = papf.person_id
           AND    trunc(sysdate) BETWEEN papf.effective_start_date 
                                 AND     papf.effective_end_date
           AND    self.start_date BETWEEN paaf.effective_start_date
                                  AND     paaf.effective_end_date
           AND    papf.person_id = img.person_id(+)) p;

    CURSOR csr_approver_person IS
    SELECT distinct mgra.person_id
    FROM   TABLE(xxas_com_hier_pkg.mgrs_for_asg_tbl
                   (self.resource_id, trunc(sysdate))) mgra;

    CURSOR csr_abs_cal_event_xml(cp_business_group_id IN number) IS
    SELECT XMLElement
            ("ROW"
            ,XMLForest
              (self.person_id    as person_id
              ,l_date_time_range as date_time_range
              ,l_date_range      as date_range
              ,self.resource_id  as assignment_id
              ,null              as transaction_id
              ,null              as transaction_step_id
              ,self.source_block as source_block
              ,self.saved_search_id as saved_search_id
              ,xxas_cal_util_pkg.source_block_called_view(self.source_block) as called_view
              ,l_abs_def_att_type_id as absence_type_id
			  ,CASE WHEN xxas_com_sec_pkg.test_instance('XXAS_ABS_VAC_RULE_SETUP',self.person_id) = 'Y' THEN
			    'Y'
			   END               as "VACATION_RULES_PERM"
              ,self.start_date   as startdate
              ,self.end_date     as enddate
              ,self.start_time   as starttime
              ,self.end_time     as endtime
              ,self.person_id    as employees
              ,self.manager_id   as manager_id
              ,cp_business_group_id business_group_id
              ,l_ol_absences_char ol_absences_li
              )
            )
    FROM   dual;

    CURSOR csr_tcm_period_id IS
    SELECT xtpa.period_id
    FROM   xxas_tcm_period_assignments_v xtpa
    WHERE  xtpa.person_id     = self.person_id
    AND    xtpa.assignment_id = self.resource_id
    AND    self.start_date BETWEEN xtpa.start_date
                           AND     xtpa.end_date
    ORDER BY xtpa.start_date DESC;

    CURSOR csr_tcm_event_xml(cp_business_group_id  IN number
                            ,cp_today_log_summary  IN varchar2
                            ,cp_total_log_summary  IN varchar2) IS
    SELECT XMLElement
            ("ROW"
            ,XMLForest
              (self.person_id as person_id
              ,self.source_block as source_block
              ,xxas_cal_util_pkg.source_block_called_view(self.source_block) as called_view
              ,self.saved_search_id as saved_search_id
              ,self.start_date  as date_start
              ,'DATE_START' as date_start_type
              ,self.start_date as dr_date_start
              ,l_date_range  as date_range
              ,cp_today_log_summary as today_logged_summary
              ,cp_total_log_summary as total_logged_summary
              ,to_char(self.start_date_time,'HH24:MI') time_start
              ,to_char(self.end_date_time,'HH24:MI') time_stop
              ,l_time_range     as time_range
              ,self.end_date    as date_end
              ,(CASE WHEN self.start_time = '00:00' AND self.end_time = '00:00'
                     THEN null
                     ELSE round(to_number(to_number(to_date(self.end_time,'HH24:MI:SS') - to_date(self.start_time,'HH24:MI:SS')) * 24),3)
                END) duration
              ,l_repeat_entry repeat_entry
              ,l_repeats repeats
              ,l_repeat_daily repeat_daily
              ,l_repeat_weekly repeat_weekly
              ,l_repeat_monthly repeat_monthly
              ,l_ends ends
              ,l_till_end_date till_end_date
              ,self.resource_id as assignment_id
              ,self.resource_id as job_id
              ,'COST'           as cost_all_ff_code
              ,xxas_cal_util_pkg.cost_all_key_ff_structure_code
                 (nvl(cp_business_group_id,to_number(fnd_profile.value('PER_BUSINESS_GROUP_ID')))
                 ,'COST'
                 ,xxas_util_pkg.get_app_id('PAY')) as cost_all_ff_st_code
              ,xxas_util_pkg.get_username_from_per(l_approver_person_id,trunc(sysdate)) override_approver
              ,l_ol_timecards_char ol_entries_li
              ,xxas_cal_util_pkg.summary_message
                 (self.start_date
                 ,l_repeat_entry
                 ,l_repeats
                 ,l_repeat_daily
                 ,l_repeat_weekly
                 ,l_repeat_monthly
                 ,l_ends
                 ,null
                 ,l_till_end_date
                 ,1) summary_msg
              )
            ,CASE WHEN xxas_cal_util_pkg.source_block_called_view(self.source_block) = 'TEAM' 
			 THEN
			   l_asg_data
			 END
            )
    FROM   dual;

    CURSOR csr_person IS
    SELECT papf.*
    FROM   per_all_people_f papf
    WHERE  papf.person_id = self.person_id
    AND    trunc(sysdate) BETWEEN papf.effective_start_date 
                          AND     papf.effective_end_date
    AND    (nvl(papf.current_employee_flag, 'N') = 'Y'
            OR nvl(papf.current_npw_flag, 'N') = 'Y');

    CURSOR csr_person_data IS
    SELECT XMLElement
            ("EMPLOYEES"
            ,XMLElement
               ("value"
               ,XMLAttributes
                 ('true' as "json:force-array"
                 ,'http://json.org/' as "xmlns:json")
               ,XMLAgg
                 (XMLElement
                   ("value"
                   ,XMLForest
                     (p.person_id "id"
                     ,p.short_name "label"
                     ,xxas_util_pkg.flag_to_boolean('Y') "deletable")
                   ,XMLElement
                     ("linked"
                     ,XMLElement
                       ("image"
                       ,XMLForest
                         (xxas_com_rest_util_pkg.uri_from_path
                            ('people/' || p.img_person_id || '/photo') "self"
                         ,p.initials "initials"
                         ,xxas_per_image_pkg.url
                            (p.img_person_id, p.img_last_update_date, 'SMALL') "small"
                         )
                       )
                     )
                   )
                 )
               ).getClobVal()
            )
    FROM  (SELECT papf.person_id
                 ,xxas_util_pkg.get_formatted_name
                    (papf.full_name
                    ,papf.global_name
                    ,papf.local_name
                    ,papf.first_name
                    ,papf.last_name
                    ,papf.known_as) short_name
                 ,xxas_util_pkg.get_initials(papf.first_name, papf.last_name, papf.known_as) initials
                 ,img.person_id              img_person_id
                 ,img.last_update_date       img_last_update_date
           FROM   per_all_people_f papf
                 ,xxas_per_thumbnails img
           WHERE  papf.person_id = self.person_id
           AND    self.start_date BETWEEN papf.effective_start_date 
                                  AND     papf.effective_end_date
           AND    papf.person_id = img.person_id(+)) p;

    CURSOR csr_otl_event_xml(cp_business_group_id IN number
                            ,cp_period_start_date IN date
                            ,cp_period_end_date   IN date) IS
    SELECT XMLElement
            ("ROW"
            ,XMLForest
              (self.person_id as person_id
              ,self.source_block as source_block
              ,xxas_cal_util_pkg.source_block_called_view(self.source_block) as called_view
              ,self.saved_search_id as saved_search_id
              ,self.start_date  as date_start
              ,to_char(self.start_date_time,'HH24:MI') time_start
              ,to_char(self.end_date_time,'HH24:MI') time_stop
              ,l_time_range     as time_range
              ,self.end_date    as date_end
              ,(CASE WHEN self.start_time = '00:00' AND self.end_time = '00:00'
                     THEN null
                     ELSE round(to_number(to_number(to_date(self.end_time,'HH24:MI:SS') - to_date(self.start_time,'HH24:MI:SS')) * 24),3)
                END) measure
              ,cp_period_start_date timecard_summary_start_date
              ,cp_period_end_date   timecard_summary_end_date
              ,l_repeat_entry repeat_entry
              ,l_repeats repeats
              ,l_repeat_daily repeat_daily
              ,l_repeat_weekly repeat_weekly
              ,l_repeat_monthly repeat_monthly
              ,l_ends ends
              ,l_till_end_date till_end_date
              ,self.resource_id as assignment_id
              ,'COST'           as cost_all_ff_code
              ,xxas_cal_util_pkg.cost_all_key_ff_structure_code
                 (nvl(cp_business_group_id,to_number(fnd_profile.value('PER_BUSINESS_GROUP_ID')))
                 ,'COST'
                 ,xxas_util_pkg.get_app_id('PAY')) as cost_all_ff_st_code
              ,l_ol_timecards_char ol_entries_li
              ,xxas_cal_util_pkg.summary_message
                 (self.start_date
                 ,l_repeat_entry
                 ,l_repeats
                 ,l_repeat_daily
                 ,l_repeat_weekly
                 ,l_repeat_monthly
                 ,l_ends
                 ,null
                 ,l_till_end_date
                 ,1) summary_msg
              )
            ,l_person_data
            )
    FROM   dual;

    l_per_row             csr_person%ROWTYPE;
    l_period_id           number;
    l_today_log_summary   varchar2(4000);
    l_total_log_summary   varchar2(4000);

  BEGIN

    IF self.action_id = 'ABSENCE' THEN

      OPEN  csr_person;
      FETCH csr_person INTO l_per_row;
      CLOSE csr_person;

      IF self.def_abs_type_flag = 'Y' THEN

         OPEN  csr_abs_def_att_type_id;
         FETCH csr_abs_def_att_type_id INTO l_abs_def_att_type_id;
         CLOSE csr_abs_def_att_type_id;

      END IF;
      
      l_ol_absences_char := to_char(self.person_id) || '_' || to_char(self.start_date_time,'YYYY-MM-DD HH24:MI:SS') 
                            || '_' || to_char(self.end_date_time,'YYYY-MM-DD HH24:MI:SS') || '_' || self.source_block
                            || '_' || self.saved_search_id;

      l_date_time_range  := to_char(self.start_date_time,'YYYY-MM-DD HH24:MI') || '_' || to_char(self.end_date_time,'YYYY-MM-DD HH24:MI');
      l_date_range       := to_char(self.start_date_time,'YYYY-MM-DD') || '_' || to_char(self.end_date_time,'YYYY-MM-DD');

      OPEN  csr_abs_cal_event_xml(l_per_row.business_group_id);
      FETCH csr_abs_cal_event_xml INTO l_xml;
      CLOSE csr_abs_cal_event_xml;

    ELSIF self.action_id = 'TIMECARD' THEN

      OPEN  csr_person;
      FETCH csr_person INTO l_per_row;
      CLOSE csr_person;

      OPEN  csr_approver_person;
      FETCH csr_approver_person INTO l_approver_person_id;
      CLOSE csr_approver_person;

      OPEN  csr_asg_data;
      FETCH csr_asg_data INTO l_asg_data;
      CLOSE csr_asg_data;

      IF self.start_date is not null AND
         self.end_date is not null THEN

        IF self.end_date = self.start_date + 1 THEN
          l_date_range := self.start_date || '_' || self.end_date;
        ELSE
          l_date_range := self.start_date || '_' || self.start_date;

          l_repeat_entry := CASE WHEN (self.end_date - self.start_date) = 0
                                 THEN 'N'
                                 ELSE 'Y'
                            END;

          l_repeats := CASE WHEN (self.end_date - self.start_date) = 0
                            THEN null
                            WHEN (self.end_date - self.start_date) <> 0
                            THEN 'D'
                            ELSE null
                       END;

          l_repeat_daily   := 1; 
          l_repeat_weekly  := 1;
          l_repeat_monthly := 1;

          l_ends := CASE WHEN (self.end_date - self.start_date) = 0
                         THEN null
                         WHEN (self.end_date - self.start_date) <> 0
                         THEN 'O'
                         ELSE null
                    END;

          l_till_end_date := CASE WHEN (self.end_date - self.start_date) = 0
                                  THEN null
                                  WHEN (self.end_date - self.start_date) <> 0
                                  THEN self.end_date
                                  ELSE null
                             END;
        END IF;

      ELSE

        l_repeat_entry := 'N';

      END IF;

      IF self.start_date_time is not null AND
         self.end_date_time   is not null THEN

        IF self.person_id is not null THEN

          l_ol_timecards_char := to_char(self.person_id) || '_' || to_char(self.start_date_time,'YYYY-MM-DD HH24:MI:SS') 
                                 || '_' || to_char(self.end_date_time,'YYYY-MM-DD HH24:MI:SS');

        END IF;
		
        l_time_range := to_char(self.start_date_time,'HH24:MI') || '_' || to_char(self.end_date_time,'HH24:MI');

      END IF;

      OPEN  csr_tcm_period_id;
      FETCH csr_tcm_period_id INTO l_period_id;
      CLOSE csr_tcm_period_id;

      l_today_log_summary := xxas_timecard_periods_pkg.asg_time_summary_log_on_date
                               (p_assignment_id => self.resource_id
                               ,p_logged_date   => self.start_date
                               ,p_asg_tbl       => null);

      l_total_log_summary := xxas_timecard_periods_pkg.asg_time_summary_log_on_period
                               (p_assignment_id => self.resource_id
                               ,p_logged_period => l_period_id
                               ,p_asg_tbl       => null);

      OPEN  csr_tcm_event_xml(l_per_row.business_group_id
                             ,l_today_log_summary
                             ,l_total_log_summary);
      FETCH csr_tcm_event_xml INTO l_xml;
      CLOSE csr_tcm_event_xml;

    ELSIF self.action_id = 'OTL' THEN

      OPEN  csr_person;
      FETCH csr_person INTO l_per_row;
      CLOSE csr_person;

      OPEN  csr_person_data;
      FETCH csr_person_data INTO l_person_data;
      CLOSE csr_person_data;

      IF self.start_date_time is not null AND
         self.end_date_time   is not null THEN

        IF self.person_id is not null THEN
          l_rec_period_id := hxc_preference_evaluation.resource_preferences(self.person_id,'TC_W_TCRD_PERIOD',1,trunc(self.start_date_time));
        END IF;

        l_periodtab.delete;

        IF l_rec_period_id is not null THEN

          hxc_timekeeper_utilities.get_resource_time_periods
            (p_resource_id           => self.person_id
            ,p_resource_type         => 'PERSON'
            ,p_current_date          => trunc(self.start_date_time)
            ,p_max_date_in_futur     => trunc(self.start_date_time) + 30
            ,p_max_date_in_past      => trunc(self.start_date_time) - 30
            ,p_recurring_period_id   => l_rec_period_id
            ,p_check_assignment	     => true
            ,p_periodtab             => l_periodtab);

          IF l_periodtab.count > 0 THEN
            l_rowindex := l_periodtab.first;
            LOOP
              EXIT WHEN NOT l_periodtab.exists(l_rowindex);
                l_period_start_date := l_periodtab(l_rowindex).start_date;
                l_period_end_date   := l_periodtab(l_rowindex).end_date;
                IF trunc(self.start_date_time) BETWEEN l_period_start_date 
                                               AND     l_period_end_date THEN
                  EXIT;
                END IF;
                l_rowindex := l_periodtab.next(l_rowindex);
            END LOOP;
          END IF;

        END IF;

        l_periodtab.delete;

        IF self.person_id is not null THEN

          l_ol_timecards_char := to_char(self.person_id) || '_' || to_char(self.start_date_time,'YYYY-MM-DD HH24:MI:SS') 
                                 || '_' || to_char(self.end_date_time,'YYYY-MM-DD HH24:MI:SS');

        END IF;

        l_time_range := to_char(self.start_date_time,'HH24:MI') || '_' || to_char(self.end_date_time,'HH24:MI');

        l_repeat_entry := CASE WHEN (self.end_date - self.start_date) = 0
                               THEN 'N'
                               ELSE 'Y'
                          END;

        l_repeats := CASE WHEN (self.end_date - self.start_date) = 0
                          THEN null
                          WHEN (self.end_date - self.start_date) <> 0
                          THEN 'D'
                          ELSE null
                     END;

        l_repeat_daily   := 1; 
        l_repeat_weekly  := 1;
        l_repeat_monthly := 1;

        l_ends := CASE WHEN (self.end_date - self.start_date) = 0
                       THEN null
                       WHEN (self.end_date - self.start_date) <> 0
                       THEN 'O'
                       ELSE null
                  END;

        l_till_end_date := CASE WHEN (self.end_date - self.start_date) = 0
                                THEN null
                                WHEN (self.end_date - self.start_date) <> 0
                                THEN self.end_date
                                ELSE null
                           END;

      ELSE

        l_repeat_entry := 'N';

      END IF;

      OPEN  csr_otl_event_xml(l_per_row.business_group_id
                             ,l_period_start_date
                             ,l_period_end_date);
      FETCH csr_otl_event_xml INTO l_xml;
      CLOSE csr_otl_event_xml;

    END IF;

    RETURN l_xml;

  END;

END;
/


CREATE OR REPLACE TYPE BODY xxas_cal_event_view IS

  CONSTRUCTOR FUNCTION xxas_cal_event_view
    (p_person_id           IN  number
    ,p_resource_id         IN  number
    ,p_abs_attendance_id   IN  number
    ,p_log_entry_id        IN  number
    ,p_transaction_id      IN  number
    ,p_transaction_step_id IN  number
    ,p_action_id           IN  varchar2
    ,p_source_block        IN  varchar2
    ,p_saved_search_id     IN  number
    ,p_manager_id          IN  number default null) RETURN SELF AS RESULT
  IS

  BEGIN

    self.person_id           :=  p_person_id;
    self.resource_id         :=  p_resource_id;
    self.abs_attendance_id   :=  p_abs_attendance_id;
    self.log_entry_id        :=  p_log_entry_id;
    self.transaction_id      :=  p_transaction_id;
    self.transaction_step_id :=  p_transaction_step_id;
    self.action_id           :=  p_action_id;
    self.source_block        :=  p_source_block;
    self.saved_search_id     :=  p_saved_search_id;
    self.manager_id          :=  p_manager_id;

    RETURN;

  END;

  MEMBER FUNCTION get_xml RETURN XMLType
  IS

    l_xml                  XMLType;
    l_leave_status         varchar2(100);
    l_date_time_range      varchar2(100);
    l_date_range           varchar2(100);
    l_time_range           varchar2(30);
    l_asg_data             XMLType;
    l_person_data          XMLType;
    l_field1_data_type     varchar2(100);
    l_field2_data_type     varchar2(100);
    l_field3_data_type     varchar2(100);
    l_field4_data_type     varchar2(100);
    l_field5_data_type     varchar2(100);
    l_timecard_summary_start_date  date;
    l_timecard_summary_end_date    date;
    l_timecard_status      varchar2(30);
    l_timecard_summary_id  number;
    l_timecard_ovn         number;
    l_timecard_summary_ovn number;
    l_timecard_date_from   varchar2(30);
    l_comments_to_approver varchar2(4000);

    CURSOR csr_abs_event_view_xml IS
    SELECT XMLElement
            ("ROW"
            ,XMLForest
              (paa.absence_attendance_id
              ,paa.object_version_number
              ,paa.business_group_id
              ,self.source_block source_block
              ,self.saved_search_id saved_search_id
              ,xxas_cal_util_pkg.source_block_called_view(self.source_block) as called_view
              ,null status
              ,l_leave_status leave_status
              ,paa.absence_attendance_type_id            absence_type_id
              ,paat.name                                 absence_type
              ,paat.absence_category                     absence_category_code
              ,fcl.meaning                               absence_category
              ,paa.abs_attendance_reason_id
              ,(SELECT rea.meaning     
                FROM   per_abs_attendance_reasons paar
                      ,hr_lookups                 rea
                WHERE  paa.abs_attendance_reason_id   = paar.abs_attendance_reason_id
                AND    paar.name                      = rea.lookup_code
                AND    rea.lookup_type                = 'ABSENCE_REASON') absence_reason 
              ,paa.person_id
              ,xxas_util_pkg.get_primary_worker_asg(paa.person_id,trunc(sysdate)) as assignment_id
              ,paa.authorising_person_id
              ,paa.replacement_person_id
              ,paa.period_of_incapacity_id
              ,paa.absence_days duration_days
              ,paa.absence_hours duration_hours
              ,paa.date_notification
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(paa.date_projected_start,'YYYY-MM-DD') || ' ' || paa.time_projected_start || '_' ||
                          to_char(paa.date_projected_end,'YYYY-MM-DD') || ' ' || paa.time_projected_end
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(paa.date_start,'YYYY-MM-DD') || ' ' || paa.time_start || '_' ||
                          to_char(paa.date_end,'YYYY-MM-DD') || ' ' || paa.time_end
                     ELSE null
                END) date_time_range
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(paa.date_projected_start,'YYYY-MM-DD') || '_' ||
                          to_char(paa.date_projected_end,'YYYY-MM-DD')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(paa.date_start,'YYYY-MM-DD') || '_' ||
                          to_char(paa.date_end,'YYYY-MM-DD')
                     ELSE null
                END) date_range
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(paa.date_projected_start,'YYYY-MM-DD')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(paa.date_start,'YYYY-MM-DD')
                     ELSE null
                END) startdate
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(paa.date_projected_end,'YYYY-MM-DD')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(paa.date_end,'YYYY-MM-DD')
                     ELSE null
                END) enddate
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN paa.time_projected_start
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN paa.time_start
                     ELSE null
                END) starttime
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN paa.time_projected_end
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN paa.time_end
                     ELSE null
                END) endtime
              ,paa.occurrence
              ,paa.ssp1_issued
              ,paa.attribute_category
              ,paa.attribute1
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE1'
                 ,paa.attribute1)                        attribute1_meaning 
              ,paa.attribute2
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE2'
                 ,paa.attribute2)                        attribute2_meaning 
              ,paa.attribute3
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE3'
                 ,paa.attribute3)                        attribute3_meaning 
              ,paa.attribute4
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE4'
                 ,paa.attribute4)                        attribute4_meaning 
              ,paa.attribute5
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE5'
                 ,paa.attribute5)                        attribute5_meaning 
              ,paa.attribute6
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE6'
                 ,paa.attribute6)                        attribute6_meaning 
              ,paa.attribute7
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE7'
                 ,paa.attribute7)                        attribute7_meaning 
              ,paa.attribute8
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE8'
                 ,paa.attribute8)                        attribute8_meaning 
              ,paa.attribute9
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE9'
                 ,paa.attribute9)                        attribute9_meaning 
              ,paa.attribute10
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE10'
                 ,paa.attribute10)                      attribute10_meaning 
              ,paa.attribute11
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE11'
                 ,paa.attribute11)                      attribute11_meaning 
              ,paa.attribute12
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE12'
                 ,paa.attribute12)                       attribute12_meaning 
              ,paa.attribute13
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE13'
                 ,paa.attribute13)                       attribute13_meaning 
              ,paa.attribute14
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE14'
                 ,paa.attribute14)                       attribute14_meaning 
              ,paa.attribute15
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE15'
                 ,paa.attribute15)                       attribute15_meaning 
              ,paa.attribute16
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE16'
                 ,paa.attribute16)                       attribute16_meaning 
              ,paa.attribute17
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE17'
                 ,paa.attribute17)                       attribute17_meaning 
              ,paa.attribute18
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE18'
                 ,paa.attribute18)                       attribute18_meaning 
              ,paa.attribute19
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE19'
                 ,paa.attribute19)                       attribute19_meaning 
              ,paa.attribute20
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE20'
                 ,paa.attribute20)                       attribute20_meaning 
              ,paa.last_update_date
              ,paa.last_updated_by
              ,paa.last_update_login
              ,paa.created_by
              ,paa.creation_date
              ,paa.maternity_id
              ,paa.sickness_start_date
              ,paa.sickness_end_date
              ,paa.pregnancy_related_illness 
              ,paa.reason_for_notification_delay 
              ,paa.accept_late_notification_flag
              ,paa.linked_absence_id
              ,paa.abs_information_category
              ,paa.abs_information1
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION1'
                 ,paa.abs_information1)                  abs_information1_meaning 
              ,paa.abs_information2
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION2'
                 ,paa.abs_information2)                  abs_information2_meaning
              ,paa.abs_information3
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION3'
                 ,paa.abs_information3)                  abs_information3_meaning
              ,paa.abs_information4
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION4'
                 ,paa.abs_information4)                  abs_information4_meaning
              ,paa.abs_information5
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION5'
                 ,paa.abs_information5)                  abs_information5_meaning
              ,paa.abs_information6
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION6'
                 ,paa.abs_information6)                  abs_information6_meaning
              ,paa.abs_information7
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION7'
                 ,paa.abs_information7)                  abs_information7_meaning
              ,paa.abs_information8
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION8'
                 ,paa.abs_information8)                  abs_information8_meaning
              ,paa.abs_information9
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION9'
                 ,paa.abs_information9)                  abs_information9_meaning
              ,paa.abs_information10
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION10'
                 ,paa.abs_information10)                 abs_information10_meaning
              ,paa.abs_information11
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION11'
                 ,paa.abs_information11)                 abs_information11_meaning
              ,paa.abs_information12
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION12'
                 ,paa.abs_information12)                 abs_information12_meaning
              ,paa.abs_information13
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION13'
                 ,paa.abs_information13)                 abs_information13_meaning
              ,paa.abs_information14
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION14'
                 ,paa.abs_information14)                 abs_information14_meaning
              ,paa.abs_information15
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION15'
                 ,paa.abs_information15)                 abs_information15_meaning
              ,paa.abs_information16
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION16'
                 ,paa.abs_information16)                 abs_information16_meaning
              ,paa.abs_information17
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION17'
                 ,paa.abs_information17)                 abs_information17_meaning
              ,paa.abs_information18
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION18'
                 ,paa.abs_information18)                 abs_information18_meaning
              ,paa.abs_information19
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION19'
                 ,paa.abs_information19)                 abs_information19_meaning
              ,paa.abs_information20
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION20'
                 ,paa.abs_information20)                 abs_information20_meaning
              ,paa.abs_information21
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION21'
                 ,paa.abs_information21)                 abs_information21_meaning
              ,paa.abs_information22
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION22'
                 ,paa.abs_information22)                 abs_information22_meaning
              ,paa.abs_information23
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION23'
                 ,paa.abs_information23)                 abs_information23_meaning
              ,paa.abs_information24
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION24'
                 ,paa.abs_information24)                 abs_information24_meaning
              ,paa.abs_information25
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION25'
                 ,paa.abs_information25)                 abs_information25_meaning
              ,paa.abs_information26
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION26'
                 ,paa.abs_information26)                 abs_information26_meaning
              ,paa.abs_information27
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION27'
                 ,paa.abs_information27)                 abs_information27_meaning
              ,paa.abs_information28
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION28'
                 ,paa.abs_information28)                 abs_information28_meaning
              ,paa.abs_information29
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION29'
                 ,paa.abs_information29)                 abs_information29_meaning
              ,paa.abs_information30
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION30'
                 ,paa.abs_information30)                 abs_information30_meaning
              ,paa.batch_id
              ,xxas_util_pkg.long_to_varchar2
                 ('PER_ABSENCE_ATTENDANCES'
		         ,'COMMENTS' 
		         ,paa.rowid)                          comments_to_approver    
              ,paa.absence_case_id
              ,paa.approval_status
              ,paa.confirmed_until
              ,paa.source
              ,paa.advance_pay
              ,null transaction_id
              ,null transaction_step_id
              ,self.person_id as employees
              ,self.manager_id as manager_id
              )
			,CASE WHEN xxas_com_sec_pkg.test_instance('XXAS_ABS_VAC_RULE_SETUP',paa.person_id) = 'Y' THEN
			   xxas_absence_event_pkg.get_vacation_rules_xml(paa.person_id,paa.date_start,paa.date_end)
			 END
            ,cast
                (multiset
                  (SELECT xxas_attachment_type
                            (attach.attached_document_id
                            ,attach.sequence_number
                            ,attach.entity_name
                            ,attach.pk1_value
                            ,attach.pk2_value
                            ,attach.pk3_value
                            ,attach.pk4_value
                            ,attach.pk5_value
                            ,attach.category_code
                            ,attach.category_name
                            ,attach.document_id
                            ,attach.file_name
                            ,attach.title
                            ,attach.description
                            ,attach.file_id
                            ,attach.mime_type
                            ,attach.lob_format
                            ,attach.file_url
                            ,attach.wp_link
                            ,attach.creation_date
                            ,attach.created_by
                            ,attach.last_update_date
                            ,attach.last_updated_by
                            )
                   FROM   TABLE(xxas_cal_util_pkg.attachments_data
                                  (p_attachment_id           => null
                                  ,p_entity_name             => 'PER_ABSENCE_ATTENDANCES'
                                  ,p_pk1_value               => to_char(paa.absence_attendance_id)
                                  ,p_pk2_value               => null
                                  ,p_pk3_value               => null
                                  ,p_pk4_value               => null
                                  ,p_pk5_value               => null
                                  ,p_transaction_id          => null
                                  ,p_transaction_entity_name => null)) attach
                  )
                 AS xxas_attachment_table
                ) attachments
            )
    FROM   per_absence_attendances      paa
          ,per_abs_attendance_types_vl  paat
          ,hr_lookups                   fcl
    WHERE  paa.absence_attendance_type_id  = paat.absence_attendance_type_id
    AND    paat.absence_category           = fcl.lookup_code(+)
    AND    fcl.lookup_type(+)              = 'ABSENCE_CATEGORY'
    AND    paa.person_id                   = self.person_id
    AND    paa.absence_attendance_id       = self.abs_attendance_id;

    CURSOR csr_abs_event_tran_in_view_xml IS
    SELECT XMLElement
            ("ROW"
            ,XMLForest
              (hat.selected_person_id as person_id
              ,xxas_util_pkg.get_primary_worker_asg(hat.selected_person_id,trunc(sysdate)) as assignment_id
              ,hat.assignment_id as override_assignment_id
              ,hat.transaction_ref_id as absence_attendance_id
              ,hat.transaction_id as transaction_id
              ,hats.transaction_step_id as transaction_step_id
              ,self.source_block as source_block
              ,self.saved_search_id as saved_search_id
              ,xxas_cal_util_pkg.source_block_called_view(self.source_block) as called_view
              ,decode(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAction', 'Transaction/TransCtx/CNode'), 'CreateMode', 'INSERT', 'UpdateMode', 'UPDATE', 'CancelMode', 'DELETE', 'INSERT')  transaction_type
              ,hat.status as hr_api_status_code
              ,xxas_selfservice_workflow_pkg.transcode_status
                 (hat.status) as workflow_status_code
              ,xxas_selfservice_workflow_pkg.decode_status
                 (xxas_selfservice_workflow_pkg.transcode_status
                    (hat.status)) as workflow_status_meaning
              ,l_leave_status leave_status
              ,to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'BusinessGroupId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')) business_group_id
              ,to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceTypeId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')) absence_type_id
              ,(SELECT paat.name
                FROM   per_abs_attendance_types_vl paat
                WHERE  paat.absence_attendance_type_id = to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceTypeId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))) absence_type
              ,to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsAttendanceReasonId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')) abs_attendance_reason_id
             ,(SELECT rea.meaning     
               FROM   per_abs_attendance_reasons paar
                     ,hr_lookups                 rea
               WHERE  paar.abs_attendance_reason_id = to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsAttendanceReasonId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
               AND    paar.name                     = rea.lookup_code
               AND    rea.lookup_type               = 'ABSENCE_REASON') abs_attendance_reason
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')),'YYYY-MM-DD') || ' ' || xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') || '_' ||
                          to_char(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')),'YYYY-MM-DD') || ' ' || xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')),'YYYY-MM-DD') || ' ' || xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') || '_' ||
                          to_char(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')),'YYYY-MM-DD') || ' ' || xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                     ELSE null
                END) date_time_range
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')),'YYYY-MM-DD') || '_' ||
                          to_char(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')),'YYYY-MM-DD')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')),'YYYY-MM-DD') || '_' ||
                          to_char(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')),'YYYY-MM-DD')
                     ELSE null
                END) date_range
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')),'YYYY-MM-DD')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')),'YYYY-MM-DD')
                     ELSE null
                END) startdate
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')),'YYYY-MM-DD')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')),'YYYY-MM-DD')
                     ELSE null
                END) enddate
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                     ELSE null
                END) starttime
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                     ELSE null
                END) endtime
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateNotification', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') date_notification
              ,(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceDays', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')) duration_days
              ,(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceHours', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')) duration_hours
              ,(SELECT paat.absence_category
                FROM   per_abs_attendance_types_vl paat
                WHERE  paat.absence_attendance_type_id = to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceTypeId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))) absence_category_code
             ,(SELECT fcl.meaning
               FROM   per_abs_attendance_types_vl paat
                     ,hr_lookups fcl
               WHERE  paat.absence_attendance_type_id = to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceTypeId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
               AND    paat.absence_category    = fcl.lookup_code(+)
               AND    fcl.lookup_type(+)       = 'ABSENCE_CATEGORY') absence_category
              ,to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'ObjectVersionNumber', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')) object_version_number
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute_category
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute1', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute1
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE1'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute1', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')) attribute1_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute2', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute2
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE2'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute2', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                    attribute2_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute3', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute3
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE3'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute3', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                    attribute3_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute4', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute4
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE4'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute4', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                    attribute4_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute5', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute5
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE5'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute5', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                    attribute5_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute6', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute6
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE6'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute6', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                    attribute6_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute7', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute7
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE7'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute7', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                    attribute7_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute8', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute8
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE8'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute8', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                    attribute8_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute9', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute9
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE9'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute9', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                    attribute9_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute10', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute10
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE10'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute10', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute10_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute11', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute11
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE11'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute11', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute11_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute12', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute12
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE12'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute12', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute12_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute13', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute13
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE13'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute13', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute13_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute14', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute14
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE14'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute14', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute14_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute15', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute15
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE15'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute15', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute15_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute16', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute16
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE16'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute16', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute16_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute17', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute17
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE17'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute17', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute17_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute18', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute18
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE18'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute18', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute18_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute19', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute19
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE19'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute19', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute19_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute20', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute20
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE20'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute20', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute20_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information_category
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation1', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information1
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 (hat.selected_person_id
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION1'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation1', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information1_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation2', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information2
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION2'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation2', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information2_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation3', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information3
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION3'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation3', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information3_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation4', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information4
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION4'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation4', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information4_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation5', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information5
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION5'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation5', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information5_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation6', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information6
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION6'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation6', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information6_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation7', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information7
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION7'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation7', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information7_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation8', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information8
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION8'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation8', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information8_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation9', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information9
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION9'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation9', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information9_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation10', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information10
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION10'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation10', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information10_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation11', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information11
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION11'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation11', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information11_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation12', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information12
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION12'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation12', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information12_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation13', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information13
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION13'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation13', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information13_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation14', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information14
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION14'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation14', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information14_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation15', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information15
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION15'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation15', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information15_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation16', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information16
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION16'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation16', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information16_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation17', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information17
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION17'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation17', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information17_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation18', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information18
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION18'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation18', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information18_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation19', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information19
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION19'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation19', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information19_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation20', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information20
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION20'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation20', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information20_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation21', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information21
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION21'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation21', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information21_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation22', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information22
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION22'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation22', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information22_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation23', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information23
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION23'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation23', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information23_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation24', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information24
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION24'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation24', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information24_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation25', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information25
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION25'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation25', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information25_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation26', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information26
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION26'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation26', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information26_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation27', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information27
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION27'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation27', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information27_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation28', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information28
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION28'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation28', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information28_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation29', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information29
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION29'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation29', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information29_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation30', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information30
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION30'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation30', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information30_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Comments', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') comments_to_approver
              ,null                                      authorising_person_id
              ,to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'ReplacementPersonId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')) replacement_person_id
              ,null                                      period_of_incapacity_id
              ,null                                      occurrence
              ,null                                      ssp1_issued 
              ,null                                      last_update_date
              ,null                                      last_updated_by
              ,null                                      last_update_login
              ,null                                      created_by
              ,null                                      creation_date
              ,null                                      maternity_id
              ,null                                      sickness_start_date
              ,null                                      sickness_end_date
              ,null                                      pregnancy_related_illness 
              ,null                                      reason_for_notification_delay 
              ,null                                      accept_late_notification_flag
              ,null                                      linked_absence_id
              ,null                                      batch_id
              ,null                                      comments       
              ,null                                      absence_case_id
              ,null                                      approval_status
              ,null                                      confirmed_until
              ,null                                      source
              ,null                                      advance_pay
              ,hat.transaction_id approvers
              ,self.person_id as employees
              ,self.manager_id as manager_id
              )
			,CASE WHEN xxas_com_sec_pkg.test_instance('XXAS_ABS_VAC_RULE_SETUP',hat.selected_person_id) = 'Y' THEN
			   XMLForest('Y' as "VACATION_RULES_PERM"
			            ,hr_transaction_api.get_varchar2_value(hats.transaction_step_id,'P_SETUP_VAC_RULE') as "REDIRECT_NOTIF"
			            ,hr_transaction_api.get_varchar2_value(hats.transaction_step_id,'P_RULE') as "RULE"
						,hr_transaction_api.get_varchar2_value(hats.transaction_step_id,'P_ASSIGNEE') as "GRANTEE")
			 END
            ,cast
                (multiset
                  (SELECT xxas_attachment_type
                            (attach.attached_document_id
                            ,attach.sequence_number
                            ,attach.entity_name
                            ,attach.pk1_value
                            ,attach.pk2_value
                            ,attach.pk3_value
                            ,attach.pk4_value
                            ,attach.pk5_value
                            ,attach.category_code
                            ,attach.category_name
                            ,attach.document_id
                            ,attach.file_name
                            ,attach.title
                            ,attach.description
                            ,attach.file_id
                            ,attach.mime_type
                            ,attach.lob_format
                            ,attach.file_url
                            ,attach.wp_link
                            ,attach.creation_date
                            ,attach.created_by
                            ,attach.last_update_date
                            ,attach.last_updated_by
                            )
                   FROM   TABLE(xxas_cal_util_pkg.attachments_data
                                  (p_attachment_id           => null
                                  ,p_entity_name             => 'PER_ABSENCE_ATTENDANCES'
                                  ,p_pk1_value               => to_char(hat.transaction_ref_id || '_' || hat.transaction_id)
                                  ,p_pk2_value               => null
                                  ,p_pk3_value               => null
                                  ,p_pk4_value               => null
                                  ,p_pk5_value               => null
                                  ,p_transaction_id          => hat.transaction_id
                                  ,p_transaction_entity_name => 'PQH_SS_ATTACHMENT')) attach
                  )
                 AS xxas_attachment_table
                ) attachments
            )
    FROM   hr_api_transactions         hat
          ,hr_api_transaction_steps    hats
    WHERE  hat.transaction_id       = hats.transaction_id
    AND    hat.status          NOT IN ('D', 'N', 'W', 'ACTIVE', 'AC')
    AND    hats.api_name            = 'HR_PERSON_ABSENCE_SWI.PROCESS_API'
    AND    decode(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAction', 'Transaction/TransCtx/CNode'), 'CreateMode', 'INSERT') = 'INSERT'
    AND    hat.selected_person_id   = self.person_id
    AND    hat.transaction_id       = self.transaction_id
    AND    hats.transaction_step_id = self.transaction_step_id
    AND    hat.transaction_ref_id   = self.abs_attendance_id;

    CURSOR csr_abs_event_tran_view_xml IS
    SELECT XMLElement
            ("ROW"
            ,XMLForest
              (paa.person_id
              ,xxas_util_pkg.get_primary_worker_asg(paa.person_id,trunc(sysdate)) as assignment_id
              ,hat.assignment_id as override_assignment_id
              ,hat.transaction_ref_id absence_attendance_id
              ,hat.transaction_id transaction_id
              ,hats.transaction_step_id transaction_step_id
              ,self.source_block source_block
              ,self.saved_search_id saved_search_id
              ,xxas_cal_util_pkg.source_block_called_view(self.source_block) as called_view
              ,decode(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAction', 'Transaction/TransCtx/CNode'), 'CreateMode', 'INSERT', 'UpdateMode', 'UPDATE', 'CancelMode', 'DELETE', 'INSERT') transaction_type
              ,hat.status hr_api_status_code
              ,xxas_selfservice_workflow_pkg.transcode_status(hat.status) workflow_status_code
              ,xxas_selfservice_workflow_pkg.decode_status
                 (xxas_selfservice_workflow_pkg.transcode_status(hat.status)) workflow_status_meaning
              ,l_leave_status leave_status
              ,nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'BusinessGroupId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                  ,paa.business_group_id) business_group_id
              ,nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceTypeId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                  ,paa.absence_attendance_type_id) absence_type_id
              ,(SELECT paat.name
                FROM   per_abs_attendance_types_vl paat
                WHERE  paat.absence_attendance_type_id = nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceTypeId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                                            ,paa.absence_attendance_type_id)) absence_type
              ,nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsAttendanceReasonId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                  ,paa.abs_attendance_reason_id) abs_attendance_reason_id
              ,(SELECT rea.meaning     
                FROM   per_abs_attendance_reasons paar
                      ,hr_lookups                 rea
                WHERE  paar.abs_attendance_reason_id = nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsAttendanceReasonId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                                          ,paa.abs_attendance_reason_id)
                AND    paar.name                     = rea.lookup_code
                AND    rea.lookup_type               = 'ABSENCE_REASON') abs_attendance_reason
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,paa.date_projected_start),'YYYY-MM-DD') || ' ' || nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                                                                                           ,paa.time_projected_start) || '_' ||
                          to_char(nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,paa.date_projected_end),'YYYY-MM-DD') || ' ' || nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                                                                                         ,paa.time_projected_end)
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,paa.date_start),'YYYY-MM-DD') || ' ' || nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                                                                                 ,paa.time_start) || '_' ||
                          to_char(nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,paa.date_end),'YYYY-MM-DD') || ' ' || nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                                                                               ,paa.time_end)
                     ELSE null
                END) date_time_range
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,paa.date_projected_start),'YYYY-MM-DD') || '_' ||
                          to_char(nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,paa.date_projected_end),'YYYY-MM-DD')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,paa.date_start),'YYYY-MM-DD') || '_' ||
                          to_char(nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,paa.date_end),'YYYY-MM-DD')
                     ELSE null
                END) date_range
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,paa.date_projected_start),'YYYY-MM-DD')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,paa.date_start),'YYYY-MM-DD')
                     ELSE null
                END) startdate
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,paa.date_projected_end),'YYYY-MM-DD')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,paa.date_end),'YYYY-MM-DD')
                     ELSE null
                END) enddate
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                             ,paa.time_projected_start)
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                             ,paa.time_start)
                     ELSE null
                END) starttime
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                             ,paa.time_projected_end)
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                             ,paa.time_end)
                     ELSE null
                END) endtime
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateNotification', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                  ,paa.date_notification) date_notification
              ,nvl((xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceDays', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                  ,paa.absence_days) duration_days
              ,nvl((xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceHours', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                  ,paa.absence_hours) duration_hours
              ,(SELECT paat.absence_category
                FROM   per_abs_attendance_types_vl paat
                WHERE  paat.absence_attendance_type_id = nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceTypeId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                                            ,paa.absence_attendance_type_id)) absence_category_code
              ,(SELECT fcl.meaning
                FROM   per_abs_attendance_types_vl paat
                      ,hr_lookups fcl
                WHERE  paat.absence_attendance_type_id = nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceTypeId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                                            ,paa.absence_attendance_type_id)
                AND    paat.absence_category           = fcl.lookup_code(+)
                AND    fcl.lookup_type(+)              = 'ABSENCE_CATEGORY') absence_category
              ,nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'ObjectVersionNumber', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                  ,paa.object_version_number) object_version_number
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category) attribute_category
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute1', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute1) attribute1
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE1'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute1', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute1)) attribute1_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute2', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute2) attribute2
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE2'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute2', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute2)) attribute2_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute3', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute3) attribute3
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE3'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute3', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute3)) attribute3_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute4', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute4) attribute4
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE4'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute4', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute4)) attribute4_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute5', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute5) attribute5
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE5'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute5', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute5)) attribute5_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute6', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute6) attribute6
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE6'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute6', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute6)) attribute6_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute7', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute7) attribute7
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE7'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute7', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute7)) attribute7_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute8', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute8) attribute8
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE8'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute8', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute8)) attribute8_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute9', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute9) attribute9
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE9'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute9', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute9)) attribute9_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute10', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute10) attribute10
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE10'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute10', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute10)) attribute10_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute11', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute11) attribute11
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE11'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute11', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute11)) attribute11_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute12', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute12) attribute12
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE12'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute12', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute12)) attribute12_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute13', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute13) attribute13
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE13'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute13', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute13)) attribute13_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute14', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute14) attribute14
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE14'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute14', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute14)) attribute14_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute15', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute15) attribute15
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE15'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute15', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute15)) attribute15_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute16', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute16) attribute16
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE16'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute16', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute16)) attribute16_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute17', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute17) attribute17
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE17'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute17', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute17)) attribute17_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute18', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute18) attribute18
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE18'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute18', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute18)) attribute18_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute19', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute19) attribute19
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE19'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute19', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute19)) attribute19_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute20', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute20) attribute20
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE20'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute20', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute20)) attribute20_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category) abs_information_category
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation1', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information1) abs_information1
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION1'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation1', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information1)) abs_information1_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation2', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information2) abs_information2
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION2'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation2', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information2)) abs_information2_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation3', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information3) abs_information3
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION3'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation3', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information3)) abs_information3_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation4', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information4) abs_information4
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION4'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation4', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information4)) abs_information4_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation5', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information5) abs_information5
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION5'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation5', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information5)) abs_information5_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation6', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information6) abs_information6
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION6'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation6', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information6)) abs_information6_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation7', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information7) abs_information7
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION7'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation7', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information7)) abs_information7_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation8', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information8) abs_information8
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION8'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation8', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information8)) abs_information8_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation9', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information9) abs_information9
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION9'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation9', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information9)) abs_information9_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation10', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information10) abs_information10
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION10'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation10', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information10)) abs_information10_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation11', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information11) abs_information11
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION11'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation11', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information11)) abs_information11_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation12', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information12) abs_information12
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION12'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation12', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information12)) abs_information12_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation13', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information13) abs_information13
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION13'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation13', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information13)) abs_information13_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation14', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information14) abs_information14
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION14'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation14', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information14)) abs_information14_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation15', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information15) abs_information15
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION15'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation15', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information15)) abs_information15_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation16', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information16) abs_information16
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION16'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation16', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information16)) abs_information16_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation17', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information17) abs_information17
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION17'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation17', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information17)) abs_information17_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation18', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information18) abs_information18
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION18'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation18', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information18)) abs_information18_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation19', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information19) abs_information19
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION19'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation19', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information19)) abs_information19_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation20', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information20) abs_information20
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION20'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation20', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information20)) abs_information20_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation21', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information21) abs_information21
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION21'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation21', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information21)) abs_information21_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation22', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information22) abs_information22
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION22'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation22', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information22)) abs_information22_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation23', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information23) abs_information23
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION23'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation23', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information23)) abs_information23_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation24', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information24) abs_information24
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION24'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation24', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information24)) abs_information24_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation25', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information25) abs_information25
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION25'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation25', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information25)) abs_information25_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation26', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information26) abs_information26
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION26'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation26', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information26)) abs_information26_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation27', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information27) abs_information27
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION27'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation27', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information27)) abs_information27_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation28', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information28) abs_information28
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION28'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation28', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information28)) abs_information28_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation29', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information29) abs_information29
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION29'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation29', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information29)) abs_information29_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation30', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information30) abs_information30
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION30'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation30', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information30)) abs_information30_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Comments', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') comments_to_approver
              ,null                                      authorising_person_id
              ,nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'ReplacementPersonId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                  ,paa.replacement_person_id) replacement_person_id
              ,null                                      period_of_incapacity_id
              ,null                                      occurrence
              ,null                                      ssp1_issued 
              ,null                                      last_update_date
              ,null                                      last_updated_by
              ,null                                      last_update_login
              ,null                                      created_by
              ,null                                      creation_date
              ,null                                      maternity_id
              ,null                                      sickness_start_date
              ,null                                      sickness_end_date
              ,null                                      pregnancy_related_illness 
              ,null                                      reason_for_notification_delay 
              ,null                                      accept_late_notification_flag
              ,null                                      linked_absence_id
              ,null                                      batch_id
              ,null                                      comments       
              ,null                                      absence_case_id
              ,null                                      approval_status
              ,null                                      confirmed_until
              ,null                                      source
              ,null                                      advance_pay
              ,hat.transaction_id approvers
              ,self.person_id as employees
              ,self.manager_id as manager_id
              )
			,CASE WHEN xxas_com_sec_pkg.test_instance('XXAS_ABS_VAC_RULE_SETUP',paa.person_id) = 'Y' THEN
			   xxas_absence_event_pkg.get_vacation_rules_xml(paa.person_id,paa.date_start,paa.date_end)
			 END
            ,cast
                (multiset
                  (SELECT xxas_attachment_type
                            (attach.attached_document_id
                            ,attach.sequence_number
                            ,attach.entity_name
                            ,attach.pk1_value
                            ,attach.pk2_value
                            ,attach.pk3_value
                            ,attach.pk4_value
                            ,attach.pk5_value
                            ,attach.category_code
                            ,attach.category_name
                            ,attach.document_id
                            ,attach.file_name
                            ,attach.title
                            ,attach.description
                            ,attach.file_id
                            ,attach.mime_type
                            ,attach.lob_format
                            ,attach.file_url
                            ,attach.wp_link
                            ,attach.creation_date
                            ,attach.created_by
                            ,attach.last_update_date
                            ,attach.last_updated_by
                            )
                   FROM   TABLE(xxas_cal_util_pkg.attachments_data
                                  (p_attachment_id           => null
                                  ,p_entity_name             => 'PER_ABSENCE_ATTENDANCES'
                                  ,p_pk1_value               => to_char(paa.absence_attendance_id || '_' || hat.transaction_id)
                                  ,p_pk2_value               => null
                                  ,p_pk3_value               => null
                                  ,p_pk4_value               => null
                                  ,p_pk5_value               => null
                                  ,p_transaction_id          => hat.transaction_id
                                  ,p_transaction_entity_name => 'PQH_SS_ATTACHMENT')) attach
                  )
                 AS xxas_attachment_table
                ) attachments
            )
    FROM   per_absence_attendances      paa
          ,per_abs_attendance_types_vl  paat
          ,hr_lookups                   fcl
          ,hr_api_transactions          hat
          ,hr_api_transaction_steps     hats
    WHERE  paa.absence_attendance_type_id  = paat.absence_attendance_type_id
    AND    paat.absence_category           = fcl.lookup_code(+)
    AND    fcl.lookup_type(+)              = 'ABSENCE_CATEGORY'
    AND    paa.person_id                   = self.person_id
    AND    paa.absence_attendance_id       = self.abs_attendance_id
    AND    paa.person_id                   = hat.selected_person_id
	AND    hat.transaction_id              = self.transaction_id
    AND    hat.status          NOT IN ('D', 'N', 'W', 'ACTIVE', 'AC')
    AND    hat.transaction_id              = hats.transaction_id
    AND    hats.transaction_step_id        = self.transaction_step_id
    AND    hats.api_name                   = 'HR_PERSON_ABSENCE_SWI.PROCESS_API';
    /*AND    paa.absence_attendance_id       = xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')*/

    CURSOR csr_asg_data IS
    SELECT XMLElement
            ("EMPLOYEES"
            ,XMLElement
               ("value"
               ,XMLAttributes
                 ('true' as "json:force-array"
                 ,'http://json.org/' as "xmlns:json")
               ,XMLAgg
                 (XMLElement
                   ("value"
                   ,XMLForest
                     (p.assignment_id "id"
                     --,p.short_name "value"
                     ,p.short_name "label"
                     ,xxas_util_pkg.flag_to_boolean('N') "deletable")
                   ,XMLElement
                     ("linked"
                     ,XMLElement
                       ("image"
                       ,XMLForest
                         (xxas_com_rest_util_pkg.uri_from_path
                            ('people/' || p.img_person_id || '/photo') "self"
                         ,p.initials "initials"
                         ,xxas_per_image_pkg.url
                            (p.img_person_id, p.img_last_update_date, 'SMALL') "small"
                         )
                       )
                     )
                   )
                 )
               ).getClobVal()
            )
    FROM  (SELECT paaf.assignment_id
                 ,xxas_util_pkg.get_formatted_name
                    (papf.full_name
                    ,papf.global_name
                    ,papf.local_name
                    ,papf.first_name
                    ,papf.last_name
                    ,papf.known_as) short_name
                 ,xxas_util_pkg.get_initials(papf.first_name, papf.last_name, papf.known_as) initials
                 ,img.person_id              img_person_id
                 ,img.last_update_date       img_last_update_date
           FROM   per_all_people_f papf
                 ,per_all_assignments_f paaf
                 ,xxas_per_thumbnails img
                 ,xxas_tcm_log_entry_approvers_v xtla
           WHERE  paaf.assignment_id = self.resource_id
           AND    paaf.person_id     = papf.person_id
           AND    trunc(sysdate) BETWEEN papf.effective_start_date 
                                 AND     papf.effective_end_date
           AND    xtla.log_entry_id  = self.log_entry_id
           AND    xtla.assignment_id = self.resource_id
           AND    trunc(xtla.start_date_time) BETWEEN paaf.effective_start_date
                                              AND     paaf.effective_end_date
           AND    papf.person_id = img.person_id(+)) p;

  CURSOR csr_time_entry_values(cp_assignment_id IN number
                              ,cp_log_entry_id  IN number) IS
  SELECT trunc(xtle.start_date_time) date_start
        ,xtle.element_type_id
  FROM   xxas_tcm_log_entries xtle
  WHERE  xtle.log_entry_id  = cp_log_entry_id
  AND    xtle.assignment_id = cp_assignment_id;

  CURSOR csr_field_values(cp_assignment_id   IN number
                         ,cp_effective_date  IN date
                         ,cp_element_type_id IN number) IS
  SELECT xxas_com_elements_pkg.get_iv_uom
           (eit.eei_information10, cp_effective_date) field1_datatype
        ,xxas_com_elements_pkg.get_iv_uom
           (eit.eei_information11, cp_effective_date) field2_datatype
        ,xxas_com_elements_pkg.get_iv_uom
           (eit.eei_information12, cp_effective_date) field3_datatype
        ,xxas_com_elements_pkg.get_iv_uom
           (eit.eei_information13, cp_effective_date) field4_datatype
        ,xxas_com_elements_pkg.get_iv_uom
           (eit.eei_information14, cp_effective_date) field5_datatype
  FROM   pay_element_types_f pet
        ,TABLE(xxas_com_elements_pkg.links_tbl(cp_assignment_id,cp_effective_date)) e
        ,pay_element_type_extra_info eit
  WHERE  pet.element_type_id = cp_element_type_id
  AND    pet.element_type_id = e.element_type_id
  AND    e.element_type_id   = eit.element_type_id
  AND    eit.information_type = 'XXAS_TCM_ENTRY_MAP'
  AND    eit.eei_information1 is not null
  AND    eit.eei_information1 = 'Y'
  AND    ((nvl(eit.eei_information20,'N') = 'N' AND nvl(eit.eei_information21,'N') = 'N')
          OR ((nvl(eit.eei_information20,'N') = 'Y' OR nvl(eit.eei_information21,'N') = 'Y')
              AND fnd_global.employee_id not in (SELECT person_id 
                                                 FROM   per_all_assignments_f 
                                                 WHERE  assignment_id = cp_assignment_id)
             )
         )
  AND    cp_effective_date BETWEEN pet.effective_start_date
                           AND     pet.effective_end_date;

    CURSOR csr_tcm_period_id IS
    SELECT xtle.period_id
    FROM   xxas_tcm_log_entry_states_v xtle
    WHERE  xtle.person_id     = self.person_id
    AND    xtle.assignment_id = self.resource_id
    AND    xtle.log_entry_id  = self.log_entry_id;

    CURSOR csr_tcm_event_view_xml(cp_business_group_id  IN number
                                 ,cp_today_log_summary  IN varchar2
                                 ,cp_total_log_summary  IN varchar2) IS
    SELECT XMLElement
            ("ROW"
            ,XMLForest
              (self.person_id person_id
              ,self.resource_id assignment_id
              ,self.log_entry_id log_entry_id
              ,self.source_block source_block
              ,self.saved_search_id saved_search_id
              ,self.log_entry_id approvers
              ,xtla.current_state_code
              ,xtla.current_ins_upd_del
              ,(CASE WHEN xtla.current_state_code = 'A' AND xtla.current_ins_upd_del = 'D'
                     THEN xxas_com_msg_pkg.get_message(xxas_util_pkg.app,'XXAS_DEL_APR')
                     ELSE NULL
                END) STATUS
              ,xxas_cal_util_pkg.costable_type(xtla.assignment_id
                                              ,nvl(trunc(xtla.start_date_time),trunc(sysdate))
                                              ,xtla.element_type_id) costable_type
              ,xxas_cal_util_pkg.source_block_called_view(self.source_block) as called_view
              ,self.resource_id as job_id
              ,trunc(xtla.start_date_time) date_start
              ,'DATE_START' as date_start_type
              ,trunc(xtla.start_date_time) as dr_date_start
              ,trunc(xtla.start_date_time) || '_' || trunc(xtla.end_date_time) as date_range
              ,cp_today_log_summary as today_logged_summary
              ,cp_total_log_summary as total_logged_summary
              ,trunc(xtla.end_date_time) date_end
              ,xtla.element_type_id payment_type_id
              ,to_char(xtla.start_date_time,'HH24:MI') time_start
              ,to_char(xtla.end_date_time,'HH24:MI') time_stop
              ,to_char(xtla.start_date_time,'HH24:MI') || '_' || to_char(xtla.end_date_time,'HH24:MI') time_range
              ,xtla.duration_hours duration
              ,(CASE WHEN l_field1_data_type IN ('H_HHMM','H_HHMMSS') AND xtla.field1 is not null 
                     THEN to_char(trunc(sysdate) + xtla.field1/24, 'HH24:MI')
                     ELSE xtla.field1
                END) field1
              ,(CASE WHEN l_field2_data_type IN ('H_HHMM','H_HHMMSS') AND xtla.field2 is not null 
                     THEN to_char(trunc(sysdate) + xtla.field2/24, 'HH24:MI')
                     ELSE xtla.field2
                END) field2
              ,(CASE WHEN l_field3_data_type IN ('H_HHMM','H_HHMMSS') AND xtla.field3 is not null
                     THEN to_char(trunc(sysdate) + xtla.field3/24, 'HH24:MI')
                     ELSE xtla.field3
                END) field3
              ,(CASE WHEN l_field4_data_type IN ('H_HHMM','H_HHMMSS') AND xtla.field4 is not null 
                     THEN to_char(trunc(sysdate) + xtla.field4/24, 'HH24:MI')
                     ELSE xtla.field4
                END) field4
              ,(CASE WHEN l_field5_data_type IN ('H_HHMM','H_HHMMSS') AND xtla.field5 is not null 
                     THEN to_char(trunc(sysdate) + xtla.field5/24, 'HH24:MI')
                     ELSE xtla.field5
                END) field5
              ,xtla.override_approver_role override_approver
              ,xtla.comments comments_to_approver
              ,'COST' cost_all_ff_code
              ,xxas_cal_util_pkg.cost_all_key_ff_structure_code
                 (nvl(cp_business_group_id,to_number(fnd_profile.value('PER_BUSINESS_GROUP_ID')))
                 ,'COST'
                 ,xxas_util_pkg.get_app_id('PAY')) cost_all_ff_st_code
              ,xtla.cost_allocation_keyflex_id
              ,ckf.segment1
              ,ckf.segment1_value
              ,ckf.segment1_meaning
              ,ckf.segment2
              ,ckf.segment2_value
              ,ckf.segment2_meaning
              ,ckf.segment3
              ,ckf.segment3_value
              ,ckf.segment3_meaning
              ,ckf.segment4
              ,ckf.segment4_value
              ,ckf.segment4_meaning
              ,ckf.segment5
              ,ckf.segment5_value
              ,ckf.segment5_meaning
              ,ckf.segment6
              ,ckf.segment6_value
              ,ckf.segment6_meaning
              ,ckf.segment7
              ,ckf.segment7_value
              ,ckf.segment7_meaning
              ,ckf.segment8
              ,ckf.segment8_value
              ,ckf.segment8_meaning
              ,ckf.segment9
              ,ckf.segment9_value
              ,ckf.segment9_meaning
              ,ckf.segment10
              ,ckf.segment10_value
              ,ckf.segment10_meaning
              ,ckf.segment11
              ,ckf.segment11_value
              ,ckf.segment11_meaning
              ,ckf.segment12
              ,ckf.segment12_value
              ,ckf.segment12_meaning
              ,ckf.segment13
              ,ckf.segment13_value
              ,ckf.segment13_meaning
              ,ckf.segment14
              ,ckf.segment14_value
              ,ckf.segment14_meaning
              ,ckf.segment15
              ,ckf.segment15_value
              ,ckf.segment15_meaning
              ,ckf.segment16
              ,ckf.segment16_value
              ,ckf.segment16_meaning
              ,ckf.segment17
              ,ckf.segment17_value
              ,ckf.segment17_meaning
              ,ckf.segment18
              ,ckf.segment18_value
              ,ckf.segment18_meaning
              ,ckf.segment19
              ,ckf.segment19_value
              ,ckf.segment19_meaning
              ,ckf.segment20
              ,ckf.segment20_value
              ,ckf.segment20_meaning
              ,ckf.segment21
              ,ckf.segment21_value
              ,ckf.segment21_meaning
              ,ckf.segment22
              ,ckf.segment22_value
              ,ckf.segment22_meaning
              ,ckf.segment23
              ,ckf.segment23_value
              ,ckf.segment23_meaning
              ,ckf.segment24
              ,ckf.segment24_value
              ,ckf.segment24_meaning
              ,ckf.segment25
              ,ckf.segment25_value
              ,ckf.segment25_meaning
              ,ckf.segment26
              ,ckf.segment26_value
              ,ckf.segment26_meaning
              ,ckf.segment27
              ,ckf.segment27_value
              ,ckf.segment27_meaning
              ,ckf.segment28
              ,ckf.segment28_value
              ,ckf.segment28_meaning
              ,ckf.segment29
              ,ckf.segment29_value
              ,ckf.segment29_meaning
              ,ckf.segment30
              ,ckf.segment30_value
              ,ckf.segment30_meaning
              )
            ,l_asg_data
            )
    FROM   xxas_tcm_log_entry_approvers_v xtla
          ,xxas_cost_key_ff_v             ckf
    WHERE  xtla.log_entry_id  = self.log_entry_id
    AND    xtla.assignment_id = self.resource_id
    AND    xtla.cost_allocation_keyflex_id = ckf.cost_allocation_keyflex_id(+);

    CURSOR csr_person IS
    SELECT papf.*
    FROM   per_all_people_f papf
    WHERE  papf.person_id = self.person_id
    AND    trunc(sysdate) BETWEEN papf.effective_start_date 
                          AND     papf.effective_end_date
    AND    (nvl(papf.current_employee_flag, 'N') = 'Y'
            OR nvl(papf.current_npw_flag, 'N') = 'Y');

    CURSOR csr_abs_event_dates IS
    SELECT paa.date_projected_end
          ,paa.date_projected_start
          ,paa.time_projected_end
          ,paa.time_projected_start
          ,paa.date_end
          ,paa.date_start
          ,paa.time_end
          ,paa.time_start
    FROM   per_absence_attendances paa
    WHERE  paa.person_id = self.person_id
    AND    paa.absence_attendance_id = self.abs_attendance_id;

    CURSOR csr_abs_event_count IS
    SELECT count(*)
    FROM   per_absence_attendances paa
    WHERE  paa.person_id = self.person_id
    AND    paa.absence_attendance_id = self.abs_attendance_id;

    CURSOR csr_person_data IS
    SELECT XMLElement
            ("EMPLOYEES"
            ,XMLElement
               ("value"
               ,XMLAttributes
                 ('true' as "json:force-array"
                 ,'http://json.org/' as "xmlns:json")
               ,XMLAgg
                 (XMLElement
                   ("value"
                   ,XMLForest
                     (p.person_id "id"
                     ,p.short_name "label"
                     ,xxas_util_pkg.flag_to_boolean('N') "deletable")
                   ,XMLElement
                     ("linked"
                     ,XMLElement
                       ("image"
                       ,XMLForest
                         (xxas_com_rest_util_pkg.uri_from_path
                            ('people/' || p.img_person_id || '/photo') "self"
                         ,p.initials "initials"
                         ,xxas_per_image_pkg.url
                            (p.img_person_id, p.img_last_update_date, 'SMALL') "small"
                         )
                       )
                     )
                   )
                 )
               ).getClobVal()
            )
    FROM  (SELECT papf.person_id
                 ,xxas_util_pkg.get_formatted_name
                    (papf.full_name
                    ,papf.global_name
                    ,papf.local_name
                    ,papf.first_name
                    ,papf.last_name
                    ,papf.known_as) short_name
                 ,xxas_util_pkg.get_initials(papf.first_name, papf.last_name, papf.known_as) initials
                 ,img.person_id              img_person_id
                 ,img.last_update_date       img_last_update_date
           FROM   per_all_people_f papf
                 ,xxas_per_thumbnails img
           WHERE  papf.person_id = self.person_id
           AND    trunc(sysdate) BETWEEN papf.effective_start_date 
                                 AND     papf.effective_end_date
           AND    papf.person_id = img.person_id(+)) p;

    CURSOR csr_otl_timecard_dates IS
    SELECT hts.start_time
          ,hts.stop_time
          ,hts.approval_status
          ,hts.timecard_id
          ,hts.timecard_ovn
          ,htbb_detail.object_version_number
          ,fnd_date.date_to_canonical(htbb_detail.date_from) timecard_date_from
          ,htbb_detail.comment_text comments_to_approver
    FROM   hxc_timecard_summary     hts
          ,hxc_time_building_blocks htbb_time
          ,hxc_time_building_blocks htbb_day
          ,hxc_time_building_blocks htbb_detail
          ,hxc_time_attributes hta
          ,hxc_time_attribute_usages htau
          ,pay_element_types_f petf
          ,per_all_people_f papf
          ,fnd_lookup_values flv
    WHERE  hts.resource_id                     = self.person_id
    AND    hts.timecard_id                     = htbb_time.time_building_block_id
    AND    hts.timecard_ovn                    = htbb_time.object_version_number
    AND    htbb_time.time_building_block_id    = htbb_day.parent_building_block_id
    AND    htbb_time.object_version_number     = htbb_day.parent_building_block_ovn
    AND    htbb_time.scope                     = 'TIMECARD'
    AND    htbb_time.resource_type             = 'PERSON'
    AND    htbb_time.date_to                   = hr_general.end_of_time
    AND    htbb_day.time_building_block_id     = htbb_detail.parent_building_block_id
    AND    htbb_day.object_version_number      = htbb_detail.parent_building_block_ovn
    AND    htbb_day.scope                      = 'DAY'
    AND    htbb_day.date_to                    = hr_general.end_of_time
    AND    htbb_detail.scope                   = 'DETAIL'
    AND    htbb_detail.date_to                 = hr_general.end_of_time
    AND    htbb_detail.time_building_block_id  = self.log_entry_id
    AND    htau.time_building_block_id         = htbb_detail.time_building_block_id
    AND    htau.time_building_block_ovn        = htbb_detail.object_version_number
    AND    htau.time_attribute_id              = hta.time_attribute_id
    AND    to_char(petf.element_type_id)       = (substr(hta.attribute_category,11,length(hta.attribute_category)))
    AND    hta.attribute_category is not null
    AND    hta.attribute_category not like 'SEC%'
    AND    hts.resource_id                     = papf.person_id
    AND    trunc(sysdate) BETWEEN papf.effective_start_date
                          AND     papf.effective_end_date
    AND    (nvl(papf.current_employee_flag, 'N') = 'Y'
            OR nvl(papf.current_npw_flag, 'N')   = 'Y')
    AND     hts.approval_status     = flv.lookup_code
    AND     flv.language            = userenv('LANG')
    AND     flv.view_application_id = 3
    AND     flv.lookup_type         = 'HXC_APPROVAL_STATUS'
    AND     flv.security_group_id   = 0
    AND     fnd_profile.value('XXAS_TIMESTORE') = 'OTL'
    ORDER BY htbb_detail.object_version_number DESC;

    CURSOR csr_otl_event_view_xml IS
    SELECT XMLElement
            ("ROW"
            ,XMLForest
              (self.person_id person_id
              ,self.resource_id assignment_id
              ,self.log_entry_id timecard_id
              ,l_timecard_summary_id  timecard_summary_id
              ,l_timecard_summary_ovn timecard_summary_ovn
              ,l_timecard_ovn        timecard_ovn
              ,l_timecard_date_from  timecard_date_from
              ,self.source_block source_block
              ,self.saved_search_id saved_search_id
              ,xxas_cal_util_pkg.source_block_called_view(self.source_block) as called_view
              ,self.log_entry_id approvers
              ,l_timecard_summary_start_date timecard_summary_start_date
              ,l_timecard_summary_end_date   timecard_summary_end_date
              ,l_timecard_status     current_state_code
              ,l_comments_to_approver         comments_to_approver
              )
            ,l_person_data
            ,cast
                (multiset
                  (SELECT xxas_attachment_type
                            (attach.attached_document_id
                            ,attach.sequence_number
                            ,attach.entity_name
                            ,attach.pk1_value
                            ,attach.pk2_value
                            ,attach.pk3_value
                            ,attach.pk4_value
                            ,attach.pk5_value
                            ,attach.category_code
                            ,attach.category_name
                            ,attach.document_id
                            ,attach.file_name
                            ,attach.title
                            ,attach.description
                            ,attach.file_id
                            ,attach.mime_type
                            ,attach.lob_format
                            ,attach.file_url
                            ,attach.wp_link
                            ,attach.creation_date
                            ,attach.created_by
                            ,attach.last_update_date
                            ,attach.last_updated_by
                            )
                   FROM   TABLE(xxas_cal_util_pkg.attachments_data
                                  (p_attachment_id           => null
                                  ,p_entity_name             => 'TIMECARD_ID'
                                  ,p_pk1_value               => to_char(l_timecard_summary_id)
                                  ,p_pk2_value               => null
                                  ,p_pk3_value               => null
                                  ,p_pk4_value               => null
                                  ,p_pk5_value               => null
                                  ,p_transaction_id          => null
                                  ,p_transaction_entity_name => null)) attach
                  )
                  AS xxas_attachment_table
                ) attachments
            )
    FROM   dual;

    l_per_row              csr_person%ROWTYPE;
    l_date_projected_end   date;
    l_date_projected_start date;
    l_date_end             date;
    l_date_start           date;
    l_time_projected_end   varchar2(10);
    l_time_projected_start varchar2(10);
    l_time_end             varchar2(10);
    l_time_start           varchar2(10);
    l_abs_event_dates_rec  csr_abs_event_dates%ROWTYPE;
    l_count                number;
    l_time_entry_rec       csr_time_entry_values%ROWTYPE;
    l_field_values_rec     csr_field_values%ROWTYPE;
    l_period_id            number;
    l_today_log_summary    varchar2(4000);
    l_total_log_summary    varchar2(4000);

  BEGIN

    IF self.action_id = 'ABSENCE' THEN

      IF self.transaction_id is null THEN

        OPEN  csr_abs_event_dates;
        FETCH csr_abs_event_dates INTO l_abs_event_dates_rec;
        CLOSE csr_abs_event_dates;

        l_date_projected_end   := l_abs_event_dates_rec.date_projected_end;
        l_date_projected_start := l_abs_event_dates_rec.date_projected_start;
        l_time_projected_end   := l_abs_event_dates_rec.time_projected_end;
        l_time_projected_start := l_abs_event_dates_rec.time_projected_start;
        l_date_end             := l_abs_event_dates_rec.date_end;
        l_date_start           := l_abs_event_dates_rec.date_start;
        l_time_end             := l_abs_event_dates_rec.time_end;
        l_time_start           := l_abs_event_dates_rec.time_start;

        IF l_date_projected_end is not null OR 
           l_date_projected_start is not null OR 
           l_time_projected_end is not null OR 
           l_time_projected_start is not null THEN

          l_leave_status := 'PLANNED';

        ELSIF l_date_end is not null OR 
              l_date_start is not null OR 
              l_time_end is not null OR 
              l_time_start is not null THEN

          l_leave_status := 'CONFIRMED';

        ELSE

          l_leave_status := 'CONFIRMED';

        END IF;

        OPEN  csr_abs_event_view_xml;
        FETCH csr_abs_event_view_xml INTO l_xml;
        CLOSE csr_abs_event_view_xml;

      ELSE

        OPEN  csr_abs_event_dates;
        FETCH csr_abs_event_dates INTO l_abs_event_dates_rec;
        CLOSE csr_abs_event_dates;

        l_date_projected_end   := nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(self.transaction_id, 'DateProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,l_abs_event_dates_rec.date_projected_end);
        l_date_projected_start := nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(self.transaction_id, 'DateProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,l_abs_event_dates_rec.date_projected_start);
        l_time_projected_end   := nvl(xxas_xml_node_util_pkg.get_node_value(self.transaction_id, 'TimeProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                                     ,l_abs_event_dates_rec.time_projected_end);
        l_time_projected_start := nvl(xxas_xml_node_util_pkg.get_node_value(self.transaction_id, 'TimeProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                                     ,l_abs_event_dates_rec.time_projected_start);
        l_date_end             := nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(self.transaction_id, 'DateEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,l_abs_event_dates_rec.date_end);
        l_date_start           := nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(self.transaction_id, 'DateStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,l_abs_event_dates_rec.date_start);
        l_time_end             := nvl(xxas_xml_node_util_pkg.get_node_value(self.transaction_id, 'TimeEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                                     ,l_abs_event_dates_rec.time_end);
        l_time_start           := nvl(xxas_xml_node_util_pkg.get_node_value(self.transaction_id, 'TimeStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                                     ,l_abs_event_dates_rec.time_start);

        IF ((l_date_projected_end is not null OR 
           l_date_projected_start is not null OR 
           l_time_projected_end is not null OR 
           l_time_projected_start is not null)
           AND l_date_start is null)
           THEN

          l_leave_status := 'PLANNED';

        ELSIF l_date_end is not null OR 
              l_date_start is not null OR 
              l_time_end is not null OR 
              l_time_start is not null THEN

          l_leave_status := 'CONFIRMED';

        ELSE

          l_leave_status := 'CONFIRMED';

        END IF;

        OPEN  csr_abs_event_count;
        FETCH csr_abs_event_count INTO l_count;
        CLOSE csr_abs_event_count;

        IF l_count = 0 THEN

          OPEN  csr_abs_event_tran_in_view_xml;
          FETCH csr_abs_event_tran_in_view_xml INTO l_xml;
          CLOSE csr_abs_event_tran_in_view_xml;

        ELSE

          OPEN  csr_abs_event_tran_view_xml;
          FETCH csr_abs_event_tran_view_xml INTO l_xml;
          CLOSE csr_abs_event_tran_view_xml;

        END IF;

      END IF;

    ELSIF self.action_id = 'TIMECARD' THEN

      OPEN  csr_person;
      FETCH csr_person INTO l_per_row;
      CLOSE csr_person;

      OPEN  csr_asg_data;
      FETCH csr_asg_data INTO l_asg_data;
      CLOSE csr_asg_data;

      OPEN  csr_time_entry_values(self.resource_id
                                 ,self.log_entry_id);
      FETCH csr_time_entry_values INTO l_time_entry_rec;
      CLOSE csr_time_entry_values;

      OPEN  csr_field_values(self.resource_id
                            ,l_time_entry_rec.date_start
                            ,l_time_entry_rec.element_type_id);
      FETCH csr_field_values INTO l_field_values_rec;
      CLOSE csr_field_values;

      l_field1_data_type := l_field_values_rec.field1_datatype;
      l_field2_data_type := l_field_values_rec.field2_datatype;
      l_field3_data_type := l_field_values_rec.field3_datatype;
      l_field4_data_type := l_field_values_rec.field4_datatype;
      l_field5_data_type := l_field_values_rec.field5_datatype;

      OPEN  csr_tcm_period_id;
      FETCH csr_tcm_period_id INTO l_period_id;
      CLOSE csr_tcm_period_id;

      l_today_log_summary := xxas_timecard_periods_pkg.asg_time_summary_log_on_date
                               (p_assignment_id => self.resource_id
                               ,p_logged_date   => l_time_entry_rec.date_start
                               ,p_asg_tbl       => null);

      l_total_log_summary := xxas_timecard_periods_pkg.asg_time_summary_log_on_period
                               (p_assignment_id => self.resource_id
                               ,p_logged_period => l_period_id
                               ,p_asg_tbl       => null);

      OPEN  csr_tcm_event_view_xml(l_per_row.business_group_id
                                  ,l_today_log_summary
                                  ,l_total_log_summary);
      FETCH csr_tcm_event_view_xml INTO l_xml;
      CLOSE csr_tcm_event_view_xml;

    ELSIF self.action_id = 'OTL' THEN

      OPEN  csr_person_data;
      FETCH csr_person_data INTO l_person_data;
      CLOSE csr_person_data;

      IF self.log_entry_id is not null THEN

        OPEN  csr_otl_timecard_dates;
        FETCH csr_otl_timecard_dates INTO l_timecard_summary_start_date
                                         ,l_timecard_summary_end_date
                                         ,l_timecard_status
                                         ,l_timecard_summary_id
                                         ,l_timecard_summary_ovn
                                         ,l_timecard_ovn
                                         ,l_timecard_date_from
                                         ,l_comments_to_approver;
        CLOSE csr_otl_timecard_dates;

      END IF;

      OPEN  csr_otl_event_view_xml;
      FETCH csr_otl_event_view_xml INTO l_xml;
      CLOSE csr_otl_event_view_xml;

    END IF;

    RETURN l_xml;

  END;

END;
/

CREATE OR REPLACE TYPE BODY xxas_cal_event_edit IS

  CONSTRUCTOR FUNCTION xxas_cal_event_edit
    (p_person_id           IN  number
    ,p_resource_id         IN  number
    ,p_abs_attendance_id   IN  number
    ,p_log_entry_id        IN  number
    ,p_transaction_id      IN  number
    ,p_transaction_step_id IN  number
    ,p_action_id           IN  varchar2
    ,p_source_block        IN  varchar2
    ,p_saved_search_id     IN number
    ,p_manager_id          IN  number default null) RETURN SELF AS RESULT
  IS

  BEGIN

    self.person_id           :=  p_person_id;
    self.resource_id         :=  p_resource_id;
    self.abs_attendance_id   :=  p_abs_attendance_id;
    self.log_entry_id        :=  p_log_entry_id;
    self.transaction_id      :=  p_transaction_id;
    self.transaction_step_id :=  p_transaction_step_id;
    self.action_id           :=  p_action_id;
    self.source_block        :=  p_source_block;
    self.saved_search_id     :=  p_saved_search_id;
    self.manager_id          :=  p_manager_id;

    RETURN;

  END;

  MEMBER FUNCTION get_xml RETURN XMLType
  IS

    l_xml                  XMLType;
    l_leave_status         varchar2(100);
    l_approver_person_id   number;
    l_asg_data             XMLType;
    l_person_data          XMLType;
    l_field1_data_type     varchar2(100);
    l_field2_data_type     varchar2(100);
    l_field3_data_type     varchar2(100);
    l_field4_data_type     varchar2(100);
    l_field5_data_type     varchar2(100);

    CURSOR csr_abs_event_edit_xml IS
    SELECT XMLElement
            ("ROW"
            ,XMLForest
              (paa.absence_attendance_id
              ,paa.object_version_number
              ,paa.business_group_id
              ,null status
              ,self.source_block source_block
              ,self.saved_search_id saved_search_id
              ,self.manager_id as manager_id
              ,xxas_cal_util_pkg.source_block_called_view(self.source_block) as called_view
              ,l_leave_status leave_status
              ,paa.absence_attendance_type_id            absence_type_id
              ,paat.name                                 absence_type
              ,paat.absence_category                     absence_category_code
              ,fcl.meaning                               absence_category
              ,paa.abs_attendance_reason_id
              ,(SELECT rea.meaning     
                FROM   per_abs_attendance_reasons paar
                      ,hr_lookups                 rea
                WHERE  paa.abs_attendance_reason_id   = paar.abs_attendance_reason_id
                AND    paar.name                      = rea.lookup_code
                AND    rea.lookup_type                = 'ABSENCE_REASON') absence_reason 
              ,paa.person_id
              ,xxas_util_pkg.get_primary_worker_asg(paa.person_id,trunc(sysdate)) as assignment_id
              ,paa.authorising_person_id
              ,paa.replacement_person_id
              ,paa.period_of_incapacity_id
              ,paa.absence_days duration_days
              ,paa.absence_hours duration_hours
              ,paa.date_notification
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(paa.date_projected_start,'YYYY-MM-DD') || ' ' || paa.time_projected_start || '_' ||
                          to_char(paa.date_projected_end,'YYYY-MM-DD') || ' ' || paa.time_projected_end
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(paa.date_start,'YYYY-MM-DD') || ' ' || paa.time_start || '_' ||
                          to_char(paa.date_end,'YYYY-MM-DD') || ' ' || paa.time_end
                     ELSE null
                END) date_time_range
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(paa.date_projected_start,'YYYY-MM-DD') || '_' ||
                          to_char(paa.date_projected_end,'YYYY-MM-DD')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(paa.date_start,'YYYY-MM-DD') || '_' ||
                          to_char(paa.date_end,'YYYY-MM-DD')
                     ELSE null
                END) date_range
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(paa.date_projected_start,'YYYY-MM-DD')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(paa.date_start,'YYYY-MM-DD')
                     ELSE null
                END) startdate
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(paa.date_projected_end,'YYYY-MM-DD')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(paa.date_end,'YYYY-MM-DD')
                     ELSE null
                END) enddate
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN paa.time_projected_start
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN paa.time_start
                     ELSE null
                END) starttime
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN paa.time_projected_end
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN paa.time_end
                     ELSE null
                END) endtime
              ,paa.occurrence
              ,paa.ssp1_issued
              ,paa.attribute_category
              ,paa.attribute1
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE1'
                 ,paa.attribute1)                        attribute1_meaning 
              ,paa.attribute2
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE2'
                 ,paa.attribute2)                        attribute2_meaning 
              ,paa.attribute3
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE3'
                 ,paa.attribute3)                        attribute3_meaning 
              ,paa.attribute4
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE4'
                 ,paa.attribute4)                        attribute4_meaning 
              ,paa.attribute5
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE5'
                 ,paa.attribute5)                        attribute5_meaning 
              ,paa.attribute6
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE6'
                 ,paa.attribute6)                        attribute6_meaning 
              ,paa.attribute7
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE7'
                 ,paa.attribute7)                        attribute7_meaning 
              ,paa.attribute8
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE8'
                 ,paa.attribute8)                        attribute8_meaning 
              ,paa.attribute9
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE9'
                 ,paa.attribute9)                        attribute9_meaning 
              ,paa.attribute10
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE10'
                 ,paa.attribute10)                      attribute10_meaning 
              ,paa.attribute11
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE11'
                 ,paa.attribute11)                      attribute11_meaning 
              ,paa.attribute12
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE12'
                 ,paa.attribute12)                       attribute12_meaning 
              ,paa.attribute13
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE13'
                 ,paa.attribute13)                       attribute13_meaning 
              ,paa.attribute14
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE14'
                 ,paa.attribute14)                       attribute14_meaning 
              ,paa.attribute15
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE15'
                 ,paa.attribute15)                       attribute15_meaning 
              ,paa.attribute16
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE16'
                 ,paa.attribute16)                       attribute16_meaning 
              ,paa.attribute17
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE17'
                 ,paa.attribute17)                       attribute17_meaning 
              ,paa.attribute18
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE18'
                 ,paa.attribute18)                       attribute18_meaning 
              ,paa.attribute19
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE19'
                 ,paa.attribute19)                       attribute19_meaning 
              ,paa.attribute20
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE20'
                 ,paa.attribute20)                       attribute20_meaning 
              ,paa.last_update_date
              ,paa.last_updated_by
              ,paa.last_update_login
              ,paa.created_by
              ,paa.creation_date
              ,paa.maternity_id
              ,paa.sickness_start_date
              ,paa.sickness_end_date
              ,paa.pregnancy_related_illness 
              ,paa.reason_for_notification_delay 
              ,paa.accept_late_notification_flag
              ,paa.linked_absence_id
              ,paa.abs_information_category
              ,paa.abs_information1
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION1'
                 ,paa.abs_information1)                  abs_information1_meaning 
              ,paa.abs_information2
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION2'
                 ,paa.abs_information2)                  abs_information2_meaning
              ,paa.abs_information3
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION3'
                 ,paa.abs_information3)                  abs_information3_meaning
              ,paa.abs_information4
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION4'
                 ,paa.abs_information4)                  abs_information4_meaning
              ,paa.abs_information5
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION5'
                 ,paa.abs_information5)                  abs_information5_meaning
              ,paa.abs_information6
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION6'
                 ,paa.abs_information6)                  abs_information6_meaning
              ,paa.abs_information7
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION7'
                 ,paa.abs_information7)                  abs_information7_meaning
              ,paa.abs_information8
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION8'
                 ,paa.abs_information8)                  abs_information8_meaning
              ,paa.abs_information9
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION9'
                 ,paa.abs_information9)                  abs_information9_meaning
              ,paa.abs_information10
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION10'
                 ,paa.abs_information10)                 abs_information10_meaning
              ,paa.abs_information11
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION11'
                 ,paa.abs_information11)                 abs_information11_meaning
              ,paa.abs_information12
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION12'
                 ,paa.abs_information12)                 abs_information12_meaning
              ,paa.abs_information13
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION13'
                 ,paa.abs_information13)                 abs_information13_meaning
              ,paa.abs_information14
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION14'
                 ,paa.abs_information14)                 abs_information14_meaning
              ,paa.abs_information15
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION15'
                 ,paa.abs_information15)                 abs_information15_meaning
              ,paa.abs_information16
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION16'
                 ,paa.abs_information16)                 abs_information16_meaning
              ,paa.abs_information17
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION17'
                 ,paa.abs_information17)                 abs_information17_meaning
              ,paa.abs_information18
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION18'
                 ,paa.abs_information18)                 abs_information18_meaning
              ,paa.abs_information19
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION19'
                 ,paa.abs_information19)                 abs_information19_meaning
              ,paa.abs_information20
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION20'
                 ,paa.abs_information20)                 abs_information20_meaning
              ,paa.abs_information21
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION21'
                 ,paa.abs_information21)                 abs_information21_meaning
              ,paa.abs_information22
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION22'
                 ,paa.abs_information22)                 abs_information22_meaning
              ,paa.abs_information23
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION23'
                 ,paa.abs_information23)                 abs_information23_meaning
              ,paa.abs_information24
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION24'
                 ,paa.abs_information24)                 abs_information24_meaning
              ,paa.abs_information25
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION25'
                 ,paa.abs_information25)                 abs_information25_meaning
              ,paa.abs_information26
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION26'
                 ,paa.abs_information26)                 abs_information26_meaning
              ,paa.abs_information27
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION27'
                 ,paa.abs_information27)                 abs_information27_meaning
              ,paa.abs_information28
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION28'
                 ,paa.abs_information28)                 abs_information28_meaning
              ,paa.abs_information29
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION29'
                 ,paa.abs_information29)                 abs_information29_meaning
              ,paa.abs_information30
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION30'
                 ,paa.abs_information30)                 abs_information30_meaning
              ,paa.batch_id
              ,xxas_util_pkg.long_to_varchar2
                 ('PER_ABSENCE_ATTENDANCES'
		         ,'COMMENTS' 
		         ,paa.rowid)                          comments_to_approver    
              ,paa.absence_case_id
              ,paa.approval_status
              ,paa.confirmed_until
              ,paa.source
              ,paa.advance_pay
              ,null transaction_id
              ,null transaction_step_id
              ,self.person_id as employees
              )
			,CASE WHEN xxas_com_sec_pkg.test_instance('XXAS_ABS_VAC_RULE_SETUP',paa.person_id) = 'Y' THEN
			   xxas_absence_event_pkg.get_vacation_rules_xml(paa.person_id,paa.date_start,paa.date_end)
			 END
            ,cast
                (multiset
                  (SELECT xxas_attachment_type
                            (attach.attached_document_id
                            ,attach.sequence_number
                            ,attach.entity_name
                            ,attach.pk1_value
                            ,attach.pk2_value
                            ,attach.pk3_value
                            ,attach.pk4_value
                            ,attach.pk5_value
                            ,attach.category_code
                            ,attach.category_name
                            ,attach.document_id
                            ,attach.file_name
                            ,attach.title
                            ,attach.description
                            ,attach.file_id
                            ,attach.mime_type
                            ,attach.lob_format
                            ,attach.file_url
                            ,attach.wp_link
                            ,attach.creation_date
                            ,attach.created_by
                            ,attach.last_update_date
                            ,attach.last_updated_by
                            )
                   FROM   TABLE(xxas_cal_util_pkg.attachments_data
                                  (p_attachment_id           => null
                                  ,p_entity_name             => 'PER_ABSENCE_ATTENDANCES'
                                  ,p_pk1_value               => to_char(paa.absence_attendance_id)
                                  ,p_pk2_value               => null
                                  ,p_pk3_value               => null
                                  ,p_pk4_value               => null
                                  ,p_pk5_value               => null
                                  ,p_transaction_id          => null
                                  ,p_transaction_entity_name => null)) attach
                  )
                 AS xxas_attachment_table
                ) attachments
            )
    FROM   per_absence_attendances      paa
          ,per_abs_attendance_types_vl  paat
          ,hr_lookups                   fcl
    WHERE  paa.absence_attendance_type_id  = paat.absence_attendance_type_id
    AND    paat.absence_category           = fcl.lookup_code(+)
    AND    fcl.lookup_type(+)              = 'ABSENCE_CATEGORY'
    AND    paa.person_id                   = self.person_id
    AND    paa.absence_attendance_id       = self.abs_attendance_id;

    CURSOR csr_abs_event_tran_in_edit_xml IS
    SELECT XMLElement
            ("ROW"
            ,XMLForest
              (hat.selected_person_id as person_id
              ,xxas_util_pkg.get_primary_worker_asg(hat.selected_person_id,trunc(sysdate)) as assignment_id
              ,hat.assignment_id as override_assignment_id
              ,hat.transaction_ref_id as absence_attendance_id
              ,hat.transaction_id as transaction_id
              ,hats.transaction_step_id as transaction_step_id
              ,self.source_block source_block
              ,self.saved_search_id saved_search_id
              ,self.manager_id as manager_id
              ,xxas_cal_util_pkg.source_block_called_view(self.source_block) as called_view
              ,decode(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAction', 'Transaction/TransCtx/CNode'), 'CreateMode', 'INSERT', 'UpdateMode', 'UPDATE', 'CancelMode', 'DELETE', 'INSERT')  transaction_type
              ,hat.status as hr_api_status_code
              ,xxas_selfservice_workflow_pkg.transcode_status
                 (hat.status) as workflow_status_code
              ,xxas_selfservice_workflow_pkg.decode_status
                 (xxas_selfservice_workflow_pkg.transcode_status
                    (hat.status)) as workflow_status_meaning
              ,l_leave_status leave_status
              ,to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'BusinessGroupId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')) business_group_id
              ,to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceTypeId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')) absence_type_id
              ,(SELECT paat.name
                FROM   per_abs_attendance_types_vl paat
                WHERE  paat.absence_attendance_type_id = to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceTypeId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))) absence_type
              ,to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsAttendanceReasonId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')) abs_attendance_reason_id
             ,(SELECT rea.meaning     
               FROM   per_abs_attendance_reasons paar
                     ,hr_lookups                 rea
               WHERE  paar.abs_attendance_reason_id = to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsAttendanceReasonId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
               AND    paar.name                     = rea.lookup_code
               AND    rea.lookup_type               = 'ABSENCE_REASON') abs_attendance_reason
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')),'YYYY-MM-DD') || ' ' || xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') || '_' ||
                          to_char(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')),'YYYY-MM-DD') || ' ' || xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')),'YYYY-MM-DD') || ' ' || xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') || '_' ||
                          to_char(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')),'YYYY-MM-DD') || ' ' || xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                     ELSE null
                END) date_time_range
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')),'YYYY-MM-DD') || '_' ||
                          to_char(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')),'YYYY-MM-DD')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')),'YYYY-MM-DD') || '_' ||
                          to_char(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')),'YYYY-MM-DD')
                     ELSE null
                END) date_range
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')),'YYYY-MM-DD')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')),'YYYY-MM-DD')
                     ELSE null
                END) startdate
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')),'YYYY-MM-DD')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')),'YYYY-MM-DD')
                     ELSE null
                END) enddate
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                     ELSE null
                END) starttime
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                     ELSE null
                END) endtime
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateNotification', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') date_notification
              ,(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceDays', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')) duration_days
              ,(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceHours', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')) duration_hours
              ,(SELECT paat.absence_category
                FROM   per_abs_attendance_types_vl paat
                WHERE  paat.absence_attendance_type_id = to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceTypeId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))) absence_category_code
             ,(SELECT fcl.meaning
               FROM   per_abs_attendance_types_vl paat
                     ,hr_lookups fcl
               WHERE  paat.absence_attendance_type_id = to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceTypeId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
               AND    paat.absence_category    = fcl.lookup_code(+)
               AND    fcl.lookup_type(+)       = 'ABSENCE_CATEGORY') absence_category
              ,to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'ObjectVersionNumber', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')) object_version_number
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute_category
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute1', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute1
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE1'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute1', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')) attribute1_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute2', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute2
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE2'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute2', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                    attribute2_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute3', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute3
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE3'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute3', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                    attribute3_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute4', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute4
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE4'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute4', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                    attribute4_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute5', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute5
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE5'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute5', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                    attribute5_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute6', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute6
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE6'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute6', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                    attribute6_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute7', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute7
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE7'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute7', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                    attribute7_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute8', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute8
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE8'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute8', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                    attribute8_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute9', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute9
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE9'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute9', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                    attribute9_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute10', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute10
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE10'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute10', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute10_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute11', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute11
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE11'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute11', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute11_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute12', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute12
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE12'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute12', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute12_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute13', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute13
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE13'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute13', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute13_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute14', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute14
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE14'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute14', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute14_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute15', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute15
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE15'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute15', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute15_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute16', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute16
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE16'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute16', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute16_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute17', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute17
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE17'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute17', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute17_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute18', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute18
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE18'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute18', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute18_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute19', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute19
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE19'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute19', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute19_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute20', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute20
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE20'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute20', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute20_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information_category
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation1', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information1
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 (hat.selected_person_id
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION1'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation1', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information1_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation2', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information2
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION2'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation2', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information2_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation3', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information3
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION3'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation3', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information3_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation4', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information4
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION4'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation4', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information4_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation5', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information5
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION5'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation5', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information5_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation6', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information6
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION6'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation6', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information6_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation7', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information7
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION7'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation7', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information7_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation8', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information8
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION8'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation8', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information8_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation9', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information9
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION9'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation9', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information9_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation10', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information10
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION10'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation10', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information10_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation11', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information11
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION11'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation11', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information11_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation12', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information12
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION12'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation12', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information12_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation13', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information13
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION13'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation13', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information13_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation14', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information14
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION14'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation14', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information14_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation15', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information15
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION15'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation15', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information15_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation16', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information16
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION16'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation16', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information16_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation17', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information17
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION17'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation17', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information17_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation18', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information18
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION18'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation18', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information18_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation19', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information19
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION19'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation19', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information19_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation20', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information20
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION20'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation20', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information20_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation21', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information21
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION21'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation21', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information21_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation22', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information22
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION22'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation22', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information22_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation23', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information23
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION23'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation23', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information23_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation24', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information24
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION24'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation24', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information24_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation25', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information25
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION25'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation25', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information25_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation26', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information26
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION26'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation26', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information26_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation27', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information27
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION27'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation27', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information27_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation28', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information28
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION28'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation28', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information28_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation29', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information29
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION29'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation29', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information29_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation30', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information30
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION30'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation30', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information30_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Comments', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') comments_to_approver
              ,null                                      authorising_person_id
              ,to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'ReplacementPersonId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')) replacement_person_id
              ,null                                      period_of_incapacity_id
              ,null                                      occurrence
              ,null                                      ssp1_issued 
              ,null                                      last_update_date
              ,null                                      last_updated_by
              ,null                                      last_update_login
              ,null                                      created_by
              ,null                                      creation_date
              ,null                                      maternity_id
              ,null                                      sickness_start_date
              ,null                                      sickness_end_date
              ,null                                      pregnancy_related_illness 
              ,null                                      reason_for_notification_delay 
              ,null                                      accept_late_notification_flag
              ,null                                      linked_absence_id
              ,null                                      batch_id
              ,null                                      comments       
              ,null                                      absence_case_id
              ,null                                      approval_status
              ,null                                      confirmed_until
              ,null                                      source
              ,null                                      advance_pay
              ,hat.transaction_id approvers
              ,self.person_id as employees
              )
			,CASE WHEN xxas_com_sec_pkg.test_instance('XXAS_ABS_VAC_RULE_SETUP',hat.selected_person_id) = 'Y' THEN
			   XMLForest('Y' as "VACATION_RULES_PERM"
			            ,hr_transaction_api.get_varchar2_value(hats.transaction_step_id,'P_SETUP_VAC_RULE') as "REDIRECT_NOTIF"
			            ,hr_transaction_api.get_varchar2_value(hats.transaction_step_id,'P_RULE') as "RULE"
						,hr_transaction_api.get_varchar2_value(hats.transaction_step_id,'P_ASSIGNEE') as "GRANTEE")
			 END
            ,cast
                (multiset
                  (SELECT xxas_attachment_type
                            (attach.attached_document_id
                            ,attach.sequence_number
                            ,attach.entity_name
                            ,attach.pk1_value
                            ,attach.pk2_value
                            ,attach.pk3_value
                            ,attach.pk4_value
                            ,attach.pk5_value
                            ,attach.category_code
                            ,attach.category_name
                            ,attach.document_id
                            ,attach.file_name
                            ,attach.title
                            ,attach.description
                            ,attach.file_id
                            ,attach.mime_type
                            ,attach.lob_format
                            ,attach.file_url
                            ,attach.wp_link
                            ,attach.creation_date
                            ,attach.created_by
                            ,attach.last_update_date
                            ,attach.last_updated_by
                            )
                   FROM   TABLE(xxas_cal_util_pkg.attachments_data
                                  (p_attachment_id           => null
                                  ,p_entity_name             => 'PER_ABSENCE_ATTENDANCES'
                                  ,p_pk1_value               => hat.transaction_ref_id || '_' || hat.transaction_id
                                  ,p_pk2_value               => null
                                  ,p_pk3_value               => null
                                  ,p_pk4_value               => null
                                  ,p_pk5_value               => null
                                  ,p_transaction_id          => hat.transaction_id
                                  ,p_transaction_entity_name => 'PQH_SS_ATTACHMENT')) attach
                  )
                 AS xxas_attachment_table
                ) attachments
            )
    FROM   hr_api_transactions         hat
          ,hr_api_transaction_steps    hats
    WHERE  hat.transaction_id       = hats.transaction_id
    AND    hat.status          NOT IN ('D', 'N', 'W', 'ACTIVE', 'AC')
    AND    hats.api_name            = 'HR_PERSON_ABSENCE_SWI.PROCESS_API'
    AND    decode(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAction', 'Transaction/TransCtx/CNode'), 'CreateMode', 'INSERT') = 'INSERT'
    AND    hat.selected_person_id   = self.person_id
    AND    hat.transaction_id       = self.transaction_id
    AND    hats.transaction_step_id = self.transaction_step_id
    AND    hat.transaction_ref_id   = self.abs_attendance_id;

    CURSOR csr_abs_event_tran_edit_xml IS
    SELECT XMLElement
            ("ROW"
            ,XMLForest
              (paa.person_id
              ,xxas_util_pkg.get_primary_worker_asg(paa.person_id,trunc(sysdate)) as assignment_id
              ,hat.assignment_id as override_assignment_id
              ,hat.transaction_ref_id absence_attendance_id
              ,hat.transaction_id transaction_id
              ,hats.transaction_step_id transaction_step_id
              ,self.source_block source_block
              ,self.saved_search_id saved_search_id
              ,self.manager_id as manager_id
              ,xxas_cal_util_pkg.source_block_called_view(self.source_block) as called_view
              ,decode(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAction', 'Transaction/TransCtx/CNode'), 'CreateMode', 'INSERT', 'UpdateMode', 'UPDATE', 'CancelMode', 'DELETE', 'INSERT') transaction_type
              ,hat.status hr_api_status_code
              ,xxas_selfservice_workflow_pkg.transcode_status(hat.status) workflow_status_code
              ,xxas_selfservice_workflow_pkg.decode_status
                 (xxas_selfservice_workflow_pkg.transcode_status(hat.status)) workflow_status_meaning
              ,l_leave_status leave_status
              ,nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'BusinessGroupId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                  ,paa.business_group_id) business_group_id
              ,nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceTypeId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                  ,paa.absence_attendance_type_id) absence_type_id
              ,(SELECT paat.name
                FROM   per_abs_attendance_types_vl paat
                WHERE  paat.absence_attendance_type_id = nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceTypeId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                                            ,paa.absence_attendance_type_id)) absence_type
              ,nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsAttendanceReasonId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                  ,paa.abs_attendance_reason_id) abs_attendance_reason_id
              ,(SELECT rea.meaning     
                FROM   per_abs_attendance_reasons paar
                      ,hr_lookups                 rea
                WHERE  paar.abs_attendance_reason_id = nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsAttendanceReasonId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                                          ,paa.abs_attendance_reason_id)
                AND    paar.name                     = rea.lookup_code
                AND    rea.lookup_type               = 'ABSENCE_REASON') abs_attendance_reason
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,paa.date_projected_start),'YYYY-MM-DD') || ' ' || nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                                                                                           ,paa.time_projected_start) || '_' ||
                          to_char(nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,paa.date_projected_end),'YYYY-MM-DD') || ' ' || nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                                                                                         ,paa.time_projected_end)
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,paa.date_start),'YYYY-MM-DD') || ' ' || nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                                                                                 ,paa.time_start) || '_' ||
                          to_char(nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,paa.date_end),'YYYY-MM-DD') || ' ' || nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                                                                               ,paa.time_end)
                     ELSE null
                END) date_time_range
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,paa.date_projected_start),'YYYY-MM-DD') || '_' ||
                          to_char(nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,paa.date_projected_end),'YYYY-MM-DD')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,paa.date_start),'YYYY-MM-DD') || '_' ||
                          to_char(nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,paa.date_end),'YYYY-MM-DD')
                     ELSE null
                END) date_range
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,paa.date_projected_start),'YYYY-MM-DD')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,paa.date_start),'YYYY-MM-DD')
                     ELSE null
                END) startdate
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,paa.date_projected_end),'YYYY-MM-DD')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,paa.date_end),'YYYY-MM-DD')
                     ELSE null
                END) enddate
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                             ,paa.time_projected_start)
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                             ,paa.time_start)
                     ELSE null
                END) starttime
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                             ,paa.time_projected_end)
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'TimeEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                             ,paa.time_end)
                     ELSE null
                END) endtime
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateNotification', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                  ,paa.date_notification) date_notification
              ,nvl((xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceDays', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                  ,paa.absence_days) duration_days
              ,nvl((xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceHours', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                  ,paa.absence_hours) duration_hours
              ,(SELECT paat.absence_category
                FROM   per_abs_attendance_types_vl paat
                WHERE  paat.absence_attendance_type_id = nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceTypeId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                                            ,paa.absence_attendance_type_id)) absence_category_code
              ,(SELECT fcl.meaning
                FROM   per_abs_attendance_types_vl paat
                      ,hr_lookups fcl
                WHERE  paat.absence_attendance_type_id = nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceTypeId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                                            ,paa.absence_attendance_type_id)
                AND    paat.absence_category           = fcl.lookup_code(+)
                AND    fcl.lookup_type(+)              = 'ABSENCE_CATEGORY') absence_category
              ,nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'ObjectVersionNumber', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                  ,paa.object_version_number) object_version_number
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category) attribute_category
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute1', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute1) attribute1
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE1'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute1', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute1)) attribute1_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute2', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute2) attribute2
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE2'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute2', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute2)) attribute2_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute3', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute3) attribute3
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE3'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute3', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute3)) attribute3_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute4', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute4) attribute4
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE4'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute4', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute4)) attribute4_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute5', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute5) attribute5
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE5'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute5', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute5)) attribute5_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute6', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute6) attribute6
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE6'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute6', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute6)) attribute6_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute7', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute7) attribute7
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE7'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute7', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute7)) attribute7_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute8', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute8) attribute8
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE8'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute8', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute8)) attribute8_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute9', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute9) attribute9
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE9'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute9', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute9)) attribute9_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute10', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute10) attribute10
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE10'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute10', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute10)) attribute10_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute11', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute11) attribute11
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE11'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute11', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute11)) attribute11_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute12', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute12) attribute12
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE12'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute12', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute12)) attribute12_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute13', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute13) attribute13
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE13'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute13', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute13)) attribute13_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute14', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute14) attribute14
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE14'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute14', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute14)) attribute14_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute15', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute15) attribute15
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE15'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute15', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute15)) attribute15_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute16', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute16) attribute16
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE16'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute16', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute16)) attribute16_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute17', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute17) attribute17
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE17'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute17', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute17)) attribute17_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute18', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute18) attribute18
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE18'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute18', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute18)) attribute18_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute19', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute19) attribute19
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE19'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute19', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute19)) attribute19_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute20', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute20) attribute20
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE20'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute20', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute20)) attribute20_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category) abs_information_category
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation1', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information1) abs_information1
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION1'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation1', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information1)) abs_information1_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation2', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information2) abs_information2
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION2'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation2', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information2)) abs_information2_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation3', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information3) abs_information3
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION3'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation3', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information3)) abs_information3_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation4', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information4) abs_information4
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION4'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation4', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information4)) abs_information4_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation5', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information5) abs_information5
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION5'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation5', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information5)) abs_information5_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation6', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information6) abs_information6
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION6'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation6', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information6)) abs_information6_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation7', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information7) abs_information7
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION7'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation7', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information7)) abs_information7_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation8', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information8) abs_information8
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION8'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation8', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information8)) abs_information8_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation9', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information9) abs_information9
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION9'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation9', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information9)) abs_information9_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation10', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information10) abs_information10
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION10'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation10', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information10)) abs_information10_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation11', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information11) abs_information11
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION11'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation11', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information11)) abs_information11_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation12', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information12) abs_information12
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION12'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation12', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information12)) abs_information12_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation13', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information13) abs_information13
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION13'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation13', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information13)) abs_information13_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation14', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information14) abs_information14
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION14'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation14', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information14)) abs_information14_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation15', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information15) abs_information15
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION15'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation15', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information15)) abs_information15_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation16', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information16) abs_information16
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION16'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation16', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information16)) abs_information16_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation17', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information17) abs_information17
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION17'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation17', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information17)) abs_information17_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation18', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information18) abs_information18
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION18'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation18', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information18)) abs_information18_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation19', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information19) abs_information19
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION19'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation19', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information19)) abs_information19_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation20', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information20) abs_information20
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION20'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation20', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information20)) abs_information20_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation21', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information21) abs_information21
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION21'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation21', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information21)) abs_information21_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation22', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information22) abs_information22
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION22'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation22', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information22)) abs_information22_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation23', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information23) abs_information23
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION23'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation23', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information23)) abs_information23_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation24', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information24) abs_information24
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION24'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation24', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information24)) abs_information24_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation25', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information25) abs_information25
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION25'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation25', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information25)) abs_information25_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation26', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information26) abs_information26
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION26'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation26', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information26)) abs_information26_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation27', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information27) abs_information27
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION27'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation27', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information27)) abs_information27_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation28', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information28) abs_information28
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION28'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation28', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information28)) abs_information28_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation29', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information29) abs_information29
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION29'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation29', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information29)) abs_information29_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation30', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information30) abs_information30
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION30'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation30', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information30)) abs_information30_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Comments', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') comments_to_approver
              ,null                                      authorising_person_id
              ,nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'ReplacementPersonId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                  ,paa.replacement_person_id) replacement_person_id
              ,null                                      period_of_incapacity_id
              ,null                                      occurrence
              ,null                                      ssp1_issued 
              ,null                                      last_update_date
              ,null                                      last_updated_by
              ,null                                      last_update_login
              ,null                                      created_by
              ,null                                      creation_date
              ,null                                      maternity_id
              ,null                                      sickness_start_date
              ,null                                      sickness_end_date
              ,null                                      pregnancy_related_illness 
              ,null                                      reason_for_notification_delay 
              ,null                                      accept_late_notification_flag
              ,null                                      linked_absence_id
              ,null                                      batch_id
              ,null                                      comments       
              ,null                                      absence_case_id
              ,null                                      approval_status
              ,null                                      confirmed_until
              ,null                                      source
              ,null                                      advance_pay
              ,hat.transaction_id approvers
              ,self.person_id as employees
              )
			,CASE WHEN xxas_com_sec_pkg.test_instance('XXAS_ABS_VAC_RULE_SETUP',paa.person_id) = 'Y' THEN
			   xxas_absence_event_pkg.get_vacation_rules_xml(paa.person_id,paa.date_start,paa.date_end)
			 END
            ,cast
                (multiset
                  (SELECT xxas_attachment_type
                            (attach.attached_document_id
                            ,attach.sequence_number
                            ,attach.entity_name
                            ,attach.pk1_value
                            ,attach.pk2_value
                            ,attach.pk3_value
                            ,attach.pk4_value
                            ,attach.pk5_value
                            ,attach.category_code
                            ,attach.category_name
                            ,attach.document_id
                            ,attach.file_name
                            ,attach.title
                            ,attach.description
                            ,attach.file_id
                            ,attach.mime_type
                            ,attach.lob_format
                            ,attach.file_url
                            ,attach.wp_link
                            ,attach.creation_date
                            ,attach.created_by
                            ,attach.last_update_date
                            ,attach.last_updated_by
                            )
                   FROM   TABLE(xxas_cal_util_pkg.attachments_data
                                  (p_attachment_id           => null
                                  ,p_entity_name             => 'PER_ABSENCE_ATTENDANCES'
                                  ,p_pk1_value               => to_char(paa.absence_attendance_id || '_' || hat.transaction_id)
                                  ,p_pk2_value               => null
                                  ,p_pk3_value               => null
                                  ,p_pk4_value               => null
                                  ,p_pk5_value               => null
                                  ,p_transaction_id          => hat.transaction_id
                                  ,p_transaction_entity_name => 'PQH_SS_ATTACHMENT')) attach
                  )
                 AS xxas_attachment_table
                ) attachments
            )
    FROM   per_absence_attendances      paa
          ,per_abs_attendance_types_vl  paat
          ,hr_lookups                   fcl
          ,hr_api_transactions          hat
          ,hr_api_transaction_steps     hats
    WHERE  paa.absence_attendance_type_id  = paat.absence_attendance_type_id
    AND    paat.absence_category           = fcl.lookup_code(+)
    AND    fcl.lookup_type(+)              = 'ABSENCE_CATEGORY'
    AND    paa.person_id                   = self.person_id
    AND    paa.absence_attendance_id       = self.abs_attendance_id
    AND    paa.person_id                   = hat.selected_person_id
	AND    hat.transaction_id              = self.transaction_id
    AND    hat.status          NOT IN ('D', 'N', 'W', 'ACTIVE', 'AC')
    AND    hat.transaction_id              = hats.transaction_id
    AND    hats.transaction_step_id        = self.transaction_step_id
    AND    hats.api_name                   = 'HR_PERSON_ABSENCE_SWI.PROCESS_API';
    /*AND    paa.absence_attendance_id       = xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')*/

    CURSOR csr_approver_person IS
    SELECT distinct mgra.person_id
    FROM   TABLE(xxas_com_hier_pkg.mgrs_for_asg_tbl
                   (self.resource_id, trunc(sysdate))) mgra;

    CURSOR csr_asg_data IS
    SELECT XMLElement
            ("EMPLOYEES"
            ,XMLElement
               ("value"
               ,XMLAttributes
                 ('true' as "json:force-array"
                 ,'http://json.org/' as "xmlns:json")
               ,XMLAgg
                 (XMLElement
                   ("value"
                   ,XMLForest
                     (p.assignment_id "id"
                     --,p.short_name "value"
                     ,p.short_name "label"
                     ,xxas_util_pkg.flag_to_boolean('N') "deletable")
                   ,XMLElement
                     ("linked"
                     ,XMLElement
                       ("image"
                       ,XMLForest
                         (xxas_com_rest_util_pkg.uri_from_path
                            ('people/' || p.img_person_id || '/photo') "self"
                         ,p.initials "initials"
                         ,xxas_per_image_pkg.url
                            (p.img_person_id, p.img_last_update_date, 'SMALL') "small"
                         )
                       )
                     )
                   )
                 )
               ).getClobVal()
            )
    FROM  (SELECT paaf.assignment_id
                 ,xxas_util_pkg.get_formatted_name
                    (papf.full_name
                    ,papf.global_name
                    ,papf.local_name
                    ,papf.first_name
                    ,papf.last_name
                    ,papf.known_as) short_name
                 ,xxas_util_pkg.get_initials(papf.first_name, papf.last_name, papf.known_as) initials
                 ,img.person_id              img_person_id
                 ,img.last_update_date       img_last_update_date
           FROM   per_all_people_f papf
                 ,per_all_assignments_f paaf
                 ,xxas_per_thumbnails img
                 ,xxas_tcm_log_entry_approvers_v xtla
           WHERE  paaf.assignment_id = self.resource_id
           AND    paaf.person_id     = papf.person_id
           AND    trunc(sysdate) BETWEEN papf.effective_start_date 
                                 AND     papf.effective_end_date
           AND    xtla.log_entry_id  = self.log_entry_id
           AND    xtla.assignment_id = self.resource_id
           AND    trunc(xtla.start_date_time) BETWEEN paaf.effective_start_date
                                              AND     paaf.effective_end_date
           AND    papf.person_id = img.person_id(+)) p;


  CURSOR csr_time_entry_values(cp_assignment_id IN number
                              ,cp_log_entry_id  IN number) IS
  SELECT trunc(xtle.start_date_time) date_start
        ,xtle.element_type_id
  FROM   xxas_tcm_log_entries xtle
  WHERE  xtle.log_entry_id  = cp_log_entry_id
  AND    xtle.assignment_id = cp_assignment_id;

  CURSOR csr_field_values(cp_assignment_id   IN number
                         ,cp_effective_date  IN date
                         ,cp_element_type_id IN number) IS
  SELECT xxas_com_elements_pkg.get_iv_uom
           (eit.eei_information10, cp_effective_date) field1_datatype
        ,xxas_com_elements_pkg.get_iv_uom
           (eit.eei_information11, cp_effective_date) field2_datatype
        ,xxas_com_elements_pkg.get_iv_uom
           (eit.eei_information12, cp_effective_date) field3_datatype
        ,xxas_com_elements_pkg.get_iv_uom
           (eit.eei_information13, cp_effective_date) field4_datatype
        ,xxas_com_elements_pkg.get_iv_uom
           (eit.eei_information14, cp_effective_date) field5_datatype
  FROM   pay_element_types_f pet
        ,TABLE(xxas_com_elements_pkg.links_tbl(cp_assignment_id,cp_effective_date)) e
        ,pay_element_type_extra_info eit
  WHERE  pet.element_type_id = cp_element_type_id
  AND    pet.element_type_id = e.element_type_id
  AND    e.element_type_id   = eit.element_type_id
  AND    eit.information_type = 'XXAS_TCM_ENTRY_MAP'
  AND    eit.eei_information1 is not null
  AND    eit.eei_information1 = 'Y'
  AND    ((nvl(eit.eei_information20,'N') = 'N' AND nvl(eit.eei_information21,'N') = 'N')
          OR ((nvl(eit.eei_information20,'N') = 'Y' OR nvl(eit.eei_information21,'N') = 'Y')
              AND fnd_global.employee_id not in (SELECT person_id 
                                                 FROM   per_all_assignments_f 
                                                 WHERE  assignment_id = cp_assignment_id)
             )
         )
  AND    cp_effective_date BETWEEN pet.effective_start_date
                           AND     pet.effective_end_date;

    CURSOR csr_tcm_period_id IS
    SELECT xtle.period_id
    FROM   xxas_tcm_log_entry_states_v xtle
    WHERE  xtle.person_id     = self.person_id
    AND    xtle.assignment_id = self.resource_id
    AND    xtle.log_entry_id  = self.log_entry_id;

    CURSOR csr_tcm_event_edit_xml(cp_business_group_id  IN number
                                 ,cp_today_log_summary  IN varchar2
                                 ,cp_total_log_summary  IN varchar2) IS
    SELECT XMLElement
            ("ROW"
            ,XMLForest
              (self.person_id person_id
              ,self.resource_id assignment_id
              ,self.log_entry_id log_entry_id
              ,self.source_block source_block
              ,self.saved_search_id saved_search_id
              ,xxas_cal_util_pkg.costable_type(xtla.assignment_id
                                              ,nvl(trunc(xtla.start_date_time),trunc(sysdate))
                                              ,xtla.element_type_id) costable_type
              ,xxas_cal_util_pkg.source_block_called_view(self.source_block) as called_view
              ,self.resource_id as job_id
              ,trunc(xtla.start_date_time) date_start
              ,'DATE_START' as date_start_type
              ,trunc(xtla.start_date_time) as dr_date_start
              ,trunc(xtla.start_date_time) || '_' || trunc(xtla.end_date_time) as date_range
              ,cp_today_log_summary as today_logged_summary
              ,cp_total_log_summary as total_logged_summary
              ,trunc(xtla.end_date_time) date_end
              ,xtla.element_type_id payment_type_id
              ,to_char(xtla.start_date_time,'HH24:MI') time_start
              ,to_char(xtla.end_date_time,'HH24:MI') time_stop
              ,to_char(xtla.start_date_time,'HH24:MI') || '_' || to_char(xtla.end_date_time,'HH24:MI') time_range
              ,xtla.duration_hours duration
              ,'N' repeat_entry
              ,(CASE WHEN l_field1_data_type IN ('H_HHMM','H_HHMMSS') AND xtla.field1 is not null
                     THEN to_char(trunc(sysdate) + xtla.field1/24, 'HH24:MI')
                     ELSE xtla.field1
                END) field1
              ,(CASE WHEN l_field2_data_type IN ('H_HHMM','H_HHMMSS') AND xtla.field2 is not null 
                     THEN to_char(trunc(sysdate) + xtla.field2/24, 'HH24:MI')
                     ELSE xtla.field2
                END) field2
              ,(CASE WHEN l_field3_data_type IN ('H_HHMM','H_HHMMSS') AND xtla.field3 is not null
                     THEN to_char(trunc(sysdate) + xtla.field3/24, 'HH24:MI')
                     ELSE xtla.field3
                END) field3
              ,(CASE WHEN l_field4_data_type IN ('H_HHMM','H_HHMMSS') AND xtla.field4 is not null 
                     THEN to_char(trunc(sysdate) + xtla.field4/24, 'HH24:MI')
                     ELSE xtla.field4
                END) field4
              ,(CASE WHEN l_field5_data_type IN ('H_HHMM','H_HHMMSS') AND xtla.field5 is not null 
                     THEN to_char(trunc(sysdate) + xtla.field5/24, 'HH24:MI')
                     ELSE xtla.field5
                END) field5
              ,CASE WHEN xtla.override_approver_role is not null 
                    THEN xtla.override_approver_role
                    ELSE xxas_util_pkg.get_username_from_per(l_approver_person_id,trunc(sysdate))
               END override_approver
              ,null comments_to_approver
              ,'COST' cost_all_ff_code
              ,xxas_cal_util_pkg.cost_all_key_ff_structure_code
                 (nvl(cp_business_group_id,to_number(fnd_profile.value('PER_BUSINESS_GROUP_ID')))
                 ,'COST'
                 ,xxas_util_pkg.get_app_id('PAY')) cost_all_ff_st_code
              ,xtla.cost_allocation_keyflex_id
              ,ckf.segment1
              ,ckf.segment1_value
              ,ckf.segment1_meaning
              ,ckf.segment2
              ,ckf.segment2_value
              ,ckf.segment2_meaning
              ,ckf.segment3
              ,ckf.segment3_value
              ,ckf.segment3_meaning
              ,ckf.segment4
              ,ckf.segment4_value
              ,ckf.segment4_meaning
              ,ckf.segment5
              ,ckf.segment5_value
              ,ckf.segment5_meaning
              ,ckf.segment6
              ,ckf.segment6_value
              ,ckf.segment6_meaning
              ,ckf.segment7
              ,ckf.segment7_value
              ,ckf.segment7_meaning
              ,ckf.segment8
              ,ckf.segment8_value
              ,ckf.segment8_meaning
              ,ckf.segment9
              ,ckf.segment9_value
              ,ckf.segment9_meaning
              ,ckf.segment10
              ,ckf.segment10_value
              ,ckf.segment10_meaning
              ,ckf.segment11
              ,ckf.segment11_value
              ,ckf.segment11_meaning
              ,ckf.segment12
              ,ckf.segment12_value
              ,ckf.segment12_meaning
              ,ckf.segment13
              ,ckf.segment13_value
              ,ckf.segment13_meaning
              ,ckf.segment14
              ,ckf.segment14_value
              ,ckf.segment14_meaning
              ,ckf.segment15
              ,ckf.segment15_value
              ,ckf.segment15_meaning
              ,ckf.segment16
              ,ckf.segment16_value
              ,ckf.segment16_meaning
              ,ckf.segment17
              ,ckf.segment17_value
              ,ckf.segment17_meaning
              ,ckf.segment18
              ,ckf.segment18_value
              ,ckf.segment18_meaning
              ,ckf.segment19
              ,ckf.segment19_value
              ,ckf.segment19_meaning
              ,ckf.segment20
              ,ckf.segment20_value
              ,ckf.segment20_meaning
              ,ckf.segment21
              ,ckf.segment21_value
              ,ckf.segment21_meaning
              ,ckf.segment22
              ,ckf.segment22_value
              ,ckf.segment22_meaning
              ,ckf.segment23
              ,ckf.segment23_value
              ,ckf.segment23_meaning
              ,ckf.segment24
              ,ckf.segment24_value
              ,ckf.segment24_meaning
              ,ckf.segment25
              ,ckf.segment25_value
              ,ckf.segment25_meaning
              ,ckf.segment26
              ,ckf.segment26_value
              ,ckf.segment26_meaning
              ,ckf.segment27
              ,ckf.segment27_value
              ,ckf.segment27_meaning
              ,ckf.segment28
              ,ckf.segment28_value
              ,ckf.segment28_meaning
              ,ckf.segment29
              ,ckf.segment29_value
              ,ckf.segment29_meaning
              ,ckf.segment30
              ,ckf.segment30_value
              ,ckf.segment30_meaning
              )
            ,l_asg_data
            )
    FROM   xxas_tcm_log_entry_approvers_v xtla
          ,xxas_cost_key_ff_v             ckf
    WHERE  xtla.log_entry_id  = self.log_entry_id
    AND    xtla.assignment_id = self.resource_id
    AND    xtla.cost_allocation_keyflex_id = ckf.cost_allocation_keyflex_id(+);

    CURSOR csr_person IS
    SELECT papf.*
    FROM   per_all_people_f papf
    WHERE  papf.person_id = self.person_id
    AND    trunc(sysdate) BETWEEN papf.effective_start_date 
                          AND     papf.effective_end_date
    AND    (nvl(papf.current_employee_flag, 'N') = 'Y'
            OR nvl(papf.current_npw_flag, 'N') = 'Y');

    CURSOR csr_abs_event_dates IS
    SELECT paa.date_projected_end
          ,paa.date_projected_start
          ,paa.time_projected_end
          ,paa.time_projected_start
          ,paa.date_end
          ,paa.date_start
          ,paa.time_end
          ,paa.time_start
    FROM   per_absence_attendances paa
    WHERE  paa.person_id = self.person_id
    AND    paa.absence_attendance_id = self.abs_attendance_id;

    CURSOR csr_abs_event_count IS
    SELECT count(*)
    FROM   per_absence_attendances paa
    WHERE  paa.person_id = self.person_id
    AND    paa.absence_attendance_id = self.abs_attendance_id;

    CURSOR csr_person_data IS
    SELECT XMLElement
            ("EMPLOYEES"
            ,XMLElement
               ("value"
               ,XMLAttributes
                 ('true' as "json:force-array"
                 ,'http://json.org/' as "xmlns:json")
               ,XMLAgg
                 (XMLElement
                   ("value"
                   ,XMLForest
                     (p.person_id "id"
                     ,p.short_name "label"
                     ,xxas_util_pkg.flag_to_boolean('N') "deletable")
                   ,XMLElement
                     ("linked"
                     ,XMLElement
                       ("image"
                       ,XMLForest
                         (xxas_com_rest_util_pkg.uri_from_path
                            ('people/' || p.img_person_id || '/photo') "self"
                         ,p.initials "initials"
                         ,xxas_per_image_pkg.url
                            (p.img_person_id, p.img_last_update_date, 'SMALL') "small"
                         )
                       )
                     )
                   )
                 )
               ).getClobVal()
            )
    FROM  (SELECT papf.person_id
                 ,xxas_util_pkg.get_formatted_name
                    (papf.full_name
                    ,papf.global_name
                    ,papf.local_name
                    ,papf.first_name
                    ,papf.last_name
                    ,papf.known_as) short_name
                 ,xxas_util_pkg.get_initials(papf.first_name, papf.last_name, papf.known_as) initials
                 ,img.person_id              img_person_id
                 ,img.last_update_date       img_last_update_date
           FROM   per_all_people_f papf
                 ,xxas_per_thumbnails img
           WHERE  papf.person_id = self.person_id
           AND    trunc(sysdate) BETWEEN papf.effective_start_date 
                                 AND     papf.effective_end_date
           AND    papf.person_id = img.person_id(+)) p;

    CURSOR csr_otl_timecard_dates IS
    SELECT hts.start_time
          ,hts.stop_time
          ,hts.approval_status
          ,hts.timecard_id
          ,hts.timecard_ovn
          ,htbb_detail.object_version_number
          ,fnd_date.date_to_canonical(htbb_detail.date_from) timecard_date_from
          ,htbb_detail.comment_text comments_to_approver
    FROM   hxc_timecard_summary     hts
          ,hxc_time_building_blocks htbb_time
          ,hxc_time_building_blocks htbb_day
          ,hxc_time_building_blocks htbb_detail
          ,hxc_time_attributes hta
          ,hxc_time_attribute_usages htau
          ,pay_element_types_f petf
          ,per_all_people_f papf
          ,fnd_lookup_values flv
    WHERE  hts.resource_id                     = self.person_id
    AND    hts.timecard_id                     = htbb_time.time_building_block_id
    AND    hts.timecard_ovn                    = htbb_time.object_version_number
    AND    htbb_time.time_building_block_id    = htbb_day.parent_building_block_id
    AND    htbb_time.object_version_number     = htbb_day.parent_building_block_ovn
    AND    htbb_time.scope                     = 'TIMECARD'
    AND    htbb_time.resource_type             = 'PERSON'
    AND    htbb_time.date_to                   = hr_general.end_of_time
    AND    htbb_day.time_building_block_id     = htbb_detail.parent_building_block_id
    AND    htbb_day.object_version_number      = htbb_detail.parent_building_block_ovn
    AND    htbb_day.scope                      = 'DAY'
    AND    htbb_day.date_to                    = hr_general.end_of_time
    AND    htbb_detail.scope                   = 'DETAIL'
    AND    htbb_detail.date_to                 = hr_general.end_of_time
    AND    htbb_detail.time_building_block_id  = self.log_entry_id
    AND    htau.time_building_block_id         = htbb_detail.time_building_block_id
    AND    htau.time_building_block_ovn        = htbb_detail.object_version_number
    AND    htau.time_attribute_id              = hta.time_attribute_id
    AND    to_char(petf.element_type_id)       = (substr(hta.attribute_category,11,length(hta.attribute_category)))
    AND    hta.attribute_category is not null
    AND    hta.attribute_category not like 'SEC%'
    AND    hts.resource_id                     = papf.person_id
    AND    trunc(sysdate) BETWEEN papf.effective_start_date
                          AND     papf.effective_end_date
    AND    (nvl(papf.current_employee_flag, 'N') = 'Y'
            OR nvl(papf.current_npw_flag, 'N')   = 'Y')
    AND     hts.approval_status     = flv.lookup_code
    AND     flv.language            = userenv('LANG')
    AND     flv.view_application_id = 3
    AND     flv.lookup_type         = 'HXC_APPROVAL_STATUS'
    AND     flv.security_group_id   = 0
    AND     fnd_profile.value('XXAS_TIMESTORE') = 'OTL'
    ORDER BY htbb_detail.object_version_number DESC;

    l_per_row              csr_person%ROWTYPE;
    l_date_projected_end   date;
    l_date_projected_start date;
    l_date_end             date;
    l_date_start           date;
    l_time_projected_end   varchar2(10);
    l_time_projected_start varchar2(10);
    l_time_end             varchar2(10);
    l_time_start           varchar2(10);
    l_abs_event_dates_rec  csr_abs_event_dates%ROWTYPE;
    l_count                number;
    l_time_entry_rec       csr_time_entry_values%ROWTYPE;
    l_field_values_rec     csr_field_values%ROWTYPE;
    l_timecard_summary_start_date  date;
    l_timecard_summary_end_date    date;
    l_timecard_status      varchar2(30);
    l_timecard_summary_id  number;
    l_timecard_ovn         number;
    l_timecard_summary_ovn number;
    l_timecard_date_from   varchar2(30);
    l_comments_to_approver varchar2(4000);
    l_period_id            number;
    l_today_log_summary    varchar2(4000);
    l_total_log_summary    varchar2(4000);

    CURSOR csr_otl_event_edit_xml IS
    SELECT XMLElement
            ("ROW"
            ,XMLForest
              (self.person_id person_id
              ,self.resource_id assignment_id
              ,self.log_entry_id timecard_id
              ,l_timecard_summary_id  timecard_summary_id
              ,l_timecard_summary_ovn timecard_summary_ovn
              ,l_timecard_ovn        timecard_ovn
              ,l_timecard_date_from  timecard_date_from
              ,self.source_block source_block
              ,self.saved_search_id saved_search_id
              ,xxas_cal_util_pkg.source_block_called_view(self.source_block) as called_view
              ,self.log_entry_id approvers
              ,l_timecard_summary_start_date timecard_summary_start_date
              ,l_timecard_summary_end_date   timecard_summary_end_date
              ,l_timecard_status     current_state_code
              ,l_comments_to_approver         comments_to_approver
              ,'N' repeat_entry
              )
            ,l_person_data
            ,cast
                (multiset
                  (SELECT xxas_attachment_type
                            (attach.attached_document_id
                            ,attach.sequence_number
                            ,attach.entity_name
                            ,attach.pk1_value
                            ,attach.pk2_value
                            ,attach.pk3_value
                            ,attach.pk4_value
                            ,attach.pk5_value
                            ,attach.category_code
                            ,attach.category_name
                            ,attach.document_id
                            ,attach.file_name
                            ,attach.title
                            ,attach.description
                            ,attach.file_id
                            ,attach.mime_type
                            ,attach.lob_format
                            ,attach.file_url
                            ,attach.wp_link
                            ,attach.creation_date
                            ,attach.created_by
                            ,attach.last_update_date
                            ,attach.last_updated_by
                            )
                   FROM   TABLE(xxas_cal_util_pkg.attachments_data
                                  (p_attachment_id           => null
                                  ,p_entity_name             => 'TIMECARD_ID'
                                  ,p_pk1_value               => to_char(l_timecard_summary_id)
                                  ,p_pk2_value               => null
                                  ,p_pk3_value               => null
                                  ,p_pk4_value               => null
                                  ,p_pk5_value               => null
                                  ,p_transaction_id          => null
                                  ,p_transaction_entity_name => null)) attach
                  )
                  AS xxas_attachment_table
                ) attachments
            )
    FROM   dual;

  BEGIN

    IF self.action_id = 'ABSENCE' THEN

      IF self.transaction_id is null THEN

        OPEN  csr_abs_event_dates;
        FETCH csr_abs_event_dates INTO l_abs_event_dates_rec;
        CLOSE csr_abs_event_dates;

        l_date_projected_end   := l_abs_event_dates_rec.date_projected_end;
        l_date_projected_start := l_abs_event_dates_rec.date_projected_start;
        l_time_projected_end   := l_abs_event_dates_rec.time_projected_end;
        l_time_projected_start := l_abs_event_dates_rec.time_projected_start;
        l_date_end             := l_abs_event_dates_rec.date_end;
        l_date_start           := l_abs_event_dates_rec.date_start;
        l_time_end             := l_abs_event_dates_rec.time_end;
        l_time_start           := l_abs_event_dates_rec.time_start;

        IF l_date_projected_end is not null OR 
           l_date_projected_start is not null OR 
           l_time_projected_end is not null OR 
           l_time_projected_start is not null THEN

          l_leave_status := 'PLANNED';

        ELSIF l_date_end is not null OR 
              l_date_start is not null OR 
              l_time_end is not null OR 
              l_time_start is not null THEN

          l_leave_status := 'CONFIRMED';

        ELSE

          l_leave_status := 'CONFIRMED';

        END IF;

        OPEN  csr_abs_event_edit_xml;
        FETCH csr_abs_event_edit_xml INTO l_xml;
        CLOSE csr_abs_event_edit_xml;

      ELSE

        l_date_projected_end   := nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(self.transaction_id, 'DateProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,l_abs_event_dates_rec.date_projected_end);
        l_date_projected_start := nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(self.transaction_id, 'DateProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,l_abs_event_dates_rec.date_projected_start);
        l_time_projected_end   := nvl(xxas_xml_node_util_pkg.get_node_value(self.transaction_id, 'TimeProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                                     ,l_abs_event_dates_rec.time_projected_end);
        l_time_projected_start := nvl(xxas_xml_node_util_pkg.get_node_value(self.transaction_id, 'TimeProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                                     ,l_abs_event_dates_rec.time_projected_start);
        l_date_end             := nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(self.transaction_id, 'DateEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,l_abs_event_dates_rec.date_end);
        l_date_start           := nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(self.transaction_id, 'DateStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,l_abs_event_dates_rec.date_start);
        l_time_end             := nvl(xxas_xml_node_util_pkg.get_node_value(self.transaction_id, 'TimeEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                                     ,l_abs_event_dates_rec.time_end);
        l_time_start           := nvl(xxas_xml_node_util_pkg.get_node_value(self.transaction_id, 'TimeStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                                     ,l_abs_event_dates_rec.time_start);

        IF ((l_date_projected_end is not null OR 
           l_date_projected_start is not null OR 
           l_time_projected_end is not null OR 
           l_time_projected_start is not null)
           AND l_date_start is null)
           THEN

          l_leave_status := 'PLANNED';

        ELSIF l_date_end is not null OR 
              l_date_start is not null OR 
              l_time_end is not null OR 
              l_time_start is not null THEN

          l_leave_status := 'CONFIRMED';

        ELSE

          l_leave_status := 'CONFIRMED';

        END IF;

        OPEN  csr_abs_event_count;
        FETCH csr_abs_event_count INTO l_count;
        CLOSE csr_abs_event_count;

        IF l_count = 0 THEN

          OPEN  csr_abs_event_tran_in_edit_xml;
          FETCH csr_abs_event_tran_in_edit_xml INTO l_xml;
          CLOSE csr_abs_event_tran_in_edit_xml;

        ELSE

          OPEN  csr_abs_event_tran_edit_xml;
          FETCH csr_abs_event_tran_edit_xml INTO l_xml;
          CLOSE csr_abs_event_tran_edit_xml;

        END IF;

      END IF;

    ELSIF self.action_id = 'TIMECARD' THEN

      OPEN  csr_person;
      FETCH csr_person INTO l_per_row;
      CLOSE csr_person;

      OPEN  csr_approver_person;
      FETCH csr_approver_person INTO l_approver_person_id;
      CLOSE csr_approver_person;

      OPEN  csr_asg_data;
      FETCH csr_asg_data INTO l_asg_data;
      CLOSE csr_asg_data;

      OPEN  csr_time_entry_values(self.resource_id
                                 ,self.log_entry_id);
      FETCH csr_time_entry_values INTO l_time_entry_rec;
      CLOSE csr_time_entry_values;

      OPEN  csr_field_values(self.resource_id
                            ,l_time_entry_rec.date_start
                            ,l_time_entry_rec.element_type_id);
      FETCH csr_field_values INTO l_field_values_rec;
      CLOSE csr_field_values;

      l_field1_data_type := l_field_values_rec.field1_datatype;
      l_field2_data_type := l_field_values_rec.field2_datatype;
      l_field3_data_type := l_field_values_rec.field3_datatype;
      l_field4_data_type := l_field_values_rec.field4_datatype;
      l_field5_data_type := l_field_values_rec.field5_datatype;

      OPEN  csr_tcm_period_id;
      FETCH csr_tcm_period_id INTO l_period_id;
      CLOSE csr_tcm_period_id;

      l_today_log_summary := xxas_timecard_periods_pkg.asg_time_summary_log_on_date
                               (p_assignment_id => self.resource_id
                               ,p_logged_date   => l_time_entry_rec.date_start
                               ,p_asg_tbl       => null);

      l_total_log_summary := xxas_timecard_periods_pkg.asg_time_summary_log_on_period
                               (p_assignment_id => self.resource_id
                               ,p_logged_period => l_period_id
                               ,p_asg_tbl       => null);

      OPEN  csr_tcm_event_edit_xml(l_per_row.business_group_id
                                  ,l_today_log_summary
                                  ,l_total_log_summary);
      FETCH csr_tcm_event_edit_xml INTO l_xml;
      CLOSE csr_tcm_event_edit_xml;

    ELSIF self.action_id = 'OTL' THEN

      OPEN  csr_person_data;
      FETCH csr_person_data INTO l_person_data;
      CLOSE csr_person_data;

      IF self.log_entry_id is not null THEN

        OPEN  csr_otl_timecard_dates;
        FETCH csr_otl_timecard_dates INTO l_timecard_summary_start_date
                                         ,l_timecard_summary_end_date
                                         ,l_timecard_status
                                         ,l_timecard_summary_id
                                         ,l_timecard_summary_ovn
                                         ,l_timecard_ovn
                                         ,l_timecard_date_from
                                         ,l_comments_to_approver;
        CLOSE csr_otl_timecard_dates;

      END IF;

      OPEN  csr_otl_event_edit_xml;
      FETCH csr_otl_event_edit_xml INTO l_xml;
      CLOSE csr_otl_event_edit_xml;

    END IF;

    RETURN l_xml;

  END;

END;
/


CREATE OR REPLACE TYPE BODY xxas_cal_event_drag IS

  CONSTRUCTOR FUNCTION xxas_cal_event_drag
    (p_person_id           IN number
    ,p_resource_id         IN number
    ,p_abs_attendance_id   IN number
    ,p_log_entry_id        IN number
    ,p_transaction_id      IN number
    ,p_transaction_step_id IN number
    ,p_action_id           IN varchar2
    ,p_start_time          IN date
    ,p_end_time            IN date
    ,p_source_block        IN varchar2
    ,p_saved_search_id     IN number
    ,p_manager_id          IN  number default null) RETURN SELF AS RESULT
  IS

  BEGIN

    self.person_id           :=  p_person_id;
    self.resource_id         :=  p_resource_id;
    self.abs_attendance_id   :=  p_abs_attendance_id;
    self.log_entry_id        :=  p_log_entry_id;
    self.transaction_id      :=  p_transaction_id;
    self.transaction_step_id :=  p_transaction_step_id;
    self.action_id           :=  p_action_id;
    self.start_date          :=  trunc(p_start_time);
    self.start_time          :=  to_char(p_start_time,'HH24:MI');
    self.end_date            :=  trunc(p_end_time);
    self.end_time            :=  to_char(p_end_time,'HH24:MI');
    self.start_date_time     :=  p_start_time;
    self.end_date_time       :=  p_end_time;
    self.source_block        :=  p_source_block;
    self.saved_search_id     :=  p_saved_search_id;
    self.manager_id          :=  p_manager_id;

    RETURN;

  END;

  MEMBER FUNCTION get_xml RETURN XMLType
  IS

    l_xml                  XMLType;
    l_leave_status         varchar2(100);
    l_start_time           varchar2(5);
    l_end_time             varchar2(5);
    l_approver_person_id   number;
    l_ol_absences_char     varchar2(4000);
    l_ol_timecards_char    varchar2(4000);
    l_repeat_entry         varchar2(1);
    l_repeats              varchar2(30);
    l_repeat_daily         number;
    l_repeat_weekly        number;
    l_repeat_monthly       number;
    l_ends                 varchar2(30);
    l_occurences           number;
    l_till_end_date        date;
    l_asg_data             XMLType;
    l_person_data          XMLType;
    l_field1_data_type     varchar2(100);
    l_field2_data_type     varchar2(100);
    l_field3_data_type     varchar2(100);
    l_field4_data_type     varchar2(100);
    l_field5_data_type     varchar2(100);
    l_timecard_summary_start_date  date;
    l_timecard_summary_end_date    date;
    l_timecard_status      varchar2(30);
    l_timecard_summary_id  number;
    l_timecard_ovn         number;
    l_timecard_summary_ovn number;
    l_timecard_date_from   varchar2(30);

    CURSOR csr_abs_event_drag_xml IS
    SELECT XMLElement
            ("ROW"
            ,XMLForest
              (paa.absence_attendance_id
              ,paa.object_version_number
              ,paa.business_group_id
              ,self.source_block source_block
              ,self.saved_search_id saved_search_id
              ,self.manager_id as manager_id
              ,xxas_cal_util_pkg.source_block_called_view(self.source_block) as called_view
              ,null status
              ,l_leave_status leave_status
              ,paa.absence_attendance_type_id            absence_type_id
              ,paat.name                                 absence_type
              ,paat.absence_category                     absence_category_code
              ,fcl.meaning                               absence_category
              ,paa.abs_attendance_reason_id
              ,(SELECT rea.meaning     
                FROM   per_abs_attendance_reasons paar
                      ,hr_lookups                 rea
                WHERE  paa.abs_attendance_reason_id   = paar.abs_attendance_reason_id
                AND    paar.name                      = rea.lookup_code
                AND    rea.lookup_type                = 'ABSENCE_REASON') absence_reason 
              ,paa.person_id
              ,xxas_util_pkg.get_primary_worker_asg(paa.person_id,trunc(sysdate)) as assignment_id
              ,l_ol_absences_char ol_absences_li
              ,paa.authorising_person_id
              ,paa.replacement_person_id
              ,paa.period_of_incapacity_id
              ,paa.absence_days duration_days
              ,paa.absence_hours duration_hours
              ,paa.date_notification
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(self.start_date,'YYYY-MM-DD') || ' ' || l_start_time || '_' ||
                          to_char(self.end_date,'YYYY-MM-DD') || ' ' || l_end_time
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(self.start_date,'YYYY-MM-DD') || ' ' || l_start_time || '_' ||
                          to_char(self.end_date,'YYYY-MM-DD') || ' ' || l_end_time
                     ELSE null
                END) date_time_range
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(self.start_date,'YYYY-MM-DD') || '_' ||
                          to_char(self.end_date,'YYYY-MM-DD')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(self.start_date,'YYYY-MM-DD') || '_' ||
                          to_char(self.end_date,'YYYY-MM-DD')
                     ELSE null
                END) date_range
              ,self.start_date as startdate
              ,self.end_date   as enddate
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN l_start_time
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN l_start_time
                     ELSE null
                END) starttime
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN l_end_time
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN l_end_time
                     ELSE null
                END) endtime
              ,paa.occurrence
              ,paa.ssp1_issued
              ,paa.attribute_category
              ,paa.attribute1
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE1'
                 ,paa.attribute1)                        attribute1_meaning 
              ,paa.attribute2
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE2'
                 ,paa.attribute2)                        attribute2_meaning 
              ,paa.attribute3
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE3'
                 ,paa.attribute3)                        attribute3_meaning 
              ,paa.attribute4
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE4'
                 ,paa.attribute4)                        attribute4_meaning 
              ,paa.attribute5
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE5'
                 ,paa.attribute5)                        attribute5_meaning 
              ,paa.attribute6
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE6'
                 ,paa.attribute6)                        attribute6_meaning 
              ,paa.attribute7
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE7'
                 ,paa.attribute7)                        attribute7_meaning 
              ,paa.attribute8
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE8'
                 ,paa.attribute8)                        attribute8_meaning 
              ,paa.attribute9
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE9'
                 ,paa.attribute9)                        attribute9_meaning 
              ,paa.attribute10
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE10'
                 ,paa.attribute10)                      attribute10_meaning 
              ,paa.attribute11
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE11'
                 ,paa.attribute11)                      attribute11_meaning 
              ,paa.attribute12
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE12'
                 ,paa.attribute12)                       attribute12_meaning 
              ,paa.attribute13
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE13'
                 ,paa.attribute13)                       attribute13_meaning 
              ,paa.attribute14
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE14'
                 ,paa.attribute14)                       attribute14_meaning 
              ,paa.attribute15
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE15'
                 ,paa.attribute15)                       attribute15_meaning 
              ,paa.attribute16
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE16'
                 ,paa.attribute16)                       attribute16_meaning 
              ,paa.attribute17
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE17'
                 ,paa.attribute17)                       attribute17_meaning 
              ,paa.attribute18
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE18'
                 ,paa.attribute18)                       attribute18_meaning 
              ,paa.attribute19
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE19'
                 ,paa.attribute19)                       attribute19_meaning 
              ,paa.attribute20
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,paa.attribute_category
                 ,'ATTRIBUTE20'
                 ,paa.attribute20)                       attribute20_meaning 
              ,paa.last_update_date
              ,paa.last_updated_by
              ,paa.last_update_login
              ,paa.created_by
              ,paa.creation_date
              ,paa.maternity_id
              ,paa.sickness_start_date
              ,paa.sickness_end_date
              ,paa.pregnancy_related_illness 
              ,paa.reason_for_notification_delay 
              ,paa.accept_late_notification_flag
              ,paa.linked_absence_id
              ,paa.abs_information_category
              ,paa.abs_information1
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION1'
                 ,paa.abs_information1)                  abs_information1_meaning 
              ,paa.abs_information2
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION2'
                 ,paa.abs_information2)                  abs_information2_meaning
              ,paa.abs_information3
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION3'
                 ,paa.abs_information3)                  abs_information3_meaning
              ,paa.abs_information4
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION4'
                 ,paa.abs_information4)                  abs_information4_meaning
              ,paa.abs_information5
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION5'
                 ,paa.abs_information5)                  abs_information5_meaning
              ,paa.abs_information6
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION6'
                 ,paa.abs_information6)                  abs_information6_meaning
              ,paa.abs_information7
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION7'
                 ,paa.abs_information7)                  abs_information7_meaning
              ,paa.abs_information8
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION8'
                 ,paa.abs_information8)                  abs_information8_meaning
              ,paa.abs_information9
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION9'
                 ,paa.abs_information9)                  abs_information9_meaning
              ,paa.abs_information10
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION10'
                 ,paa.abs_information10)                 abs_information10_meaning
              ,paa.abs_information11
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION11'
                 ,paa.abs_information11)                 abs_information11_meaning
              ,paa.abs_information12
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION12'
                 ,paa.abs_information12)                 abs_information12_meaning
              ,paa.abs_information13
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION13'
                 ,paa.abs_information13)                 abs_information13_meaning
              ,paa.abs_information14
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION14'
                 ,paa.abs_information14)                 abs_information14_meaning
              ,paa.abs_information15
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION15'
                 ,paa.abs_information15)                 abs_information15_meaning
              ,paa.abs_information16
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION16'
                 ,paa.abs_information16)                 abs_information16_meaning
              ,paa.abs_information17
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION17'
                 ,paa.abs_information17)                 abs_information17_meaning
              ,paa.abs_information18
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION18'
                 ,paa.abs_information18)                 abs_information18_meaning
              ,paa.abs_information19
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION19'
                 ,paa.abs_information19)                 abs_information19_meaning
              ,paa.abs_information20
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION20'
                 ,paa.abs_information20)                 abs_information20_meaning
              ,paa.abs_information21
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION21'
                 ,paa.abs_information21)                 abs_information21_meaning
              ,paa.abs_information22
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION22'
                 ,paa.abs_information22)                 abs_information22_meaning
              ,paa.abs_information23
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION23'
                 ,paa.abs_information23)                 abs_information23_meaning
              ,paa.abs_information24
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION24'
                 ,paa.abs_information24)                 abs_information24_meaning
              ,paa.abs_information25
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION25'
                 ,paa.abs_information25)                 abs_information25_meaning
              ,paa.abs_information26
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION26'
                 ,paa.abs_information26)                 abs_information26_meaning
              ,paa.abs_information27
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION27'
                 ,paa.abs_information27)                 abs_information27_meaning
              ,paa.abs_information28
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION28'
                 ,paa.abs_information28)                 abs_information28_meaning
              ,paa.abs_information29
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION29'
                 ,paa.abs_information29)                 abs_information29_meaning
              ,paa.abs_information30
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,paa.abs_information_category
                 ,'ABS_INFORMATION30'
                 ,paa.abs_information30)                 abs_information30_meaning
              ,paa.batch_id
              ,xxas_util_pkg.long_to_varchar2
                 ('PER_ABSENCE_ATTENDANCES'
		         ,'COMMENTS' 
		         ,paa.rowid)                          comments_to_approver    
              ,paa.absence_case_id
              ,paa.approval_status
              ,paa.confirmed_until
              ,paa.source
              ,paa.advance_pay
              ,null transaction_id
              ,null transaction_step_id
              ,self.person_id as employees
              )
            ,cast
                (multiset
                  (SELECT xxas_attachment_type
                            (attach.attached_document_id
                            ,attach.sequence_number
                            ,attach.entity_name
                            ,attach.pk1_value
                            ,attach.pk2_value
                            ,attach.pk3_value
                            ,attach.pk4_value
                            ,attach.pk5_value
                            ,attach.category_code
                            ,attach.category_name
                            ,attach.document_id
                            ,attach.file_name
                            ,attach.title
                            ,attach.description
                            ,attach.file_id
                            ,attach.mime_type
                            ,attach.lob_format
                            ,attach.file_url
                            ,attach.wp_link
                            ,attach.creation_date
                            ,attach.created_by
                            ,attach.last_update_date
                            ,attach.last_updated_by
                            )
                   FROM   TABLE(xxas_cal_util_pkg.attachments_data
                                  (p_attachment_id           => null
                                  ,p_entity_name             => 'PER_ABSENCE_ATTENDANCES'
                                  ,p_pk1_value               => to_char(paa.absence_attendance_id)
                                  ,p_pk2_value               => null
                                  ,p_pk3_value               => null
                                  ,p_pk4_value               => null
                                  ,p_pk5_value               => null
                                  ,p_transaction_id          => null
                                  ,p_transaction_entity_name => null)) attach
                  )
                 AS xxas_attachment_table
                ) attachments
            )
    FROM   per_absence_attendances      paa
          ,per_abs_attendance_types_vl  paat
          ,hr_lookups                   fcl
    WHERE  paa.absence_attendance_type_id  = paat.absence_attendance_type_id
    AND    paat.absence_category           = fcl.lookup_code(+)
    AND    fcl.lookup_type(+)              = 'ABSENCE_CATEGORY'
    AND    paa.person_id                   = self.person_id
    AND    paa.absence_attendance_id       = self.abs_attendance_id;

    CURSOR csr_abs_event_tran_in_drag_xml IS
    SELECT XMLElement
            ("ROW"
            ,XMLForest
              (hat.selected_person_id as person_id
              ,xxas_util_pkg.get_primary_worker_asg(hat.selected_person_id,trunc(sysdate)) as assignment_id
              ,hat.assignment_id as override_assignment_id
              ,l_ol_absences_char ol_absences_li
              ,hat.transaction_ref_id as absence_attendance_id
              ,hat.transaction_id as transaction_id
              ,hats.transaction_step_id as transaction_step_id
              ,self.source_block source_block
              ,self.saved_search_id saved_search_id
              ,self.manager_id as manager_id
              ,xxas_cal_util_pkg.source_block_called_view(self.source_block) as called_view
              ,decode(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAction', 'Transaction/TransCtx/CNode'), 'CreateMode', 'INSERT', 'UpdateMode', 'UPDATE', 'CancelMode', 'DELETE', 'INSERT')  transaction_type
              ,hat.status as hr_api_status_code
              ,xxas_selfservice_workflow_pkg.transcode_status
                 (hat.status) as workflow_status_code
              ,xxas_selfservice_workflow_pkg.decode_status
                 (xxas_selfservice_workflow_pkg.transcode_status
                    (hat.status)) as workflow_status_meaning
              ,l_leave_status leave_status
              ,to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'BusinessGroupId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')) business_group_id
              ,to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceTypeId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')) absence_type_id
              ,(SELECT paat.name
                FROM   per_abs_attendance_types_vl paat
                WHERE  paat.absence_attendance_type_id = to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceTypeId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))) absence_type
              ,to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsAttendanceReasonId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')) abs_attendance_reason_id
             ,(SELECT rea.meaning     
               FROM   per_abs_attendance_reasons paar
                     ,hr_lookups                 rea
               WHERE  paar.abs_attendance_reason_id = to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsAttendanceReasonId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
               AND    paar.name                     = rea.lookup_code
               AND    rea.lookup_type               = 'ABSENCE_REASON') abs_attendance_reason
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(self.start_date,'YYYY-MM-DD') || ' ' || l_start_time || '_' ||
                          to_char(self.end_date,'YYYY-MM-DD') || ' ' || l_end_time
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(self.start_date,'YYYY-MM-DD') || ' ' || l_start_time || '_' ||
                          to_char(self.end_date,'YYYY-MM-DD') || ' ' || l_end_time
                     ELSE null
                END) date_time_range
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(self.start_date,'YYYY-MM-DD') || '_' ||
                          to_char(self.end_date,'YYYY-MM-DD')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(self.start_date,'YYYY-MM-DD') || '_' ||
                          to_char(self.end_date,'YYYY-MM-DD')
                     ELSE null
                END) date_range
              ,self.start_date as startdate
              ,self.end_date   as enddate
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN l_start_time
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN l_start_time
                     ELSE null
                END) starttime
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN l_end_time
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN l_end_time
                     ELSE null
                END) endtime
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateNotification', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') date_notification
              ,(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceDays', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')) duration_days
              ,(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceHours', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')) duration_hours
              ,(SELECT paat.absence_category
                FROM   per_abs_attendance_types_vl paat
                WHERE  paat.absence_attendance_type_id = to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceTypeId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))) absence_category_code
             ,(SELECT fcl.meaning
               FROM   per_abs_attendance_types_vl paat
                     ,hr_lookups fcl
               WHERE  paat.absence_attendance_type_id = to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceTypeId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
               AND    paat.absence_category    = fcl.lookup_code(+)
               AND    fcl.lookup_type(+)       = 'ABSENCE_CATEGORY') absence_category
              ,to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'ObjectVersionNumber', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')) object_version_number
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute_category
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute1', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute1
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE1'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute1', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')) attribute1_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute2', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute2
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE2'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute2', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                    attribute2_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute3', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute3
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE3'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute3', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                    attribute3_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute4', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute4
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE4'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute4', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                    attribute4_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute5', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute5
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE5'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute5', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                    attribute5_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute6', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute6
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE6'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute6', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                    attribute6_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute7', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute7
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE7'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute7', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                    attribute7_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute8', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute8
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE8'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute8', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                    attribute8_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute9', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute9
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE9'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute9', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                    attribute9_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute10', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute10
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE10'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute10', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute10_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute11', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute11
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE11'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute11', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute11_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute12', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute12
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE12'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute12', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute12_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute13', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute13
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE13'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute13', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute13_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute14', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute14
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE14'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute14', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute14_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute15', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute15
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE15'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute15', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute15_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute16', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute16
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE16'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute16', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute16_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute17', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute17
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE17'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute17', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute17_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute18', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute18
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE18'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute18', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute18_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute19', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute19
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE19'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute19', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute19_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute20', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') attribute20
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                 ,'ATTRIBUTE20'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute20', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))                   attribute20_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information_category
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation1', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information1
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 (hat.selected_person_id
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION1'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation1', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information1_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation2', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information2
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION2'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation2', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information2_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation3', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information3
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION3'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation3', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information3_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation4', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information4
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION4'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation4', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information4_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation5', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information5
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION5'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation5', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information5_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation6', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information6
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION6'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation6', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information6_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation7', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information7
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION7'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation7', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information7_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation8', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information8
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION8'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation8', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information8_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation9', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information9
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION9'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation9', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))              abs_information9_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation10', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information10
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION10'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation10', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information10_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation11', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information11
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION11'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation11', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information11_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation12', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information12
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION12'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation12', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information12_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation13', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information13
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION13'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation13', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information13_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation14', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information14
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION14'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation14', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information14_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation15', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information15
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION15'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation15', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information15_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation16', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information16
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION16'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation16', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information16_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation17', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information17
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION17'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation17', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information17_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation18', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information18
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION18'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation18', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information18_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation19', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information19
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION19'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation19', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information19_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation20', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information20
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION20'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation20', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information20_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation21', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information21
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION21'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation21', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information21_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation22', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information22
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION22'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation22', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information22_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation23', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information23
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION23'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation23', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information23_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation24', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information24
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION24'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation24', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information24_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation25', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information25
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION25'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation25', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information25_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation26', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information26
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION26'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation26', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information26_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation27', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information27
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION27'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation27', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information27_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation28', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information28
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION28'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation28', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information28_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation29', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information29
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION29'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation29', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information29_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation30', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') abs_information30
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                 ,'ABS_INFORMATION30'
                 ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation30', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))             abs_information30_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Comments', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') comments_to_approver
              ,null                                      authorising_person_id
              ,to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'ReplacementPersonId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')) replacement_person_id
              ,null                                      period_of_incapacity_id
              ,null                                      occurrence
              ,null                                      ssp1_issued 
              ,null                                      last_update_date
              ,null                                      last_updated_by
              ,null                                      last_update_login
              ,null                                      created_by
              ,null                                      creation_date
              ,null                                      maternity_id
              ,null                                      sickness_start_date
              ,null                                      sickness_end_date
              ,null                                      pregnancy_related_illness 
              ,null                                      reason_for_notification_delay 
              ,null                                      accept_late_notification_flag
              ,null                                      linked_absence_id
              ,null                                      batch_id
              ,null                                      comments       
              ,null                                      absence_case_id
              ,null                                      approval_status
              ,null                                      confirmed_until
              ,null                                      source
              ,null                                      advance_pay
              ,hat.transaction_id approvers
              ,self.person_id as employees
              )
            ,cast
                (multiset
                  (SELECT xxas_attachment_type
                            (attach.attached_document_id
                            ,attach.sequence_number
                            ,attach.entity_name
                            ,attach.pk1_value
                            ,attach.pk2_value
                            ,attach.pk3_value
                            ,attach.pk4_value
                            ,attach.pk5_value
                            ,attach.category_code
                            ,attach.category_name
                            ,attach.document_id
                            ,attach.file_name
                            ,attach.title
                            ,attach.description
                            ,attach.file_id
                            ,attach.mime_type
                            ,attach.lob_format
                            ,attach.file_url
                            ,attach.wp_link
                            ,attach.creation_date
                            ,attach.created_by
                            ,attach.last_update_date
                            ,attach.last_updated_by
                            )
                   FROM   TABLE(xxas_cal_util_pkg.attachments_data
                                  (p_attachment_id           => null
                                  ,p_entity_name             => 'PER_ABSENCE_ATTENDANCES'
                                  ,p_pk1_value               => hat.transaction_ref_id || '_' || hat.transaction_id
                                  ,p_pk2_value               => null
                                  ,p_pk3_value               => null
                                  ,p_pk4_value               => null
                                  ,p_pk5_value               => null
                                  ,p_transaction_id          => hat.transaction_id
                                  ,p_transaction_entity_name => 'PQH_SS_ATTACHMENT')) attach
                  )
                 AS xxas_attachment_table
                ) attachments
            )
    FROM   hr_api_transactions         hat
          ,hr_api_transaction_steps    hats
    WHERE  hat.transaction_id       = hats.transaction_id
    AND    hat.status          NOT IN ('D', 'N', 'W', 'ACTIVE', 'AC')
    AND    hats.api_name            = 'HR_PERSON_ABSENCE_SWI.PROCESS_API'
    AND    decode(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAction', 'Transaction/TransCtx/CNode'), 'CreateMode', 'INSERT') = 'INSERT'
    AND    hat.selected_person_id   = self.person_id
    AND    hat.transaction_id       = self.transaction_id
    AND    hats.transaction_step_id = self.transaction_step_id
    AND    hat.transaction_ref_id   = self.abs_attendance_id;

    CURSOR csr_abs_event_tran_drag_xml IS
    SELECT XMLElement
            ("ROW"
            ,XMLForest
              (paa.person_id
              ,xxas_util_pkg.get_primary_worker_asg(paa.person_id,trunc(sysdate)) as assignment_id
              ,hat.assignment_id as override_assignment_id
              ,l_ol_absences_char ol_absences_li
              ,hat.transaction_ref_id absence_attendance_id
              ,hat.transaction_id transaction_id
              ,hats.transaction_step_id transaction_step_id
              ,self.source_block source_block
              ,self.saved_search_id saved_search_id
              ,self.manager_id as manager_id
              ,xxas_cal_util_pkg.source_block_called_view(self.source_block) as called_view
              ,decode(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAction', 'Transaction/TransCtx/CNode'), 'CreateMode', 'INSERT', 'UpdateMode', 'UPDATE', 'CancelMode', 'DELETE', 'INSERT') transaction_type
              ,hat.status hr_api_status_code
              ,xxas_selfservice_workflow_pkg.transcode_status(hat.status) workflow_status_code
              ,xxas_selfservice_workflow_pkg.decode_status
                 (xxas_selfservice_workflow_pkg.transcode_status(hat.status)) workflow_status_meaning
              ,l_leave_status leave_status
              ,nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'BusinessGroupId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                  ,paa.business_group_id) business_group_id
              ,nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceTypeId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                  ,paa.absence_attendance_type_id) absence_type_id
              ,(SELECT paat.name
                FROM   per_abs_attendance_types_vl paat
                WHERE  paat.absence_attendance_type_id = nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceTypeId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                                            ,paa.absence_attendance_type_id)) absence_type
              ,nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsAttendanceReasonId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                  ,paa.abs_attendance_reason_id) abs_attendance_reason_id
              ,(SELECT rea.meaning     
                FROM   per_abs_attendance_reasons paar
                      ,hr_lookups                 rea
                WHERE  paar.abs_attendance_reason_id = nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsAttendanceReasonId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                                          ,paa.abs_attendance_reason_id)
                AND    paar.name                     = rea.lookup_code
                AND    rea.lookup_type               = 'ABSENCE_REASON') abs_attendance_reason
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(self.start_date,'YYYY-MM-DD') || ' ' || l_start_time || '_' ||
                          to_char(self.end_date,'YYYY-MM-DD') || ' ' || l_end_time
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(self.start_date,'YYYY-MM-DD') || ' ' || l_start_time || '_' ||
                          to_char(self.end_date,'YYYY-MM-DD') || ' ' || l_end_time
                     ELSE null
                END) date_time_range
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN to_char(self.start_date,'YYYY-MM-DD') || '_' ||
                          to_char(self.end_date,'YYYY-MM-DD')
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN to_char(self.start_date,'YYYY-MM-DD') || '_' ||
                          to_char(self.end_date,'YYYY-MM-DD')
                     ELSE null
                END) date_range
              ,self.start_date as startdate
              ,self.end_date   as enddate
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN l_start_time
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN l_start_time
                     ELSE null
                END) starttime
              ,(CASE WHEN l_leave_status = 'PLANNED' 
                     THEN l_end_time
                     WHEN l_leave_status = 'CONFIRMED'
                     THEN l_end_time
                     ELSE null
                END) endtime
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'DateNotification', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                  ,paa.date_notification) date_notification
              ,nvl((xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceDays', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                  ,paa.absence_days) duration_days
              ,nvl((xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceHours', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                  ,paa.absence_hours) duration_hours
              ,(SELECT paat.absence_category
                FROM   per_abs_attendance_types_vl paat
                WHERE  paat.absence_attendance_type_id = nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceTypeId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                                            ,paa.absence_attendance_type_id)) absence_category_code
              ,(SELECT fcl.meaning
                FROM   per_abs_attendance_types_vl paat
                      ,hr_lookups fcl
                WHERE  paat.absence_attendance_type_id = nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceTypeId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                                            ,paa.absence_attendance_type_id)
                AND    paat.absence_category           = fcl.lookup_code(+)
                AND    fcl.lookup_type(+)              = 'ABSENCE_CATEGORY') absence_category
              ,nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'ObjectVersionNumber', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                  ,paa.object_version_number) object_version_number
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category) attribute_category
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute1', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute1) attribute1
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE1'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute1', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute1)) attribute1_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute2', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute2) attribute2
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE2'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute2', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute2)) attribute2_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute3', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute3) attribute3
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE3'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute3', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute3)) attribute3_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute4', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute4) attribute4
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE4'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute4', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute4)) attribute4_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute5', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute5) attribute5
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE5'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute5', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute5)) attribute5_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute6', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute6) attribute6
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE6'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute6', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute6)) attribute6_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute7', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute7) attribute7
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE7'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute7', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute7)) attribute7_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute8', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute8) attribute8
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE8'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute8', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute8)) attribute8_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute9', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute9) attribute9
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE9'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute9', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute9)) attribute9_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute10', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute10) attribute10
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE10'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute10', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute10)) attribute10_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute11', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute11) attribute11
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE11'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute11', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute11)) attribute11_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute12', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute12) attribute12
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE12'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute12', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute12)) attribute12_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute13', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute13) attribute13
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE13'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute13', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute13)) attribute13_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute14', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute14) attribute14
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE14'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute14', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute14)) attribute14_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute15', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute15) attribute15
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE15'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute15', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute15)) attribute15_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute16', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute16) attribute16
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE16'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute16', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute16)) attribute16_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute17', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute17) attribute17
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE17'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute17', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute17)) attribute17_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute18', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute18) attribute18
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE18'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute18', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute18)) attribute18_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute19', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute19) attribute19
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE19'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute19', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute19)) attribute19_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute20', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute20) attribute20
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABSENCE_ATTENDANCES'
                 ,nvl(nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AttributeCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute_category),nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category))
                 ,'ATTRIBUTE20'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Attribute20', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.attribute20)) attribute20_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category) abs_information_category
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation1', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information1) abs_information1
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION1'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation1', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information1)) abs_information1_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation2', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information2) abs_information2
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION2'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation2', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information2)) abs_information2_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation3', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information3) abs_information3
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION3'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation3', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information3)) abs_information3_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation4', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information4) abs_information4
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION4'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation4', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information4)) abs_information4_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation5', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information5) abs_information5
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION5'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation5', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information5)) abs_information5_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation6', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information6) abs_information6
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION6'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation6', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information6)) abs_information6_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation7', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information7) abs_information7
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION7'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation7', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information7)) abs_information7_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation8', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information8) abs_information8
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION8'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation8', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information8)) abs_information8_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation9', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information9) abs_information9
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION9'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation9', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information9)) abs_information9_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation10', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information10) abs_information10
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION10'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation10', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information10)) abs_information10_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation11', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information11) abs_information11
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION11'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation11', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information11)) abs_information11_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation12', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information12) abs_information12
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION12'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation12', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information12)) abs_information12_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation13', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information13) abs_information13
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION13'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation13', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information13)) abs_information13_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation14', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information14) abs_information14
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION14'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation14', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information14)) abs_information14_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation15', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information15) abs_information15
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION15'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation15', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information15)) abs_information15_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation16', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information16) abs_information16
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION16'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation16', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information16)) abs_information16_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation17', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information17) abs_information17
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION17'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation17', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information17)) abs_information17_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation18', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information18) abs_information18
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION18'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation18', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information18)) abs_information18_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation19', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information19) abs_information19
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION19'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation19', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information19)) abs_information19_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation20', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information20) abs_information20
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION20'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation20', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information20)) abs_information20_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation21', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information21) abs_information21
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION21'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation21', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information21)) abs_information21_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation22', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information22) abs_information22
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION22'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation22', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information22)) abs_information22_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation23', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information23) abs_information23
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION23'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation23', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information23)) abs_information23_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation24', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information24) abs_information24
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION24'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation24', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information24)) abs_information24_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation25', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information25) abs_information25
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION25'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation25', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information25)) abs_information25_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation26', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information26) abs_information26
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION26'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation26', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information26)) abs_information26_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation27', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information27) abs_information27
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION27'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation27', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information27)) abs_information27_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation28', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information28) abs_information28
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION28'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation28', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information28)) abs_information28_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation29', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information29) abs_information29
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION29'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation29', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information29)) abs_information29_meaning
              ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation30', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information30) abs_information30
              ,xxas_com_vset_pkg.decode_descr_vset_value
                 ('800'
                 ,'PER_ABS_DEVELOPER_DF'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformationCategory', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information_category)
                 ,'ABS_INFORMATION30'
                 ,nvl(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsInformation30', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'),paa.abs_information30)) abs_information30_meaning
              ,xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'Comments', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow') comments_to_approver
              ,null                                      authorising_person_id
              ,nvl(to_number(xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'ReplacementPersonId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                  ,paa.replacement_person_id) replacement_person_id
              ,null                                      period_of_incapacity_id
              ,null                                      occurrence
              ,null                                      ssp1_issued 
              ,null                                      last_update_date
              ,null                                      last_updated_by
              ,null                                      last_update_login
              ,null                                      created_by
              ,null                                      creation_date
              ,null                                      maternity_id
              ,null                                      sickness_start_date
              ,null                                      sickness_end_date
              ,null                                      pregnancy_related_illness 
              ,null                                      reason_for_notification_delay 
              ,null                                      accept_late_notification_flag
              ,null                                      linked_absence_id
              ,null                                      batch_id
              ,null                                      comments       
              ,null                                      absence_case_id
              ,null                                      approval_status
              ,null                                      confirmed_until
              ,null                                      source
              ,null                                      advance_pay
              ,hat.transaction_id approvers
              ,self.person_id as employees
              )
            ,cast
                (multiset
                  (SELECT xxas_attachment_type
                            (attach.attached_document_id
                            ,attach.sequence_number
                            ,attach.entity_name
                            ,attach.pk1_value
                            ,attach.pk2_value
                            ,attach.pk3_value
                            ,attach.pk4_value
                            ,attach.pk5_value
                            ,attach.category_code
                            ,attach.category_name
                            ,attach.document_id
                            ,attach.file_name
                            ,attach.title
                            ,attach.description
                            ,attach.file_id
                            ,attach.mime_type
                            ,attach.lob_format
                            ,attach.file_url
                            ,attach.wp_link
                            ,attach.creation_date
                            ,attach.created_by
                            ,attach.last_update_date
                            ,attach.last_updated_by
                            )
                   FROM   TABLE(xxas_cal_util_pkg.attachments_data
                                  (p_attachment_id           => null
                                  ,p_entity_name             => 'PER_ABSENCE_ATTENDANCES'
                                  ,p_pk1_value               => to_char(paa.absence_attendance_id || '_' || hat.transaction_id)
                                  ,p_pk2_value               => null
                                  ,p_pk3_value               => null
                                  ,p_pk4_value               => null
                                  ,p_pk5_value               => null
                                  ,p_transaction_id          => hat.transaction_id
                                  ,p_transaction_entity_name => 'PQH_SS_ATTACHMENT')) attach
                  )
                 AS xxas_attachment_table
                ) attachments
            )
    FROM   per_absence_attendances      paa
          ,per_abs_attendance_types_vl  paat
          ,hr_lookups                   fcl
          ,hr_api_transactions          hat
          ,hr_api_transaction_steps     hats
    WHERE  paa.absence_attendance_type_id  = paat.absence_attendance_type_id
    AND    paat.absence_category           = fcl.lookup_code(+)
    AND    fcl.lookup_type(+)              = 'ABSENCE_CATEGORY'
    AND    paa.person_id                   = self.person_id
    AND    paa.absence_attendance_id       = self.abs_attendance_id
    AND    paa.person_id                   = hat.selected_person_id
	AND    hat.transaction_id              = self.transaction_id
    AND    hat.status          NOT IN ('D', 'N', 'W', 'ACTIVE', 'AC')
    AND    hat.transaction_id              = hats.transaction_id
    AND    hats.transaction_step_id        = self.transaction_step_id
    AND    hats.api_name                   = 'HR_PERSON_ABSENCE_SWI.PROCESS_API';
    /*AND    paa.absence_attendance_id       = xxas_xml_node_util_pkg.get_node_value(hat.transaction_id, 'AbsenceAttendanceId', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')*/

    CURSOR csr_approver_person IS
    SELECT distinct mgra.person_id
    FROM   TABLE(xxas_com_hier_pkg.mgrs_for_asg_tbl
                   (self.resource_id, trunc(sysdate))) mgra;

    CURSOR csr_asg_data IS
    SELECT XMLElement
            ("EMPLOYEES"
            ,XMLElement
               ("value"
               ,XMLAttributes
                 ('true' as "json:force-array"
                 ,'http://json.org/' as "xmlns:json")
               ,XMLAgg
                 (XMLElement
                   ("value"
                   ,XMLForest
                     (p.assignment_id "id"
                     --,p.short_name "value"
                     ,p.short_name "label"
                     ,xxas_util_pkg.flag_to_boolean('N') "deletable")
                   ,XMLElement
                     ("linked"
                     ,XMLElement
                       ("image"
                       ,XMLForest
                         (xxas_com_rest_util_pkg.uri_from_path
                            ('people/' || p.img_person_id || '/photo') "self"
                         ,p.initials "initials"
                         ,xxas_per_image_pkg.url
                            (p.img_person_id, p.img_last_update_date, 'SMALL') "small"
                         )
                       )
                     )
                   )
                 )
               ).getClobVal()
            )
    FROM  (SELECT paaf.assignment_id
                 ,xxas_util_pkg.get_formatted_name
                    (papf.full_name
                    ,papf.global_name
                    ,papf.local_name
                    ,papf.first_name
                    ,papf.last_name
                    ,papf.known_as) short_name
                 ,xxas_util_pkg.get_initials(papf.first_name, papf.last_name, papf.known_as) initials
                 ,img.person_id              img_person_id
                 ,img.last_update_date       img_last_update_date
           FROM   per_all_people_f papf
                 ,per_all_assignments_f paaf
                 ,xxas_per_thumbnails img
           WHERE  paaf.assignment_id = self.resource_id
           AND    paaf.person_id     = papf.person_id
           AND    trunc(sysdate) BETWEEN papf.effective_start_date 
                                 AND     papf.effective_end_date
           AND    self.start_date BETWEEN paaf.effective_start_date
                                  AND     paaf.effective_end_date
           AND    papf.person_id = img.person_id(+)) p;

    CURSOR csr_person_data IS
    SELECT XMLElement
            ("EMPLOYEES"
            ,XMLElement
               ("value"
               ,XMLAttributes
                 ('true' as "json:force-array"
                 ,'http://json.org/' as "xmlns:json")
               ,XMLAgg
                 (XMLElement
                   ("value"
                   ,XMLForest
                     (p.person_id "id"
                     ,p.short_name "label"
                     ,xxas_util_pkg.flag_to_boolean('N') "deletable")
                   ,XMLElement
                     ("linked"
                     ,XMLElement
                       ("image"
                       ,XMLForest
                         (xxas_com_rest_util_pkg.uri_from_path
                            ('people/' || p.img_person_id || '/photo') "self"
                         ,p.initials "initials"
                         ,xxas_per_image_pkg.url
                            (p.img_person_id, p.img_last_update_date, 'SMALL') "small"
                         )
                       )
                     )
                   )
                 )
               ).getClobVal()
            )
    FROM  (SELECT papf.person_id
                 ,xxas_util_pkg.get_formatted_name
                    (papf.full_name
                    ,papf.global_name
                    ,papf.local_name
                    ,papf.first_name
                    ,papf.last_name
                    ,papf.known_as) short_name
                 ,xxas_util_pkg.get_initials(papf.first_name, papf.last_name, papf.known_as) initials
                 ,img.person_id              img_person_id
                 ,img.last_update_date       img_last_update_date
           FROM   per_all_people_f papf
                 ,xxas_per_thumbnails img
           WHERE  papf.person_id = self.person_id
           AND    trunc(sysdate) BETWEEN papf.effective_start_date 
                                 AND     papf.effective_end_date
           AND    papf.person_id = img.person_id(+)) p;

  CURSOR csr_time_entry_values(cp_assignment_id IN number
                              ,cp_log_entry_id  IN number) IS
  SELECT trunc(xtle.start_date_time) date_start
        ,xtle.element_type_id
  FROM   xxas_tcm_log_entries xtle
  WHERE  xtle.log_entry_id  = cp_log_entry_id
  AND    xtle.assignment_id = cp_assignment_id;

  CURSOR csr_field_values(cp_assignment_id   IN number
                         ,cp_effective_date  IN date
                         ,cp_element_type_id IN number) IS
  SELECT xxas_com_elements_pkg.get_iv_uom
           (eit.eei_information10, cp_effective_date) field1_datatype
        ,xxas_com_elements_pkg.get_iv_uom
           (eit.eei_information11, cp_effective_date) field2_datatype
        ,xxas_com_elements_pkg.get_iv_uom
           (eit.eei_information12, cp_effective_date) field3_datatype
        ,xxas_com_elements_pkg.get_iv_uom
           (eit.eei_information13, cp_effective_date) field4_datatype
        ,xxas_com_elements_pkg.get_iv_uom
           (eit.eei_information14, cp_effective_date) field5_datatype
        ,nvl(xxas_com_elements_pkg.get_iv_uom
               (eit.eei_information8, cp_effective_date), 'H_DECIMAL1') duration_datatype
  FROM   pay_element_types_f pet
        ,TABLE(xxas_com_elements_pkg.links_tbl(cp_assignment_id,cp_effective_date)) e
        ,pay_element_type_extra_info eit
  WHERE  pet.element_type_id = cp_element_type_id
  AND    pet.element_type_id = e.element_type_id
  AND    e.element_type_id   = eit.element_type_id
  AND    eit.information_type = 'XXAS_TCM_ENTRY_MAP'
  AND    eit.eei_information1 is not null
  AND    eit.eei_information1 = 'Y'
  AND    ((nvl(eit.eei_information20,'N') = 'N' AND nvl(eit.eei_information21,'N') = 'N')
          OR ((nvl(eit.eei_information20,'N') = 'Y' OR nvl(eit.eei_information21,'N') = 'Y')
              AND fnd_global.employee_id not in (SELECT person_id 
                                                 FROM   per_all_assignments_f 
                                                 WHERE  assignment_id = cp_assignment_id)
             )
         )
  AND    cp_effective_date BETWEEN pet.effective_start_date
                           AND     pet.effective_end_date;

    CURSOR csr_timecard_details IS
    SELECT xtle.period_id
          ,xtle.start_date_time
          ,xtle.end_date_time
    FROM   xxas_tcm_log_entry_states_v xtle
    WHERE  xtle.person_id     = self.person_id
    AND    xtle.assignment_id = self.resource_id
    AND    xtle.log_entry_id  = self.log_entry_id;

    CURSOR csr_tcm_event_drag_no_rep_xml(cp_business_group_id IN number
                                        ,cp_today_log_summary  IN varchar2
                                        ,cp_total_log_summary  IN varchar2) IS
    SELECT XMLElement
            ("ROW"
            ,XMLForest
              (self.person_id person_id
              ,self.resource_id assignment_id
              ,self.log_entry_id log_entry_id
              ,self.source_block source_block
              ,self.saved_search_id saved_search_id
              ,xxas_cal_util_pkg.costable_type(xtla.assignment_id
                                              ,nvl(trunc(xtla.start_date_time),trunc(sysdate))
                                              ,xtla.element_type_id) costable_type
              ,xxas_cal_util_pkg.source_block_called_view(self.source_block) as called_view
              ,self.resource_id as job_id
              ,self.start_date date_start
              ,'DATE_START' as date_start_type
              ,self.start_date as dr_date_start
              ,self.start_date || '_' || self.end_date as date_range
              ,cp_today_log_summary as today_logged_summary
              ,cp_total_log_summary as total_logged_summary
              ,self.end_date date_end
              ,xtla.element_type_id payment_type_id
              ,l_start_time time_start
              ,l_end_time time_stop
              ,l_start_time || '_' || l_end_time time_range
              ,(CASE WHEN self.start_time = '00:00' AND self.end_time = '00:00'
                     THEN null
                     ELSE round(to_number(to_number(to_date(self.end_time,'HH24:MI:SS') - to_date(self.start_time,'HH24:MI:SS')) * 24),3)
                END) duration
              ,'N' repeat_entry
              ,(CASE WHEN l_field1_data_type IN ('H_HHMM','H_HHMMSS') AND xtla.field1 is not null 
                     THEN to_char(trunc(sysdate) + xtla.field1/24, 'HH24:MI')
                     ELSE xtla.field1
                END) field1
              ,(CASE WHEN l_field2_data_type IN ('H_HHMM','H_HHMMSS') AND xtla.field2 is not null 
                     THEN to_char(trunc(sysdate) + xtla.field2/24, 'HH24:MI')
                     ELSE xtla.field2
                END) field2
              ,(CASE WHEN l_field3_data_type IN ('H_HHMM','H_HHMMSS') AND xtla.field3 is not null
                     THEN to_char(trunc(sysdate) + xtla.field3/24, 'HH24:MI')
                     ELSE xtla.field3
                END) field3
              ,(CASE WHEN l_field4_data_type IN ('H_HHMM','H_HHMMSS') AND xtla.field4 is not null 
                     THEN to_char(trunc(sysdate) + xtla.field4/24, 'HH24:MI')
                     ELSE xtla.field4
                END) field4
              ,(CASE WHEN l_field5_data_type IN ('H_HHMM','H_HHMMSS') AND xtla.field5 is not null 
                     THEN to_char(trunc(sysdate) + xtla.field5/24, 'HH24:MI')
                     ELSE xtla.field5
                END) field5
              ,CASE WHEN xtla.override_approver_role is not null 
                    THEN xtla.override_approver_role
                    ELSE xxas_util_pkg.get_username_from_per(l_approver_person_id,trunc(sysdate))
               END override_approver
              ,null comments_to_approver
              ,'COST' cost_all_ff_code
              ,xxas_cal_util_pkg.cost_all_key_ff_structure_code
                 (nvl(cp_business_group_id,to_number(fnd_profile.value('PER_BUSINESS_GROUP_ID')))
                 ,'COST'
                 ,xxas_util_pkg.get_app_id('PAY')) cost_all_ff_st_code
              ,xtla.cost_allocation_keyflex_id
              ,ckf.segment1
              ,ckf.segment1_value
              ,ckf.segment1_meaning
              ,ckf.segment2
              ,ckf.segment2_value
              ,ckf.segment2_meaning
              ,ckf.segment3
              ,ckf.segment3_value
              ,ckf.segment3_meaning
              ,ckf.segment4
              ,ckf.segment4_value
              ,ckf.segment4_meaning
              ,ckf.segment5
              ,ckf.segment5_value
              ,ckf.segment5_meaning
              ,ckf.segment6
              ,ckf.segment6_value
              ,ckf.segment6_meaning
              ,ckf.segment7
              ,ckf.segment7_value
              ,ckf.segment7_meaning
              ,ckf.segment8
              ,ckf.segment8_value
              ,ckf.segment8_meaning
              ,ckf.segment9
              ,ckf.segment9_value
              ,ckf.segment9_meaning
              ,ckf.segment10
              ,ckf.segment10_value
              ,ckf.segment10_meaning
              ,ckf.segment11
              ,ckf.segment11_value
              ,ckf.segment11_meaning
              ,ckf.segment12
              ,ckf.segment12_value
              ,ckf.segment12_meaning
              ,ckf.segment13
              ,ckf.segment13_value
              ,ckf.segment13_meaning
              ,ckf.segment14
              ,ckf.segment14_value
              ,ckf.segment14_meaning
              ,ckf.segment15
              ,ckf.segment15_value
              ,ckf.segment15_meaning
              ,ckf.segment16
              ,ckf.segment16_value
              ,ckf.segment16_meaning
              ,ckf.segment17
              ,ckf.segment17_value
              ,ckf.segment17_meaning
              ,ckf.segment18
              ,ckf.segment18_value
              ,ckf.segment18_meaning
              ,ckf.segment19
              ,ckf.segment19_value
              ,ckf.segment19_meaning
              ,ckf.segment20
              ,ckf.segment20_value
              ,ckf.segment20_meaning
              ,ckf.segment21
              ,ckf.segment21_value
              ,ckf.segment21_meaning
              ,ckf.segment22
              ,ckf.segment22_value
              ,ckf.segment22_meaning
              ,ckf.segment23
              ,ckf.segment23_value
              ,ckf.segment23_meaning
              ,ckf.segment24
              ,ckf.segment24_value
              ,ckf.segment24_meaning
              ,ckf.segment25
              ,ckf.segment25_value
              ,ckf.segment25_meaning
              ,ckf.segment26
              ,ckf.segment26_value
              ,ckf.segment26_meaning
              ,ckf.segment27
              ,ckf.segment27_value
              ,ckf.segment27_meaning
              ,ckf.segment28
              ,ckf.segment28_value
              ,ckf.segment28_meaning
              ,ckf.segment29
              ,ckf.segment29_value
              ,ckf.segment29_meaning
              ,ckf.segment30
              ,ckf.segment30_value
              ,ckf.segment30_meaning
              ,l_ol_timecards_char ol_entries_li
              )
            ,l_asg_data
            )
    FROM   xxas_tcm_log_entry_approvers_v xtla
          ,xxas_cost_key_ff_v             ckf
    WHERE  xtla.log_entry_id  = self.log_entry_id
    AND    xtla.assignment_id = self.resource_id
    AND    xtla.cost_allocation_keyflex_id = ckf.cost_allocation_keyflex_id(+);

    CURSOR csr_tcm_event_drag_rep_xml(cp_business_group_id   IN number
                                     ,cp_today_log_summary   IN varchar2
                                     ,cp_total_log_summary   IN varchar2
                                     ,cp_time_entry_duration IN number) IS
    SELECT XMLElement
            ("ROW"
            ,XMLForest
              (self.person_id person_id
              ,self.resource_id assignment_id
              ,self.log_entry_id log_entry_id
              ,self.source_block source_block
              ,self.saved_search_id saved_search_id
              ,xxas_cal_util_pkg.costable_type(xtla.assignment_id
                                              ,nvl(trunc(xtla.start_date_time),trunc(sysdate))
                                              ,xtla.element_type_id) costable_type
              ,xxas_cal_util_pkg.source_block_called_view(self.source_block) as called_view
              ,(CASE WHEN trunc(xtla.start_date_time) <> self.start_date
                     THEN self.start_date
                     WHEN trunc(xtla.end_date_time) <> self.end_date
                     THEN self.start_date + 1
                     ELSE null
                END) date_start
              ,'DATE_START' as date_start_type
              ,(CASE WHEN trunc(xtla.start_date_time) <> self.start_date
                     THEN self.start_date
                     WHEN trunc(xtla.end_date_time) <> self.end_date
                     THEN self.start_date + 1
                     ELSE null
                END) dr_date_start
              ,(CASE WHEN trunc(xtla.start_date_time) <> self.start_date
                     THEN self.start_date
                     WHEN trunc(xtla.end_date_time) <> self.end_date
                     THEN self.start_date + 1
                     ELSE null
                END) || '_' || 
               (CASE WHEN trunc(xtla.start_date_time) <> self.start_date AND trunc(xtla.start_date_time) = trunc(xtla.end_date_time)
                     THEN self.start_date
                     WHEN trunc(xtla.start_date_time) <> self.start_date AND trunc(xtla.start_date_time) <> trunc(xtla.end_date_time)
                     THEN self.start_date + 1
                     WHEN trunc(xtla.end_date_time) <> self.end_date AND trunc(xtla.start_date_time) = trunc(xtla.end_date_time)
                     THEN self.start_date + 1
                     WHEN trunc(xtla.end_date_time) <> self.end_date AND trunc(xtla.start_date_time) <> trunc(xtla.end_date_time)
                     THEN self.start_date + 2
                     ELSE null
                END) as date_range
              ,cp_today_log_summary as today_logged_summary
              ,cp_total_log_summary as total_logged_summary
              ,xtla.element_type_id payment_type_id
              ,l_start_time time_start
              ,l_end_time time_stop
              ,l_start_time || '_' || l_end_time time_range
              ,(CASE WHEN cp_time_entry_duration is not null
                     THEN cp_time_entry_duration			  
                     WHEN self.start_time = '00:00' AND self.end_time = '00:00'
                     THEN null
                     ELSE round(to_number(to_number(to_date(self.end_time,'HH24:MI:SS') - to_date(self.start_time,'HH24:MI:SS')) * 24),3)
                END) duration
              ,'Y' repeat_entry
              ,'D' repeats
              ,1 repeat_daily
              ,'O' ends
              ,(CASE WHEN trunc(xtla.start_date_time) <> self.start_date
                     THEN self.end_date - 1
                     WHEN trunc(xtla.end_date_time) <> self.end_date
                     THEN self.end_date
                     ELSE null
                END) till_end_date
              ,(CASE WHEN l_field1_data_type IN ('H_HHMM','H_HHMMSS') AND xtla.field1 is not null 
                     THEN to_char(trunc(sysdate) + xtla.field1/24, 'HH24:MI')
                     ELSE xtla.field1
                END) field1
              ,(CASE WHEN l_field2_data_type IN ('H_HHMM','H_HHMMSS') AND xtla.field2 is not null 
                     THEN to_char(trunc(sysdate) + xtla.field2/24, 'HH24:MI')
                     ELSE xtla.field2
                END) field2
              ,(CASE WHEN l_field3_data_type IN ('H_HHMM','H_HHMMSS') AND xtla.field3 is not null
                     THEN to_char(trunc(sysdate) + xtla.field3/24, 'HH24:MI')
                     ELSE xtla.field3
                END) field3
              ,(CASE WHEN l_field4_data_type IN ('H_HHMM','H_HHMMSS') AND xtla.field4 is not null 
                     THEN to_char(trunc(sysdate) + xtla.field4/24, 'HH24:MI')
                     ELSE xtla.field4
                END) field4
              ,(CASE WHEN l_field5_data_type IN ('H_HHMM','H_HHMMSS') AND xtla.field5 is not null 
                     THEN to_char(trunc(sysdate) + xtla.field5/24, 'HH24:MI')
                     ELSE xtla.field5
                END) field5
              ,CASE WHEN xtla.override_approver_role is not null 
                    THEN xtla.override_approver_role
                    ELSE xxas_util_pkg.get_username_from_per(l_approver_person_id,trunc(sysdate))
               END override_approver
              ,null comments_to_approver
              ,'COST' cost_all_ff_code
              ,xxas_cal_util_pkg.cost_all_key_ff_structure_code
                 (nvl(cp_business_group_id,to_number(fnd_profile.value('PER_BUSINESS_GROUP_ID')))
                 ,'COST'
                 ,xxas_util_pkg.get_app_id('PAY')) cost_all_ff_st_code
              ,xtla.cost_allocation_keyflex_id
              ,ckf.segment1
              ,ckf.segment1_value
              ,ckf.segment1_meaning
              ,ckf.segment2
              ,ckf.segment2_value
              ,ckf.segment2_meaning
              ,ckf.segment3
              ,ckf.segment3_value
              ,ckf.segment3_meaning
              ,ckf.segment4
              ,ckf.segment4_value
              ,ckf.segment4_meaning
              ,ckf.segment5
              ,ckf.segment5_value
              ,ckf.segment5_meaning
              ,ckf.segment6
              ,ckf.segment6_value
              ,ckf.segment6_meaning
              ,ckf.segment7
              ,ckf.segment7_value
              ,ckf.segment7_meaning
              ,ckf.segment8
              ,ckf.segment8_value
              ,ckf.segment8_meaning
              ,ckf.segment9
              ,ckf.segment9_value
              ,ckf.segment9_meaning
              ,ckf.segment10
              ,ckf.segment10_value
              ,ckf.segment10_meaning
              ,ckf.segment11
              ,ckf.segment11_value
              ,ckf.segment11_meaning
              ,ckf.segment12
              ,ckf.segment12_value
              ,ckf.segment12_meaning
              ,ckf.segment13
              ,ckf.segment13_value
              ,ckf.segment13_meaning
              ,ckf.segment14
              ,ckf.segment14_value
              ,ckf.segment14_meaning
              ,ckf.segment15
              ,ckf.segment15_value
              ,ckf.segment15_meaning
              ,ckf.segment16
              ,ckf.segment16_value
              ,ckf.segment16_meaning
              ,ckf.segment17
              ,ckf.segment17_value
              ,ckf.segment17_meaning
              ,ckf.segment18
              ,ckf.segment18_value
              ,ckf.segment18_meaning
              ,ckf.segment19
              ,ckf.segment19_value
              ,ckf.segment19_meaning
              ,ckf.segment20
              ,ckf.segment20_value
              ,ckf.segment20_meaning
              ,ckf.segment21
              ,ckf.segment21_value
              ,ckf.segment21_meaning
              ,ckf.segment22
              ,ckf.segment22_value
              ,ckf.segment22_meaning
              ,ckf.segment23
              ,ckf.segment23_value
              ,ckf.segment23_meaning
              ,ckf.segment24
              ,ckf.segment24_value
              ,ckf.segment24_meaning
              ,ckf.segment25
              ,ckf.segment25_value
              ,ckf.segment25_meaning
              ,ckf.segment26
              ,ckf.segment26_value
              ,ckf.segment26_meaning
              ,ckf.segment27
              ,ckf.segment27_value
              ,ckf.segment27_meaning
              ,ckf.segment28
              ,ckf.segment28_value
              ,ckf.segment28_meaning
              ,ckf.segment29
              ,ckf.segment29_value
              ,ckf.segment29_meaning
              ,ckf.segment30
              ,ckf.segment30_value
              ,ckf.segment30_meaning
              ,l_ol_timecards_char ol_entries_li
              ,xxas_cal_util_pkg.summary_message
                 ((CASE WHEN trunc(xtla.start_date_time) <> self.start_date
                        THEN self.start_date
                        WHEN trunc(xtla.end_date_time) <> self.end_date
                        THEN self.start_date + 1
                        ELSE null
                   END)
                 ,'Y'
                 ,'D'
                 ,1
                 ,null
                 ,null
                 ,'O'
                 ,null
                 ,(CASE WHEN trunc(xtla.start_date_time) <> self.start_date
                        THEN self.end_date - 1
                        WHEN trunc(xtla.end_date_time) <> self.end_date
                        THEN self.end_date
                        ELSE null
                   END)
                 ,1) summary_msg
              )
            ,l_asg_data
            )
    FROM   xxas_tcm_log_entry_approvers_v xtla
          ,xxas_cost_key_ff_v             ckf
    WHERE  xtla.log_entry_id  = self.log_entry_id
    AND    xtla.assignment_id = self.resource_id
    AND    xtla.cost_allocation_keyflex_id = ckf.cost_allocation_keyflex_id(+);

    CURSOR csr_person IS
    SELECT papf.*
    FROM   per_all_people_f papf
    WHERE  papf.person_id = self.person_id
    AND    trunc(sysdate) BETWEEN papf.effective_start_date 
                          AND     papf.effective_end_date
    AND    (nvl(papf.current_employee_flag, 'N') = 'Y'
            OR nvl(papf.current_npw_flag, 'N') = 'Y');

    CURSOR csr_abs_event_dates IS
    SELECT paa.date_projected_end
          ,paa.date_projected_start
          ,paa.time_projected_end
          ,paa.time_projected_start
          ,paa.date_end
          ,paa.date_start
          ,paa.time_end
          ,paa.time_start
    FROM   per_absence_attendances paa
    WHERE  paa.person_id = self.person_id
    AND    paa.absence_attendance_id = self.abs_attendance_id;

    CURSOR csr_abs_event_count IS
    SELECT count(*)
    FROM   per_absence_attendances paa
    WHERE  paa.person_id = self.person_id
    AND    paa.absence_attendance_id = self.abs_attendance_id;

    CURSOR csr_otl_timecard_details IS
    SELECT hts.start_time timecard_summary_start_time
          ,hts.stop_time timecard_summary_stop_time
          ,hts.approval_status timecard_summary_status
          ,hts.timecard_id timecard_summary_id
          ,hts.timecard_ovn timecard_summary_ovn
          ,htbb_detail.object_version_number timecard_ovn
          ,fnd_date.date_to_canonical(htbb_detail.date_from) timecard_date_from
          ,decode(htbb_detail.type,'RANGE',htbb_detail.start_time,'MEASURE',htbb_day.start_time,null) timecard_start_time
          ,decode(htbb_detail.type,'RANGE',htbb_detail.stop_time,'MEASURE',htbb_day.stop_time,null)  timecard_stop_time
    FROM   hxc_timecard_summary     hts
          ,hxc_time_building_blocks htbb_time
          ,hxc_time_building_blocks htbb_day
          ,hxc_time_building_blocks htbb_detail
          ,hxc_time_attributes hta
          ,hxc_time_attribute_usages htau
          ,pay_element_types_f petf
          ,per_all_people_f papf
          ,fnd_lookup_values flv
    WHERE  hts.resource_id                     = self.person_id
    AND    hts.timecard_id                     = htbb_time.time_building_block_id
    AND    hts.timecard_ovn                    = htbb_time.object_version_number
    AND    htbb_time.time_building_block_id    = htbb_day.parent_building_block_id
    AND    htbb_time.object_version_number     = htbb_day.parent_building_block_ovn
    AND    htbb_time.scope                     = 'TIMECARD'
    AND    htbb_time.resource_type             = 'PERSON'
    AND    htbb_time.date_to                   = hr_general.end_of_time
    AND    htbb_day.time_building_block_id     = htbb_detail.parent_building_block_id
    AND    htbb_day.object_version_number      = htbb_detail.parent_building_block_ovn
    AND    htbb_day.scope                      = 'DAY'
    AND    htbb_day.date_to                    = hr_general.end_of_time
    AND    htbb_detail.scope                   = 'DETAIL'
    AND    htbb_detail.date_to                 = hr_general.end_of_time
    AND    htbb_detail.time_building_block_id  = self.log_entry_id
    AND    htau.time_building_block_id         = htbb_detail.time_building_block_id
    AND    htau.time_building_block_ovn        = htbb_detail.object_version_number
    AND    htau.time_attribute_id              = hta.time_attribute_id
    AND    to_char(petf.element_type_id)       = (substr(hta.attribute_category,11,length(hta.attribute_category)))
    AND    hta.attribute_category is not null
    AND    hta.attribute_category not like 'SEC%'
    AND    hts.resource_id                     = papf.person_id
    AND    trunc(sysdate) BETWEEN papf.effective_start_date
                          AND     papf.effective_end_date
    AND    (nvl(papf.current_employee_flag, 'N') = 'Y'
            OR nvl(papf.current_npw_flag, 'N')   = 'Y')
    AND     hts.approval_status     = flv.lookup_code
    AND     flv.language            = userenv('LANG')
    AND     flv.view_application_id = 3
    AND     flv.lookup_type         = 'HXC_APPROVAL_STATUS'
    AND     flv.security_group_id   = 0
    AND     fnd_profile.value('XXAS_TIMESTORE') = 'OTL'
    ORDER BY htbb_detail.object_version_number DESC;

    l_per_row              csr_person%ROWTYPE;
    l_date_projected_end   date;
    l_date_projected_start date;
    l_date_end             date;
    l_date_start           date;
    l_time_projected_end   varchar2(10);
    l_time_projected_start varchar2(10);
    l_time_end             varchar2(10);
    l_time_start           varchar2(10);
    l_abs_event_dates_rec  csr_abs_event_dates%ROWTYPE;
    l_count                number;
    l_time_entry_rec       csr_time_entry_values%ROWTYPE;
    l_field_values_rec     csr_field_values%ROWTYPE;
    l_period_id            number;
    l_today_log_summary    varchar2(4000);
    l_total_log_summary    varchar2(4000);
    l_timecard_details_rec csr_timecard_details%ROWTYPE;
    l_time_entry_dur_precision number;
    l_time_entry_duration      number(15,3);
    l_time_entry_start_date    date;
    l_time_entry_end_date      date;
    l_otl_timecard_details     csr_otl_timecard_details%ROWTYPE;

    CURSOR csr_otl_event_drag_no_rep_xml(cp_timecard_summary_start_date  IN date
                                        ,cp_timecard_summary_end_date    IN date
                                        ,cp_timecard_summary_status      IN varchar2
                                        ,cp_timecard_summary_id          IN number
                                        ,cp_timecard_summary_ovn         IN number
                                        ,cp_timecard_ovn                 IN number
                                        ,cp_timecard_date_from           IN varchar2) IS
    SELECT XMLElement
            ("ROW"
            ,XMLForest
              (self.person_id person_id
              ,self.resource_id assignment_id
              ,self.log_entry_id timecard_id
              ,cp_timecard_summary_id  timecard_summary_id
              ,cp_timecard_summary_ovn timecard_summary_ovn
              ,cp_timecard_ovn         timecard_ovn
              ,cp_timecard_date_from   timecard_date_from
              ,self.source_block source_block
              ,self.saved_search_id saved_search_id
              ,xxas_cal_util_pkg.source_block_called_view(self.source_block) as called_view
              ,self.log_entry_id approvers
              ,cp_timecard_summary_start_date timecard_summary_start_date
              ,cp_timecard_summary_end_date   timecard_summary_end_date
              ,cp_timecard_summary_status     current_state_code
              ,'N' repeat_entry
              )
            ,l_person_data
            ,cast
                (multiset
                  (SELECT xxas_attachment_type
                            (attach.attached_document_id
                            ,attach.sequence_number
                            ,attach.entity_name
                            ,attach.pk1_value
                            ,attach.pk2_value
                            ,attach.pk3_value
                            ,attach.pk4_value
                            ,attach.pk5_value
                            ,attach.category_code
                            ,attach.category_name
                            ,attach.document_id
                            ,attach.file_name
                            ,attach.title
                            ,attach.description
                            ,attach.file_id
                            ,attach.mime_type
                            ,attach.lob_format
                            ,attach.file_url
                            ,attach.wp_link
                            ,attach.creation_date
                            ,attach.created_by
                            ,attach.last_update_date
                            ,attach.last_updated_by
                            )
                   FROM   TABLE(xxas_cal_util_pkg.attachments_data
                                  (p_attachment_id           => null
                                  ,p_entity_name             => 'TIMECARD_ID'
                                  ,p_pk1_value               => to_char(l_timecard_summary_id)
                                  ,p_pk2_value               => null
                                  ,p_pk3_value               => null
                                  ,p_pk4_value               => null
                                  ,p_pk5_value               => null
                                  ,p_transaction_id          => null
                                  ,p_transaction_entity_name => null)) attach
                  )
                  AS xxas_attachment_table
                ) attachments
            )
    FROM   dual;

    CURSOR csr_otl_event_drag_rep_xml(cp_timecard_summary_start_date  IN date
                                     ,cp_timecard_summary_end_date    IN date
                                     ,cp_timecard_summary_status      IN varchar2
                                     ,cp_timecard_summary_id          IN number
                                     ,cp_timecard_summary_ovn         IN number
                                     ,cp_timecard_ovn                 IN number
                                     ,cp_timecard_date_from           IN varchar2
                                     ,cp_timecard_start_date          IN date
                                     ,cp_timecard_end_date            IN date) IS
    SELECT XMLElement
            ("ROW"
            ,XMLForest
              (self.person_id person_id
              ,self.resource_id assignment_id
              ,self.log_entry_id timecard_id
              ,cp_timecard_summary_id  timecard_summary_id
              ,cp_timecard_summary_ovn timecard_summary_ovn
              ,cp_timecard_ovn         timecard_ovn
              ,cp_timecard_date_from   timecard_date_from
              ,self.source_block source_block
              ,self.saved_search_id saved_search_id
              ,xxas_cal_util_pkg.source_block_called_view(self.source_block) as called_view
              ,self.log_entry_id approvers
              ,cp_timecard_summary_start_date timecard_summary_start_date
              ,cp_timecard_summary_end_date   timecard_summary_end_date
              ,cp_timecard_summary_status     current_state_code
              ,'Y' repeat_entry
              ,'D' repeats
              ,1 repeat_daily
              ,'O' ends
              ,(CASE WHEN trunc(cp_timecard_start_date) <> self.start_date
                     THEN self.end_date - 1
                     WHEN trunc(cp_timecard_end_date) <> self.end_date
                     THEN self.end_date
                     ELSE null
                END) till_end_date
              )
            ,l_person_data
            ,cast
                (multiset
                  (SELECT xxas_attachment_type
                            (attach.attached_document_id
                            ,attach.sequence_number
                            ,attach.entity_name
                            ,attach.pk1_value
                            ,attach.pk2_value
                            ,attach.pk3_value
                            ,attach.pk4_value
                            ,attach.pk5_value
                            ,attach.category_code
                            ,attach.category_name
                            ,attach.document_id
                            ,attach.file_name
                            ,attach.title
                            ,attach.description
                            ,attach.file_id
                            ,attach.mime_type
                            ,attach.lob_format
                            ,attach.file_url
                            ,attach.wp_link
                            ,attach.creation_date
                            ,attach.created_by
                            ,attach.last_update_date
                            ,attach.last_updated_by
                            )
                   FROM   TABLE(xxas_cal_util_pkg.attachments_data
                                  (p_attachment_id           => null
                                  ,p_entity_name             => 'TIMECARD_ID'
                                  ,p_pk1_value               => to_char(l_timecard_summary_id)
                                  ,p_pk2_value               => null
                                  ,p_pk3_value               => null
                                  ,p_pk4_value               => null
                                  ,p_pk5_value               => null
                                  ,p_transaction_id          => null
                                  ,p_transaction_entity_name => null)) attach
                  )
                  AS xxas_attachment_table
                ) attachments
            )
    FROM   dual;

  BEGIN

    l_start_time := CASE WHEN self.start_time = '00:00' AND self.end_time = '00:00'
                         THEN null
                         ELSE self.start_time
                    END;

    l_end_time := CASE WHEN self.start_time = '00:00' AND self.end_time = '00:00'
                       THEN null
                       ELSE self.end_time
                  END;

    IF self.action_id = 'ABSENCE' THEN

      IF self.transaction_id is null THEN

        OPEN  csr_abs_event_dates;
        FETCH csr_abs_event_dates INTO l_abs_event_dates_rec;
        CLOSE csr_abs_event_dates;

        l_date_projected_end   := l_abs_event_dates_rec.date_projected_end;
        l_date_projected_start := l_abs_event_dates_rec.date_projected_start;
        l_time_projected_end   := l_abs_event_dates_rec.time_projected_end;
        l_time_projected_start := l_abs_event_dates_rec.time_projected_start;
        l_date_end             := l_abs_event_dates_rec.date_end;
        l_date_start           := l_abs_event_dates_rec.date_start;
        l_time_end             := l_abs_event_dates_rec.time_end;
        l_time_start           := l_abs_event_dates_rec.time_start;

        IF l_date_projected_end is not null OR 
           l_date_projected_start is not null OR 
           l_time_projected_end is not null OR 
           l_time_projected_start is not null THEN

          l_leave_status := 'PLANNED';

        ELSIF l_date_end is not null OR 
              l_date_start is not null OR 
              l_time_end is not null OR 
              l_time_start is not null THEN

          l_leave_status := 'CONFIRMED';

        ELSE

          l_leave_status := 'CONFIRMED';

        END IF;

        l_ol_absences_char := to_char(self.person_id) || '_' || to_char(self.start_date_time,'YYYY-MM-DD HH24:MI:SS') 
                              || '_' || to_char(self.end_date_time,'YYYY-MM-DD HH24:MI:SS') || '_' || self.source_block
                              || '_' || self.saved_search_id;

        OPEN  csr_abs_event_drag_xml;
        FETCH csr_abs_event_drag_xml INTO l_xml;
        CLOSE csr_abs_event_drag_xml;

      ELSE

        l_date_projected_end   := nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(self.transaction_id, 'DateProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,l_abs_event_dates_rec.date_projected_end);
        l_date_projected_start := nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(self.transaction_id, 'DateProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,l_abs_event_dates_rec.date_projected_start);
        l_time_projected_end   := nvl(xxas_xml_node_util_pkg.get_node_value(self.transaction_id, 'TimeProjectedEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                                     ,l_abs_event_dates_rec.time_projected_end);
        l_time_projected_start := nvl(xxas_xml_node_util_pkg.get_node_value(self.transaction_id, 'TimeProjectedStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                                     ,l_abs_event_dates_rec.time_projected_start);
        l_date_end             := nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(self.transaction_id, 'DateEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,l_abs_event_dates_rec.date_end);
        l_date_start           := nvl(fnd_date.canonical_to_date(xxas_xml_node_util_pkg.get_node_value(self.transaction_id, 'DateStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow'))
                                     ,l_abs_event_dates_rec.date_start);
        l_time_end             := nvl(xxas_xml_node_util_pkg.get_node_value(self.transaction_id, 'TimeEnd', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                                     ,l_abs_event_dates_rec.time_end);
        l_time_start           := nvl(xxas_xml_node_util_pkg.get_node_value(self.transaction_id, 'TimeStart', 'Transaction/TransCache/AM/TXN/EO/PerAbsenceAttendancesEORow')
                                     ,l_abs_event_dates_rec.time_start);

        IF ((l_date_projected_end is not null OR 
           l_date_projected_start is not null OR 
           l_time_projected_end is not null OR 
           l_time_projected_start is not null)
           AND l_date_start is null)
           THEN

          l_leave_status := 'PLANNED';

        ELSIF l_date_end is not null OR 
              l_date_start is not null OR 
              l_time_end is not null OR 
              l_time_start is not null THEN

          l_leave_status := 'CONFIRMED';

        ELSE

          l_leave_status := 'CONFIRMED';

        END IF;

        OPEN  csr_abs_event_count;
        FETCH csr_abs_event_count INTO l_count;
        CLOSE csr_abs_event_count;

        l_ol_absences_char := to_char(self.person_id) || '_' || to_char(self.start_date_time,'YYYY-MM-DD HH24:MI:SS') 
                              || '_' || to_char(self.end_date_time,'YYYY-MM-DD HH24:MI:SS') || '_' || self.source_block
                              || '_' || self.saved_search_id;

        IF l_count = 0 THEN

          OPEN  csr_abs_event_tran_in_drag_xml;
          FETCH csr_abs_event_tran_in_drag_xml INTO l_xml;
          CLOSE csr_abs_event_tran_in_drag_xml;

        ELSE

          OPEN  csr_abs_event_tran_drag_xml;
          FETCH csr_abs_event_tran_drag_xml INTO l_xml;
          CLOSE csr_abs_event_tran_drag_xml;

        END IF;

      END IF;

    ELSIF self.action_id = 'TIMECARD' THEN

      OPEN  csr_person;
      FETCH csr_person INTO l_per_row;
      CLOSE csr_person;

      OPEN  csr_approver_person;
      FETCH csr_approver_person INTO l_approver_person_id;
      CLOSE csr_approver_person;
 
      l_ol_timecards_char := to_char(self.person_id) || '_' || to_char(self.start_date_time,'YYYY-MM-DD HH24:MI:SS') 
                             || '_' || to_char(self.end_date_time,'YYYY-MM-DD HH24:MI:SS');

      OPEN  csr_asg_data;
      FETCH csr_asg_data INTO l_asg_data;
      CLOSE csr_asg_data;

      OPEN  csr_time_entry_values(self.resource_id
                                 ,self.log_entry_id);
      FETCH csr_time_entry_values INTO l_time_entry_rec;
      CLOSE csr_time_entry_values;

      OPEN  csr_field_values(self.resource_id
                            ,l_time_entry_rec.date_start
                            ,l_time_entry_rec.element_type_id);
      FETCH csr_field_values INTO l_field_values_rec;
      CLOSE csr_field_values;

      l_field1_data_type := l_field_values_rec.field1_datatype;
      l_field2_data_type := l_field_values_rec.field2_datatype;
      l_field3_data_type := l_field_values_rec.field3_datatype;
      l_field4_data_type := l_field_values_rec.field4_datatype;
      l_field5_data_type := l_field_values_rec.field5_datatype;

      OPEN  csr_timecard_details;
      FETCH csr_timecard_details INTO l_timecard_details_rec;
      CLOSE csr_timecard_details;

      l_today_log_summary := xxas_timecard_periods_pkg.asg_time_summary_log_on_date
                               (p_assignment_id => self.resource_id
                               ,p_logged_date   => l_time_entry_rec.date_start
                               ,p_asg_tbl       => null);

      l_total_log_summary := xxas_timecard_periods_pkg.asg_time_summary_log_on_period
                               (p_assignment_id => self.resource_id
                               ,p_logged_period => l_timecard_details_rec.period_id
                               ,p_asg_tbl       => null);

      IF ((l_start_time is null AND 
           l_end_time   is null AND 
           self.start_date = self.end_date)
          OR (l_start_time is not null AND
              l_end_time   is not null AND 
              self.start_date = self.end_date)) THEN  --day/month/week timeline drop operation or day drag operation

        OPEN  csr_tcm_event_drag_no_rep_xml(l_per_row.business_group_id
                                           ,l_today_log_summary
                                           ,l_total_log_summary
										   );
        FETCH csr_tcm_event_drag_no_rep_xml INTO l_xml;
        CLOSE csr_tcm_event_drag_no_rep_xml;

      ELSIF (l_start_time is null AND 
             l_end_time   is null AND 
             self.start_date <> self.end_date) THEN  --month/week timeline drag operation

        OPEN  csr_tcm_event_drag_rep_xml(l_per_row.business_group_id
                                        ,l_today_log_summary
                                        ,l_total_log_summary
                                        ,null);
        FETCH csr_tcm_event_drag_rep_xml INTO l_xml;
        CLOSE csr_tcm_event_drag_rep_xml;

      ELSIF (l_start_time is not null AND 
             l_end_time   is not null AND 
             self.start_date <> self.end_date) THEN  --month/week timeline drag operation when times are present on timecard entry

        l_time_entry_dur_precision := CASE WHEN l_field_values_rec.duration_datatype = 'H_DECIMAL1'
                                           THEN 1
                                           WHEN l_field_values_rec.duration_datatype = 'H_DECIMAL2'
                                           THEN 2
                                           WHEN l_field_values_rec.duration_datatype = 'H_DECIMAL3'
                                           THEN 3
                                           WHEN l_field_values_rec.duration_datatype IN ('ND','I')
                                           THEN 0
                                           ELSE null
                                      END;

        l_time_entry_start_date := CASE WHEN trunc(l_timecard_details_rec.start_date_time) <> self.start_date
                                        THEN self.start_date
                                        WHEN trunc(l_timecard_details_rec.end_date_time) <> self.end_date
                                        THEN self.start_date + 1
                                        ELSE null
                                   END;

        l_time_entry_end_date := CASE WHEN trunc(l_timecard_details_rec.start_date_time) <> self.start_date AND 
                                           trunc(l_timecard_details_rec.start_date_time) = trunc(l_timecard_details_rec.end_date_time)
                                      THEN self.start_date
                                      WHEN trunc(l_timecard_details_rec.start_date_time) <> self.start_date AND 
                                           trunc(l_timecard_details_rec.start_date_time) <> trunc(l_timecard_details_rec.end_date_time)
                                      THEN self.start_date + 1
                                      WHEN trunc(l_timecard_details_rec.end_date_time) <> self.end_date AND 
                                           trunc(l_timecard_details_rec.start_date_time) = trunc(l_timecard_details_rec.end_date_time)
                                      THEN self.start_date + 1
                                      WHEN trunc(l_timecard_details_rec.end_date_time) <> self.end_date AND 
                                           trunc(l_timecard_details_rec.start_date_time) <> trunc(l_timecard_details_rec.end_date_time)
                                      THEN self.start_date + 2
                                      ELSE null
                                 END;

        l_time_entry_duration := CASE WHEN self.start_time = '00:00' AND self.end_time = '00:00'
                                      THEN null
                                      ELSE round(to_number(to_number(to_date(to_char(nvl(l_time_entry_end_date,l_time_entry_start_date),'YYYY-MM-DD') || ' ' || self.end_time || ':00','YYYY-MM-DD HH24:MI:SS') - 
                                           to_date(to_char(l_time_entry_start_date,'YYYY-MM-DD') || ' ' || self.start_time || ':00','YYYY-MM-DD HH24:MI:SS')) * 24),l_time_entry_dur_precision)
                                 END;

        OPEN  csr_tcm_event_drag_rep_xml(l_per_row.business_group_id
                                        ,l_today_log_summary
                                        ,l_total_log_summary
                                        ,l_time_entry_duration);
        FETCH csr_tcm_event_drag_rep_xml INTO l_xml;
        CLOSE csr_tcm_event_drag_rep_xml;

      END IF;

    ELSIF self.action_id = 'OTL' THEN

      OPEN  csr_person_data;
      FETCH csr_person_data INTO l_person_data;
      CLOSE csr_person_data;

      IF self.log_entry_id is not null THEN

        OPEN  csr_otl_timecard_details;
        FETCH csr_otl_timecard_details INTO l_otl_timecard_details;
        CLOSE csr_otl_timecard_details;

      END IF;

      IF ((l_start_time is null AND 
           l_end_time   is null AND 
           self.start_date = self.end_date)
          OR (l_start_time is not null AND
              l_end_time   is not null AND 
              self.start_date = self.end_date)) THEN  --day/month/week timeline drop operation or day drag operation

        OPEN  csr_otl_event_drag_no_rep_xml(l_otl_timecard_details.timecard_summary_start_time
                                           ,l_otl_timecard_details.timecard_summary_stop_time
                                           ,l_otl_timecard_details.timecard_summary_status
                                           ,l_otl_timecard_details.timecard_summary_id
                                           ,l_otl_timecard_details.timecard_summary_ovn
                                           ,l_otl_timecard_details.timecard_ovn
                                           ,l_otl_timecard_details.timecard_date_from);
        FETCH csr_otl_event_drag_no_rep_xml INTO l_xml;
        CLOSE csr_otl_event_drag_no_rep_xml;

      ELSIF (l_start_time is null AND 
             l_end_time   is null AND 
             self.start_date <> self.end_date) THEN  --month/week timeline drag operation

        OPEN  csr_otl_event_drag_rep_xml(l_otl_timecard_details.timecard_summary_start_time
                                        ,l_otl_timecard_details.timecard_summary_stop_time
                                        ,l_otl_timecard_details.timecard_summary_status
                                        ,l_otl_timecard_details.timecard_summary_id
                                        ,l_otl_timecard_details.timecard_summary_ovn
                                        ,l_otl_timecard_details.timecard_ovn
                                        ,l_otl_timecard_details.timecard_date_from
                                        ,trunc(l_otl_timecard_details.timecard_start_time)
                                        ,trunc(l_otl_timecard_details.timecard_stop_time));
        FETCH csr_otl_event_drag_rep_xml INTO l_xml;
        CLOSE csr_otl_event_drag_rep_xml;

      ELSIF (l_start_time is not null AND 
             l_end_time   is not null AND 
             self.start_date <> self.end_date) THEN  --month/week timeline drag operation when times are present on timecard entry

        OPEN  csr_otl_event_drag_rep_xml(l_otl_timecard_details.timecard_summary_start_time
                                        ,l_otl_timecard_details.timecard_summary_stop_time
                                        ,l_otl_timecard_details.timecard_summary_status
                                        ,l_otl_timecard_details.timecard_summary_id
                                        ,l_otl_timecard_details.timecard_summary_ovn
                                        ,l_otl_timecard_details.timecard_ovn
                                        ,l_otl_timecard_details.timecard_date_from
                                        ,trunc(l_otl_timecard_details.timecard_start_time)
                                        ,trunc(l_otl_timecard_details.timecard_stop_time));
        FETCH csr_otl_event_drag_rep_xml INTO l_xml;
        CLOSE csr_otl_event_drag_rep_xml;

      END IF;

    END IF;

    RETURN l_xml;

  END;

END;
/


CREATE OR REPLACE TYPE BODY xxas_cal_event_review IS

  CONSTRUCTOR FUNCTION xxas_cal_event_review
    (p_person_id           IN  number
    ,p_resource_id         IN  number
    ,p_abs_attendance_id   IN  number
    ,p_log_entry_id        IN  number
    ,p_transaction_id      IN  number
    ,p_transaction_step_id IN  number
    ,p_action_id           IN  varchar2
    ,p_source_block        IN  varchar2
    ,p_saved_search_id     IN  number
    ,p_manager_id          IN number default null) RETURN SELF AS RESULT
  IS

  BEGIN

    self.person_id           :=  p_person_id;
    self.resource_id         :=  p_resource_id;
    self.abs_attendance_id   :=  p_abs_attendance_id;
    self.log_entry_id        :=  p_log_entry_id;
    self.transaction_id      :=  p_transaction_id;
    self.transaction_step_id :=  p_transaction_step_id;
    self.action_id           :=  p_action_id;
    self.source_block        :=  p_source_block;
    self.saved_search_id     :=  p_saved_search_id;
    self.manager_id          :=  p_manager_id;

    RETURN;

  END;

  MEMBER FUNCTION get_xml RETURN XMLType
  IS

    l_xml XMLType;

    CURSOR csr_abs_event_review_xml IS
    SELECT XMLElement
            ("ROW"
            ,XMLForest
              (trunc(sysdate) effective_date
              ,self.person_id person_id
              ,self.abs_attendance_id absence_attendance_id
              ,self.transaction_id transaction_id
              ,self.transaction_step_id transaction_step_id
              ,self.transaction_id approvers
              ,self.source_block source_block
              ,self.saved_search_id saved_search_id
              ,self.manager_id manager_id
              )
            )
    FROM   dual;

  BEGIN

    IF self.person_id           is not null AND 
       self.abs_attendance_id   is not null AND
       self.transaction_id      is not null AND
       self.transaction_step_id is not null AND 
       self.action_id = 'ABSENCE' THEN

      OPEN  csr_abs_event_review_xml;
      FETCH csr_abs_event_review_xml INTO l_xml;
      CLOSE csr_abs_event_review_xml;

    END IF;

    RETURN l_xml;

  END;

END;
/


CREATE OR REPLACE TYPE BODY xxas_copy_time IS

  CONSTRUCTOR FUNCTION xxas_copy_time
    (p_period_id           IN  number
    ,p_person_id           IN  number
    ,p_assignment_id       IN  number
    ,p_mgr_person_id       IN  number
    ,p_source_block        IN  varchar2
    ,p_saved_search_id     IN  number) RETURN SELF AS RESULT
  IS

  BEGIN

    self.period_id           :=  p_period_id;
    self.person_id           :=  p_person_id;
    self.assignment_id       :=  p_assignment_id;
    self.mgr_person_id       :=  p_mgr_person_id;
    self.source_block        :=  p_source_block;
    self.saved_search_id     :=  p_saved_search_id;

    RETURN;

  END;

  MEMBER FUNCTION get_xml RETURN XMLType
  IS

    l_xml                    XMLType;
    l_emp_calendar_flag      varchar2(1);
    l_mgr_calendar_flag      varchar2(1);
    l_person_id              number;
    l_from_person_id         number;
    l_to_person_id           number;
    l_called_view            varchar2(30);
    l_from_period_id         number;
    l_to_period_id           number;
    l_entries_count          number;
    l_emp_cv_from_person_id  number;
    l_emp_cv_to_person_id    number;

    CURSOR csr_copy_time_xml IS
    SELECT XMLElement
            ("ROW"
            ,XMLForest
              (self.period_id period_id
              ,self.person_id person_id
              ,self.assignment_id assignment_id
              ,self.mgr_person_id mgr_person_id
              ,self.source_block source_block
              ,self.saved_search_id saved_search_id
              ,l_called_view called_view
              ,l_from_person_id from_person_id
              ,l_to_person_id   to_person_id
              ,l_from_period_id from_period_id
              ,l_to_period_id   to_period_id
              ,l_emp_cv_from_person_id emp_cv_from_person_id
              ,l_emp_cv_to_person_id emp_cv_to_person_id
              )
            )
    FROM   dual;

    CURSOR csr_log_entries_count IS
    SELECT count(*)
    FROM   xxas_tcm_log_entries_v xtle
    WHERE  xtle.period_id = self.period_id;

  BEGIN

    --
    -- When coming from the employee view, the 'From' and 'To' people should default to the employee and be hidden
    --
    IF self.source_block is not null THEN

      l_emp_calendar_flag := CASE WHEN instr(xxas_com_sec_pkg.get_function_parameter
                                               (p_function_name  => replace(self.source_block,'$','_')
                                               ,p_parameter_name => 'pRestProvider'),'for-individual') <> 0
                                  THEN 'Y'
                                  ELSE 'N'
                             END;

      l_mgr_calendar_flag := CASE WHEN instr(xxas_com_sec_pkg.get_function_parameter
                                               (p_function_name  => replace(self.source_block,'$','_')
                                               ,p_parameter_name => 'pRestProvider'),'for-team') <> 0
                                  THEN 'Y'
                                  ELSE 'N'
                             END;


      IF l_emp_calendar_flag = 'Y' THEN

        l_called_view    := 'EMP';

        IF self.person_id is not null THEN

          l_from_person_id        := self.person_id;
          l_to_person_id          := self.person_id;
          l_emp_cv_from_person_id := self.person_id;
          l_emp_cv_to_person_id   := self.person_id;

        ELSIF self.assignment_id is not null THEN

          l_person_id := xxas_util_pkg.get_person_from_asg(self.assignment_id,trunc(sysdate));

          l_from_person_id        := l_person_id;
          l_to_person_id          := l_person_id;
          l_emp_cv_from_person_id := l_person_id;
          l_emp_cv_to_person_id   := l_person_id;

        END IF;

      ELSIF l_mgr_calendar_flag = 'Y' THEN

        IF self.saved_search_id is null THEN

          l_called_view := 'TEAM_DR';

        ELSE

          l_called_view := 'TEAM_SS';

        END IF;

      END IF;

    END IF;

    IF self.period_id is not null THEN

      OPEN  csr_log_entries_count;
      FETCH csr_log_entries_count INTO l_entries_count;
      CLOSE csr_log_entries_count;

      --
      -- If that period has no log entries, automatically set it as the 'To' period.
      --
      IF l_entries_count = 0 THEN

        l_to_period_id := self.period_id;

      --
      -- If it has entries, automatically set it as the 'From' period.
      --
      ELSE

        l_from_period_id := self.period_id;

      END IF;

    END IF;

    OPEN  csr_copy_time_xml;
    FETCH csr_copy_time_xml INTO l_xml;
    CLOSE csr_copy_time_xml;

    RETURN l_xml;

  END;

END;
/


CREATE OR REPLACE TYPE BODY xxas_cal_event_weekly_edit IS

  CONSTRUCTOR FUNCTION xxas_cal_event_weekly_edit
    (p_person_id             IN  number
    ,p_start_time            IN  varchar2
    ,p_end_time              IN  varchar2
    ,p_payment_type_id       IN  number
    ,p_action_id             IN  varchar2
    ,p_source_block          IN  varchar2
    ,p_saved_search_id       IN  number
    ,p_exclude_terminated_by IN  varchar2) RETURN SELF AS RESULT
  IS

  BEGIN

    self.person_id             :=  p_person_id;
    self.start_time            :=  p_start_time;
    self.end_time              :=  p_end_time;
    self.payment_type_id       :=  p_payment_type_id;
    self.action_id             :=  p_action_id;
    self.source_block          :=  p_source_block;
    self.saved_search_id       :=  p_saved_search_id;
    self.exclude_terminated_by :=  p_exclude_terminated_by;

    RETURN;

  END;

  MEMBER FUNCTION get_xml RETURN XMLType
  IS

    CURSOR csr_tcm_weekly_edit_event_xml(cp_period_id     IN number
                                        ,cp_called_view   IN varchar2
                                        ,cp_assignment_id IN number) IS
    SELECT XMLElement
            ("ROW"
            ,XMLForest
              (self.person_id as person_id
              ,self.start_time as start_time
              ,self.end_time as end_time
              ,self.source_block as source_block
              ,self.saved_search_id as saved_search_id
              ,cp_called_view as called_view
              ,cp_assignment_id as assignment_id
              ,trunc(fnd_date.canonical_to_date(self.start_time))  as date_start
              ,cp_period_id as period_id
              ,cp_period_id as prev_period_id
              ,self.payment_type_id as payment_type_id
              ,self.payment_type_id as prev_payment_type_id
              ,self.exclude_terminated_by as exclude_term_by
              ,'TCM' as prod
              )
            )
    FROM   dual;

    CURSOR csr_otl_weekly_edit_event_xml(cp_period_id     IN number
                                        ,cp_called_view   IN varchar2
                                        ,cp_assignment_id IN number) IS
    SELECT XMLElement
            ("ROW"
            ,XMLForest
              (self.person_id as person_id
              ,self.start_time as start_time
              ,self.end_time as end_time
              ,self.source_block as source_block
              ,self.saved_search_id as saved_search_id
              ,cp_called_view as called_view
              ,cp_assignment_id as assignment_id
              ,trunc(fnd_date.canonical_to_date(self.start_time))  as date_start
              ,cp_period_id as prev_period_id
              ,self.payment_type_id as payment_type_id
              ,self.payment_type_id as prev_payment_type_id
              ,self.exclude_terminated_by as exclude_term_by
              ,'OTL' as prod
              ,trunc(fnd_date.canonical_to_date(self.start_time)) as period_date
              )
            )
    FROM   dual;

    CURSOR csr_tcm_period_id IS
    SELECT xtp.period_id
    FROM   xxas_tcm_periods xtp
    WHERE  trunc(fnd_date.canonical_to_date(self.start_time)) BETWEEN xtp.start_date
                                                              AND     xtp.end_date - 1;

    l_xml                  XMLType;
    l_period_id            xxas_tcm_periods.period_id%TYPE;
    l_called_view          varchar2(30);
    l_assignment_id        number;

  BEGIN

    IF self.action_id = 'TIMECARD' THEN

      OPEN  csr_tcm_period_id;
      FETCH csr_tcm_period_id INTO l_period_id;
      CLOSE csr_tcm_period_id;

      l_called_view := xxas_cal_util_pkg.source_block_called_view(self.source_block);
      l_assignment_id := xxas_util_pkg.get_primary_worker_asg(self.person_id,trunc(sysdate));

      OPEN  csr_tcm_weekly_edit_event_xml(l_period_id
                                         ,l_called_view
                                         ,l_assignment_id);
      FETCH csr_tcm_weekly_edit_event_xml INTO l_xml;
      CLOSE csr_tcm_weekly_edit_event_xml;

    ELSIF self.action_id = 'OTL' THEN

      l_called_view := xxas_cal_util_pkg.source_block_called_view(self.source_block);
      l_assignment_id := xxas_util_pkg.get_primary_worker_asg(self.person_id,trunc(sysdate));

      l_period_id := xxas_cal_util_pkg.otl_period_id_between_dates
                       (p_person_id  => self.person_id
                       ,p_start_date => trunc(fnd_date.canonical_to_date(self.start_time))
                       ,p_end_date   => trunc(fnd_date.canonical_to_date(self.end_time)));

      OPEN  csr_otl_weekly_edit_event_xml(l_period_id
                                         ,l_called_view
                                         ,l_assignment_id);
      FETCH csr_otl_weekly_edit_event_xml INTO l_xml;
      CLOSE csr_otl_weekly_edit_event_xml;

    END IF;

    RETURN l_xml;

  END;

END;
/

COMMIT;
EXIT;